//**************************************************
// Copyright (c) 2013-2016 Cisco Systems, Inc.
// All rights reserved.
//**************************************************

package com.cisco.ise.ers.demo;


import java.io.IOException;
import java.util.Iterator;
import java.util.List;
import javax.servlet.http.HttpServletResponse;
import javax.xml.bind.JAXBElement;
import org.apache.http.HttpResponse;
import org.apache.http.util.EntityUtils;
import com.cisco.ise.ers.BaseResource;
import com.cisco.ise.ers.v2.SearchResult;
import com.cisco.ise.ers.UpdatedField;
import com.cisco.ise.ers.UpdatedFieldsList;
import com.cisco.ise.ers.identity.AdminUser;
import com.cisco.ise.ers.identity.AdminUser.CustomAttributes;
import com.cisco.ise.ers.identity.ObjectFactory;


/**
 * AdminUserDemo
 *
 * This is as example of client code that uses ISE Adminuser Rest APIs to
 * configure adminusers. The only ISE jars that it depends upon is the ers-schema jar.
 * 
 * 
 * This example runs the following Rest Interface calls: 
 * 1. Get all adminusers
 * 2. Get Admin user by Id 
 * 3. Get Admin user by name
 
 * 
 */
public class AdminUserDemo {

	
	
	static ERSClient ersClient;
	static ObjectFactory identityFactory = new ObjectFactory();
	static JAXBHelper jaxbh = new JAXBHelper(identityFactory.getClass().getPackage().getName());
	private static  String resourceId = "";
	private static String resourceName = "";
	
	/**
	 * expect 3 arguments: serverHostIp, Admin username of type ERS Admin, and its password
	 * Then it runs 
	 * @param args
	 * @throws Exception
	 */
	public static void main(String[] args) throws Exception {
	
		String serverHost = null;
		String userName = null;
		String password = null;
		List<BaseResource> baseResources = null;

		if (args != null && args.length == 3) {
			
			serverHost = args[0]; // Server Host
			userName = args[1]; // Administrator name
			password = args[2]; // Administrator password
			
			System.out.println("\n ISE ip:" + serverHost + " user:" + userName + " psw:" + password);
			
			// Get an instance of Rest Client
			ERSClient ersClient = new ERSClient(userName, password, serverHost);
			
			System.out.println("\n\n\n\n\n\n" + "--------------------------------- START --------------------------------------------------");
			
			// Connectivity Check
			connectivityCheck(ersClient);

			
			// demo approve
			DemoAppHelpers.demoApprove();
			
			// Get all users
			System.out.println("\n\n\n\n\n\n" + "----------- Get all Admin Users ----------------------------------------------------------------");
			System.out.println("\n###############################################################" + 
			           		   "\n#  Get All Admin Users:                                    #" +
			           		   "\n#  This demo sends GET request to server to get all(paged)    #" +
			           		   "\n#  Internal users. The expected response would be status 200  #" +
			           		   "\n#  with Search Result as the reponse content containing the   #" +
			           		   "\n#  first page of the internal users. The demo will also       #" +
			           		   "\n#  marshal the xml response content to java object of type    #" +
			           		   "\n#  search result and will print its attributes.               #" +
			           		   "\n###############################################################");
	
			baseResources = getAllUsers(ersClient, true, "?page=1&size=50");
			
			
			System.out.println("\n\n\n\n\n\n" + "----------- get specify adminuser by id -------------------------------------------------------");
			System.out.println("\n###############################################################" + 
			           		   "\n#  Get user by id:                                            #" +
			           		   "\n#  This demo sends GET request to fetch a specified user by   #" +
			           		   "\n#  its id. The expected response would be status 200 whereas  #" +
			           		   "\n#  the response content will hold the requested internal user.#" +
			           		   "\n#  The demo will also unmarshal the xml response content to   #" +
			           		   "\n#  java object of type internal user and will print           #" +
			           		   "\n#  its attributes.                                            #" +
			           		   "\n###############################################################");
			           		   
			getUserByID(ersClient,resourceId);
			
			
			//search user by name
			System.out.println("\n\n\n\n\n\n" + "----------- search user by name ----------------------------------------------------------");
			System.out.println("\n###############################################################" + 
			           		   "\n#  Get user by name:                                          #" +
			           		   "\n#  This demo sends GET request to server to get all(paged)    #" +
			           		   "\n#  Admin users filtered by name.                           #" +
			           		   "\n#  The expected response would be status 200                  #" +
			           		   "\n#  with Search Result as the reponse content, containing the  #" +
			           		   "\n#  filtered users with name=last Admin Name configured in ISE.#" +
			           		   "\n#  Then it marshals the xml response content to java object of#" + 
			           		   "\n#  type SearchResult, and will print its attributes.          #" +
			           		   "\n###############################################################");
			searchUserByName(ersClient,resourceName);
			
			
		}else{
			System.out.println("\n illegal argument. \n syntax: \n AdminUserDemo.sh [host-ip] [user-name] [password]\n");
		}
	}
	
	
	
	

	/**
	 * connectivityCheck
	 * send getVersion request to server in order to try connect
	 * and make sure the credentials are OK
	 * @param ersClient
	 */
	private static void connectivityCheck(ERSClient ersClient) {
		HttpResponse response = null;
		try {
			
			System.out.println("\n###############################################################" + 
	           		   		   "\n#  Connectivity Check:                                        #" +
	           		   		   "\n#  About to send GET AdminUser version request to server   #" +
	           		   		   "\n#  in order to make sure that connection can be established.  #" +
	           		   		   "\n###############################################################");
			
			//getting internal user server version, just for checking that the  
			response = ersClient.getVersion("identity","adminuser");
			String responseAsString = RestUtils.getResponseContentAsString(response);
			int responseStatus = RestUtils.getResponseStatus(response);
			if (responseStatus == HttpServletResponse.SC_OK) {
				System.out.println("\nConnection to ISE server success");
			}else if(responseStatus == HttpServletResponse.SC_UNAUTHORIZED){
				System.out.println("\n###############################################################" + 
						           "\n#  401 - UNAUTHORIZED:                                        #" +
						           "\n#  Connection to ISE server failed !!!                        #" +
						           "\n#  Make sure ERS is enabled (CLI: application configure ise). #" +
						           "\n#  Make sure you are using  valid ERS Admin Credentials.      #" +
						           "\n#  Please note that the admin must have 'ERS-Admin' privileges#" +
						           "\n###############################################################");				
				System.exit(-1);
			}else{
				System.out.println("\nRESPONSE CONTENT: \n" + "	" +  responseAsString);
			}
		} catch (Exception e) {
			e.printStackTrace();
		} finally{
			//release the connection
			if(response != null){
				try {
					EntityUtils.consume(response.getEntity());
				} catch (IOException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
			}
		}
	}



	private static AdminUser getUserByID(ERSClient ersClient, String userId) {
		
		HttpResponse response = null;
		AdminUser adminUser = null;
		
		try {
			
			response = ersClient.getById("identity","adminuser",userId);
			RestUtils.printResponseHeaders(response);
			String responseAsString = RestUtils.getResponseContentAsString(response);
			System.out.println("\nRESPONSE CONTENT: \n" +  "	" + responseAsString);
		
			
			
			if (RestUtils.getResponseStatus(response) == HttpServletResponse.SC_OK) {
					
				//unmarshal the result from xml to java object
				adminUser = (AdminUser)RestUtils.fromXML(responseAsString, AdminUser.class);
				
				if (adminUser!=null){
					System.out.println("\n user " + adminUser.getName() + " fetched!");
					System.out.println("\n user " + adminUser.getName() + " email is: " + adminUser.getEmail());
				}
				
			}

		} catch (Exception e) {
			e.printStackTrace();
		} finally{
			//release the connection
			if(response != null){
				try {
					EntityUtils.consume(response.getEntity());
				} catch (IOException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
			}
		}
		
		return adminUser;
		
	}






	/**
	 * getAllUsers
	 * send to server GET request and retrieve all user currently in server (paged).
	 * @param ersClient
	 */
	private static List<BaseResource> getAllUsers(ERSClient ersClient, boolean verboseOn, String queryStringParams) {
		
		HttpResponse response = null;
		
		List<BaseResource> baseResources = null;  
		
		try {
		
			response = ersClient.getAll("identity","adminuser", queryStringParams);
			
			
			if(verboseOn){
				RestUtils.printResponseHeaders(response);
			}
			
			
			String searchResultAsString = RestUtils.getResponseContentAsString(response);
			
			if (RestUtils.getResponseStatus(response) == HttpServletResponse.SC_OK) {
				
				
				
				if(verboseOn){
					System.out.println("\nRESPONSE CONTENT: \n" + "	" +  searchResultAsString);
				}
				
					
				//unmarshal the result from xml to java object
				SearchResult sr = (SearchResult)RestUtils.fromXML(searchResultAsString, SearchResult.class);
				
				
				System.out.println("\n Total Admin Users found: " + sr.getTotal());
				
				baseResources =  sr.getResources().getResource();
				
				Iterator<BaseResource> it = baseResources.iterator();
				while (it.hasNext()) {
					BaseResource br = (BaseResource) it.next();
					if(verboseOn){
						resourceId = br.getId();
						resourceName = br.getName();
						System.out.println("\n	user (base resource) details: \n id=" + br.getId() + " \n name=" + br.getName() +
								" \n description=" + br.getDescription() + " \n link=" + br.getLink());
					}
				}
				
				
			}else{
				System.out.println("\nRESPONSE CONTENT: \n" + "	" +  searchResultAsString);
			}

		} catch (Exception e) {
			System.out.println("get-all failed: " + e.getMessage());;
		} finally{
			//release the connection
			if(response != null){
				try {
					EntityUtils.consume(response.getEntity());
				} catch (IOException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
			}
		}
		
		return baseResources;
	}

private static AdminUser searchUserByName(ERSClient ersClient, String name) {
		
		AdminUser adminUser = null;
		List<BaseResource> baseResources  = getAllUsers(ersClient, false, "?filter=name.EQ." + name);
		
		if(baseResources != null && baseResources.size() == 1){
			adminUser = getUserByID(ersClient, baseResources.get(0).getId()); 
		}
		return adminUser;
	}
	
	
}
