//**************************************************
//Copyright (c) 2015 Cisco Systems, Inc.
//All rights reserved.
//**************************************************

package com.cisco.ise.ers.demo;

import java.io.IOException;
import java.util.Iterator;
import java.util.List;

import javax.servlet.http.HttpServletResponse;
import javax.xml.bind.JAXBElement;

import org.apache.http.HttpResponse;

import com.cisco.ise.ers.BaseResource;
import com.cisco.ise.ers.UpdatedField;
import com.cisco.ise.ers.UpdatedFieldsList;
import com.cisco.ise.ers.anc.ErsAncAction;
import com.cisco.ise.ers.anc.ErsAncPolicy;
import com.cisco.ise.ers.anc.ErsAncPolicy.Actions;
import com.cisco.ise.ers.anc.ObjectFactory;
import com.cisco.ise.ers.v2.SearchResult;

public class AncPolicyDemo {

	static ERSClient ersClient;

	static ObjectFactory objectFactory = new ObjectFactory();

	static JAXBHelper jaxbh = new JAXBHelper(objectFactory.getClass().getPackage().getName());

	/**
	 * expect 3 arguments: serverHostIp, Admin username of type ERS Admin, and
	 * its password String serverHost = null; Then it runs
	 * 
	 * @param args
	 * @throws Exception
	 */
	public static void main(String[] args) throws Exception {

		String serverHost = null;
		String userName = null;
		String password = null;

		if (args != null && args.length == 3) {

			serverHost = args[0]; // Server Host
			userName = args[1]; // Administrator name
			password = args[2]; // Administrator password
			
			
			System.out.println("\n ISE ip:" + serverHost + " user:" + userName	+ " psw:" + password);

			// Get an instance of Rest Client
			ERSClient ersClient = new ERSClient(userName, password, serverHost);

			System.out.println("\n\n\n\n\n\n--------------------------------- START --------------------------------------------------");

			// Connectivity Check
			connectivityCheck(ersClient);

			// create
			ErsAncPolicy policy = new ErsAncPolicy();
			policy.setName("DEMO_policy");
			Actions actions = new Actions();
			List<ErsAncAction> actionList = actions.getAction();
			actionList.add(ErsAncAction.QUARANTINE);
			policy.setActions(actions);
			
			System.out.println("\n\n\n\n\n\n----------- Create ANC policy ---------------------------------------------------------------");
			System.out.println("\n###############################################################"
							+ "\n#  Create ANC policy:                                          #"
							+ "\n#  This demo sends POST request to create new ANC policy       #"
							+ "\n#  The expected response would be status 201 whereas the       #"
							+ "\n#  LOCATION header will hold the newborn ANC policy ID.        #"
							+ "\n###############################################################");
			String id = create(ersClient, policy);
			
			// Get all 
			System.out.println("\n\n\n\n\n\n" + "----------- Get all ANC policies --------------------------------------------------------------");
			System.out.println("\n###############################################################" + 
			           		   "\n#  Get All ANC policies:                                      #" +
					 		   "\n#  This demo sends GET request to server to get all(paged)    #" +
			         		   "\n#  ANC policies. The expected response would be status 200    #" +				           		   
						       "\n#  first page of the ANC policy. The demo will also           #" +
			         		   "\n#  marshal the xml response content to java object of type    #" +
		            		   "\n#  search result and will print its attributes.               #" +
			           		   "\n###############################################################");
				
			getAll(ersClient, true, "?page=1&size=20");
					    
		   // Get by ID
		    System.out.println("\n\n\n\n\n\n" + "----------- Get ANC policy by ID -------------------------------------------------------------");
		    System.out.println("\n###############################################################" + 
		                       "\n#  Get ANC policy by ID:                                      #" +
		            		   "\n#  This demo sends GET request to fetch ANC policy.           #" +
		            		   "\n#  The expected response would be status 200 whereas the      #" +
		           		       "\n#  response content will hold the requested Network Device.   #" +
		           		       "\n#  The demo will also unmarshal the xml response content to   #" +
		           		       "\n#  java object of type ANC policy and will print              #" +
		           		       "\n#  its attributes.                                            #" +
		           		       "\n###############################################################");
		    getByID(ersClient, id);
		   
		    // Update
		    System.out.println("\n\n\n\n\n\n" + "----------- Update ANC policy ----------------------------------------------------------------");
		    System.out.println("\n###############################################################" + 
		             		   "\n#  Update ANC policy:                                         #" +
		            		   "\n#  This demo sends PUT request to server,                     #" +
		            		   "\n#  for updating a ANC policy.                                 #" +
		            		   "\n#  The java ANC policy object is marshaled to xml to          #" +
		            		   "\n#  become the request content.                                #" +
		            		   "\n#  The expected response would be status 200                  #" +
		            		   "\n#  with UpdatedFields as the reponse content which will be    #" +
		           	    	   "\n#  Then it marshals to java object of type UpdatedFields, and #" +
		           		       "\n#  print it to console .                                      #" +
		           		       "\n###############################################################");
		    update(ersClient, id);
			
		    // Delete by ID
		    System.out.println("\n\n\n\n\n\n" + "----------- Delete ANC policy by ID -----------------------------------------------------------");
		    System.out.println("\n#######################################################################" + 
		           	   	       "\n#  Delete ANC policy by ID:                                           #" +
		           		       "\n#  This demo sends DELETE request to delete the Sxp Connection by ID  #" +
		           		       "\n#  The expected response would be status 204 (no content)             #" +
		           		       "\n#######################################################################");
		    deleteByID(ersClient, id);
					   
		    System.out.println("\n\n\n\n\n\n--------------------------------- END ----------------------------------------------------");

		} else {
			System.out.println("\n illegal argument. \n syntax: \n [host-ip] [user-name] [password]\n");
		}

	}

	/**
	 * connectivity check send getVersion request to server in order to try
	 * connect and make sure the credentials are OK
	 * 
	 * @param ersClient
	 */
	private static void connectivityCheck(ERSClient ersClient) {

		HttpResponse response = null;
		try {

			System.out.println("\n###############################################################"
							+ "\n#  Connectivity Check:                                        #"
							+ "\n#  About to send GET Sxp Connection version request to server #"
							+ "\n#  in order to make sure that connection can be established.  #"
							+ "\n###############################################################");

			// getting SxpPeer server version, just for checking that the
			response = ersClient.getVersion("sxp", "sxpconnections");
			String responseAsString = RestUtils.getResponseContentAsString(response);
			int responseStatus = RestUtils.getResponseStatus(response);
			if (responseStatus == HttpServletResponse.SC_OK) {
				System.out.println("\nConnection to ISE server success");
			} else if (responseStatus == HttpServletResponse.SC_UNAUTHORIZED) {
				System.out.println("\n###############################################################"
								+ "\n#  401 - UNAUTHORIZED:                                        #"
								+ "\n#  Connection to ISE server failed !!!                        #"
								+ "\n#  Make sure you are using  valid Admin Credentials.          #"
								+ "\n#  Please note that the admin must have 'ERS-Admin' privilages#"
								+ "\n###############################################################");
				System.exit(-1);
			} else {
				System.out.println("\nRESPONSE CONTENT: \n" + "	"+ responseAsString);
			}
		} catch (Exception e) {
			e.printStackTrace();
		} finally {
			// release the connection
			if (response != null) {
				try {
					if (response.getEntity() != null) {
						response.getEntity().consumeContent();
					}
				} catch (IOException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
			}
		}
	}
	
	
	private static String create(ERSClient ersClient, ErsAncPolicy policy) {

		System.out.println("\n\n *** about to create new ANC policy...");

		String newBornId = null;
		HttpResponse response = null;

		try {

			JAXBElement<ErsAncPolicy> jaxbPolicy = objectFactory.createAncpolicy(policy);

			String policyAsXml = jaxbh.marshal(jaxbPolicy);

			System.out.println("Request Xml...");
			System.out.println(policyAsXml);
			System.out.println("");
			response = ersClient.create("anc", "ancpolicy", policyAsXml);

			RestUtils.printResponseHeaders(response);

			if (RestUtils.getResponseStatus(response) == HttpServletResponse.SC_CREATED) {

				newBornId = RestUtils.getNewBornId(response);

			} else {
				// error -> print response content

				System.out.println("\nRESPONSE CONTENT: \n" + " " + RestUtils.getResponseContentAsString(response));

			}

		} catch (Exception e) {
			e.printStackTrace();
		} finally {
			// release the connection
			if (response != null) {
				try {
					if (response.getEntity() != null) {
						response.getEntity().consumeContent();
					}
				} catch (IOException e) {
					e.printStackTrace();
				}
			}

		}

		return newBornId;
	}
	
	private static List<BaseResource> getAll(ERSClient ersClient, boolean verboseOn, String queryStringParams) {

		HttpResponse response = null;

		List<BaseResource> baseResources = null;  

		try {

			response = ersClient.getAll("anc", "ancpolicy", queryStringParams);


			if(verboseOn){
				RestUtils.printResponseHeaders(response);
			}

			String searchResultAsString = RestUtils.getResponseContentAsString(response);

			if (RestUtils.getResponseStatus(response) == HttpServletResponse.SC_OK) {
				
				
				
				if(verboseOn){
					System.out.println("\nRESPONSE CONTENT: \n" + "	" +  searchResultAsString);
				}


				//unmarshal the result from xml to java object
				SearchResult sr = (SearchResult)RestUtils.fromXML(searchResultAsString, SearchResult.class);

				System.out.println("\n Total Sxp Connections found: " + sr.getTotal());

				baseResources =  sr.getResources().getResource();

				Iterator<BaseResource> it = baseResources.iterator();
				while (it.hasNext()) {
					BaseResource br = (BaseResource) it.next();
					if(verboseOn){
						System.out.println("\n	ANC policy details: \n Id =" + br.getId() + " \n name=" + br.getName() +
								" \n description=" + br.getDescription() + " \n link=" + br.getLink());
					}

				}

			}else{
				System.out.println("\nRESPONSE CONTENT: \n" + "	" +  searchResultAsString);
			}

		} catch (Exception e) {
			e.printStackTrace();
		} finally{
			//release the connection
			if(response != null){
				try {
					if (response.getEntity() != null) {
						response.getEntity().consumeContent();
					}
				} catch (IOException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
			}
		}

		return baseResources;
	}
	
	private static ErsAncPolicy getByID(ERSClient ersClient, String id) {

		HttpResponse response = null;
		ErsAncPolicy policy = null;

		try {
			System.out.println("Get by Id : " + id);
			response = ersClient.getById("anc", "ancpolicy", id);
			RestUtils.printResponseHeaders(response);
			String responseAsString = RestUtils.getResponseContentAsString(response);
			System.out.println("\nRESPONSE CONTENT: \n" +  "	" + responseAsString);


			if (RestUtils.getResponseStatus(response) == HttpServletResponse.SC_OK) {

				//unmarshal the result from xml to java object
				policy = (ErsAncPolicy)RestUtils.fromXML(responseAsString, ErsAncPolicy.class);

				if (policy!=null){
					System.out.println("\n Retrived ANC policy, Id " + policy.getId() + " , Name : " + policy.getName());
				}
			}
		} catch (Exception e) {
			e.printStackTrace();
		} finally{
			//release the connection
			if(response != null){
				try {
					response.getEntity().consumeContent();
				} catch (IOException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
			}
		}

		return policy;
	}
	
	private static void deleteByID(ERSClient ersClient, String id) {
		HttpResponse response = null;

		try {

			response = ersClient.deleteById("anc", "ancpolicy", id);
			RestUtils.printResponseHeaders(response);
			String responseAsString = RestUtils.getResponseContentAsString(response);


			System.out.println("\nRESPONSE CONTENT: \n" + "	" +  responseAsString);


			if (RestUtils.getResponseStatus(response) == HttpServletResponse.SC_NO_CONTENT) {

				System.out.println("ANC policy ID " + id + " deleted" );

			}else{
				System.out.println("Failed to delete ID: " + id);
			}

		} catch (Exception e) {
			e.printStackTrace();
		} finally{
			//release the connection
			if(response.getEntity() != null){
				try {
					response.getEntity().consumeContent();
				} catch (IOException e) {
					// TODO Auto-generated catch block
					//e.printStackTrace();
				}
			}
		}
	}
	
	private static void update(ERSClient ersClient, String id) {

		HttpResponse response = null;
		ErsAncPolicy policy = new ErsAncPolicy();

		try {
			policy = getByID(ersClient, id);
			if (policy != null) {
				Actions actions = new Actions();
				List<ErsAncAction> list = actions.getAction();
				list.add(ErsAncAction.SHUTDOWN);
				policy.setActions(actions);
				
				JAXBElement<ErsAncPolicy> jaxbPolicy = objectFactory.createAncpolicy(policy);

				String policyAsXml = jaxbh.marshal(jaxbPolicy);

				response = ersClient.update("anc", "ancpolicy", id, policyAsXml);

				RestUtils.printResponseHeaders(response);
				String responseAsString = RestUtils.getResponseContentAsString(response);
				System.out.println("\nRESPONSE CONTENT: \n" + "	" + responseAsString);

				if (RestUtils.getResponseStatus(response) == HttpServletResponse.SC_OK) {


					UpdatedFieldsList updatedFields = (UpdatedFieldsList)RestUtils.fromXML(responseAsString, UpdatedFieldsList.class);
					List<UpdatedField> uflds = updatedFields.getUpdatedField();

					for(UpdatedField uf : uflds){

						System.out.println(uf.getField() + " field was updated: oldVal=" + uf.getOldValue() + " newVal=" + uf.getNewValue() + "\n");
					}
				}
			}

		} catch (Exception e) {
			e.printStackTrace();
		} finally{
			//release the connection
			if(response != null){
				try {
					response.getEntity().consumeContent();
				} catch (IOException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
			}
		}
	}
}
