//**************************************************
// Copyright (c) 2015 Cisco Systems, Inc.
// All rights reserved.
//**************************************************

package com.cisco.ise.ers.demo;


import java.io.IOException;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import javax.servlet.http.HttpServletResponse;
import javax.xml.bind.JAXBElement;

import org.apache.http.HttpResponse;

import com.cisco.ise.ers.AbstractBulkRequest;
import com.cisco.ise.ers.BaseResource;
import com.cisco.ise.ers.AbstractBulkRequest.IdList;
import com.cisco.ise.ers.BulkResourceStatus;

import com.cisco.ise.ers.BulkStatus;
import com.cisco.ise.ers.ExecutionStatus;
import com.cisco.ise.ers.ObjectFactory;
import com.cisco.ise.ers.ResourceExecutionStatus;
import com.cisco.ise.ers.v2.SearchResult;
import com.cisco.ise.ers.identity.Endpoint;
import com.cisco.ise.ers.identity.EndpointBulkRequest;


/**
 * Bulk Endpoint Demo
 *
 * This is as example of client code that uses ISE Bulk Rest APIs to
 * configure endpoints.
 * The only ISE jars that it depends upon is the ers-schema jar.
 * 
 * 
 * This example runs the following Rest Interface calls:   
 * 
 * *) cleaning all bulk demo Endpoints from DB
 * 1) sending bulk create request to create 500 endpoints
 * 2) monitoring this request
 * 3) updating some endpoints
 * 4) monitoring this request
 * 5) performing register endpoints costume operation
 * 6) monitoring this request
 * 7) collecting all created IDs and deleting it
 * 
 * 
 * @author anoy
 * @since 1.3
 * 
 */
@SuppressWarnings({"PMD.SystemPrintln"})
public class BulkDemo {

	
	
	private static final String EP_DSC_ATTR = "ep_bulk_demo";

	static ERSClient ersClient;
	
	static ObjectFactory ersFactory = new ObjectFactory();
	
	
	static JAXBHelper jaxbh = new JAXBHelper(ersFactory.getClass().getPackage().getName());
	
	/**
	 * expect 3 arguments: serverHostIp, Admin username of type ERS Admin, and its password
	 * Then it runs 
	 * @param args
	 * @throws InterruptedException 
	 * @throws Exception
	 */
	public static void main(final String[] args) throws InterruptedException{
	
		String serverHost = null;
		String userName = null;
		String password = null;
		
		if (args != null && args.length == 3) {
			
			serverHost = args[0]; 
			userName = args[1]; 
			password = args[2]; 
			
			System.out.println("\n ISE ip:" + serverHost + " user:" + userName + " psw:" + password);
			
			// Get an instance of Rest Client
			final ERSClient ersClient = new ERSClient(userName, password, serverHost);
			
			
			
			
			System.out.println("\n\n\n\n\n\n" + "--------------------------------- START --------------------------------------------------");
			
			
			connectivityCheck(ersClient);
				
			// demo user approve
			DemoAppHelpers.demoApprove();
			
			System.out.println("\n\n\n\n\n\n" + "----------- cleanup ------------------------------------------------------");
			
			
			
			
			System.out.println("\n\n\n\n\n\n" + "----------- Send Create EP Bulk ------------------------------------------------------");
			System.out.println("\n###############################################################" + 
			           		   "\n#  Send Create EP Bulk   :                                    #" +
			           		   "\n#  This demo sends Bulk PUT request to server to create       #" +
			           		   "\n#  500 End Points.                                            #" +
			           		   "\n#  The expected response would be status 201 whereas the      #" +
			           		   "\n#  LOCATION header will hold the newborn user id.             #" +
			           		   "\n###############################################################");
	
			final String createBulkId = sendCreateBulk(ersClient);
			
			
			System.out.println("\n\n\n\n\n\n" + "----------- Monitor the create bulk progress ----------------------------------------------");
			System.out.println("\n###############################################################" + 
			           		   "\n#  get the Create bulk status   :                             #" +
			           		   "\n#  This demo sends Bulk GET request to server to get          #" +
			           		   "\n#  the specified bulkID progress                              #" +
			           		   "\n###############################################################");
	
			final BulkStatus createBulkStatus = monitorBulkProgress(ersClient, createBulkId) ;
					
				
			
			Thread.sleep(1000*10);
			
			
			System.out.println("\n\n\n\n\n\n" + "----------- Fetch EndPoints ---------------------------------------------------------");
			System.out.println("\n###############################################################" + 
			           		   "\n#  Fetch Endpoints:                                           #" +
			           		   "\n#  This demo sends GET request to server to get all(paged)    #" +
			           		   "\n#  endpoints. The expected response would be status 200       #" +
			           		   "\n#  with Search Result as the reponse content, containing the  #" +
			           		   "\n#  first page of the endpoints. The demo will also            #" +
			           		   "\n#  marshal the xml response content to java object of type    #" +
			           		   "\n#  search result, and will print its attributes.              #" +
			           		   "\n###############################################################");
	
			searchEndPoints(ersClient, false, "?page=1&size=100");
		
			System.out.println("\n\n\n\n\n\n" + "----------- Send Delete EP Bulk ------------------------------------------------------");
			System.out.println("\n###############################################################" + 
			           		   "\n#  Send Delete EP Bulk   :                                    #" +
			           		   "\n#  This demo sends Bulk DELETE request to server to delete    #" +
			           		   "\n#  all 500 recently created End Points.                       #" +
			           		   "\n###############################################################");
	
			
			
			final String deleteBulkId = sendDeleteBulk(ersClient, getIdsFromStatus(createBulkStatus));
			
			
			monitorBulkProgress(ersClient, deleteBulkId) ;
				
			
				
			System.out.println("\n\n\n\n\n\n" + "--------------------------------- END ----------------------------------------------------");
			
		}else{
			System.out.println("\n illegal argument. \n syntax: \n BulkDemo.sh [host-ip] [user-name] [password]\n");
		}
	}
	
	/**
	 * sendRegisterBulk
	 * @param ersClient2
	 * @return
	 */
	private static String sendRegisterBulk(ERSClient ersClient) {
		
		
		System.out.println("\n\n *** about to run a new bulk: register 100 Endpoints");
		
		
		String bulkID = null;
		HttpResponse response = null;
		
		try {
			
			final EndpointBulkRequest bulkR = new EndpointBulkRequest();
			
			bulkR.setResourcesList(generateStubEPs(100));
			bulkR.setOperationType("register");
			bulkR.setResourceMediaType("vnd.com.cisco.ise.identity.endpoint.1.0+xml");
			
			final JAXBElement<AbstractBulkRequest> jaxbBulk =  ersFactory.createBulkRequest(bulkR);
			
			final String bulkAsXml = jaxbh.marshal(jaxbBulk);
			
			
			response = ersClient.executeBulk("identity", "endpoint", "endpointbulkrequest",  bulkAsXml);
			
			RestUtils.printResponseHeaders(response);
			
			if(RestUtils.getResponseStatus(response) == HttpServletResponse.SC_ACCEPTED){
				
				bulkID = RestUtils.getNewBornId(response);
				
			}else{
				
				System.out.println("\nRESPONSE CONTENT: \n" + "	" +  RestUtils.getResponseContentAsString(response));
				
			}
	
		} catch (Exception e) {
			System.out.println("\nException :" + e.getMessage());
		}finally{
			if(response != null){
				try {
					response.getEntity().consumeContent();
				} catch (IOException e) {
					System.out.println("\nException :" + e.getMessage());
				}
			}
		}
		
		
		System.out.println("\n Bulk started ! Bulk ID: " + bulkID);
		
		return bulkID;
		
	}

	/**
	 * sendDeleteBulk
	 * @param ersClient
	 * @param createBulkStatus
	 * @return
	 */
	private static String sendDeleteBulk(final ERSClient ersClient,
			final IdList ids) {
		System.out.println("\n\n *** about to run a new delete bulk request...");
		
		
		String bulkID = null;
		HttpResponse response = null;
		
		try {
			
			final EndpointBulkRequest bulkR = new EndpointBulkRequest();
			
			bulkR.setIdList(ids);
			bulkR.setOperationType("delete");
			bulkR.setResourceMediaType("vnd.com.cisco.ise.identity.endpoint.1.0+xml");
			
			final JAXBElement<AbstractBulkRequest> jaxbBulk =  ersFactory.createBulkRequest(bulkR);
			
			final String bulkAsXml = jaxbh.marshal(jaxbBulk);
			
			//System.out.println("\n" + bulkAsXml);
			
			
			
			response = ersClient.executeBulk("identity", "endpoint", "endpointbulkrequest",  bulkAsXml);
			
			RestUtils.printResponseHeaders(response);
			
			if(RestUtils.getResponseStatus(response) == HttpServletResponse.SC_ACCEPTED){
				
				bulkID = RestUtils.getNewBornId(response);
				
			}else{
				
				System.out.println("\nRESPONSE CONTENT: \n" + "	" +  RestUtils.getResponseContentAsString(response));
				
			}
	
		} catch (Exception e) {
			System.out.println("\nException :" + e.getMessage());
		}finally{
			if(response != null){
				try {
					response.getEntity().consumeContent();
				} catch (IOException e) {
					System.out.println("\nException :" + e.getMessage());
				}
			}
		}
		
		
		System.out.println("\n Bulk started ! Bulk ID: " + bulkID);
		
		return bulkID;		
	}


	/**
	 * getIdsFromStatus
	 * @param createBulkStatus
	 * @return
	 */
	private static IdList getIdsFromStatus(final BulkStatus createBulkStatus) {
		
		 final IdList ids = new IdList();
		 
		 if(createBulkStatus != null && createBulkStatus.getResourcesStatus() != null){
			 
			 for(BulkResourceStatus resourceStat :  createBulkStatus.getResourcesStatus().getResourceStatus()){
				 
				 if (resourceStat.getStatus() == ResourceExecutionStatus.SUCCESS){
					 ids.getId().add(resourceStat.getId());
				 }
			 }
		 }
		 return ids;
	}



	/**
	 * monitorBulkProgress
	 * @param bulkId
	 * @return
	 * @throws InterruptedException
	 */
	private static BulkStatus monitorBulkProgress(final ERSClient ersClient, final String bulkId) throws InterruptedException {
		
		BulkStatus bulkStatus = getBulkStatus(ersClient, bulkId);
		
		if(bulkStatus.getExecutionStatus() != ExecutionStatus.COMPLETED){
			while(bulkStatus.getExecutionStatus() != ExecutionStatus.COMPLETED){
				Thread.sleep(1000*5);
				bulkStatus = getBulkStatus(ersClient, bulkId);			
			}
		}	
		return bulkStatus;
	}



	/**
	 * getBulkStatus
	 * @param ersClient
	 * @param createBulkId
	 */
	private static BulkStatus getBulkStatus(final ERSClient ersClient, final String bulkId) {
		
		BulkStatus retVal = null;
		HttpResponse response = null;
		
		
		try {
			
			response = ersClient.getBulkStatus("endpoint", bulkId);
			RestUtils.printResponseHeaders(response);
			String responseAsString = RestUtils.getResponseContentAsString(response);
						
			//System.out.println("\nRESPONSE CONTENT: \n" +  "	" + responseAsString);
		
			if (RestUtils.getResponseStatus(response) == HttpServletResponse.SC_OK) {
				retVal = (BulkStatus)RestUtils.fromXML(responseAsString, BulkStatus.class);
				if (retVal != null){
					System.out.println("\n bulkstatus of bulkid: " + bulkId + " fetched!");
					System.out.println("\n bulk execution status is: " + retVal.getExecutionStatus());
					System.out.println("\n progress: " + catlculateProgress(retVal) + " %");
					if(retVal.getExecutionStatus().equals(ExecutionStatus.COMPLETED)){
						System.out.println("\n success: " + retVal.getSuccessCount());
						System.out.println("\n fail: " + retVal.getFailCount());
						System.out.println("\n total: " + retVal.getResourcesCount());
					}
				}
				
			}

		} catch (Exception e) {
			e.printStackTrace();
		} finally{
			//release the connection
			if(response != null){
				try {
					response.getEntity().consumeContent();
				} catch (IOException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
			}
		}
		
		
		return retVal;
		
		
	}



	/**
	 * catlculateProgress
	 * @param retVal
	 * @return
	 */
	private static String catlculateProgress(BulkStatus retVal) {
		int progress = 0;
		
		progress = (retVal.getSuccessCount() + retVal.getFailCount())/ retVal.getResourcesCount();
		
		return String.valueOf(100*progress);
	}



	/**
	 * sendCreateBulk
	 * @param ersClient
	 * @param verboseOn
	 */
	private static String sendCreateBulk(final ERSClient ersClient) {
		
		
		System.out.println("\n\n *** about to run a new bulk: creating 500 Endpoints");
		
		
		String bulkID = null;
		HttpResponse response = null;
		
		try {
			
			final EndpointBulkRequest bulkR = new EndpointBulkRequest();
			
			bulkR.setResourcesList(generateStubEPs(500));
			bulkR.setOperationType("create");
			bulkR.setResourceMediaType("vnd.com.cisco.ise.identity.endpoint.1.0+xml");
			
			final JAXBElement<AbstractBulkRequest> jaxbBulk =  ersFactory.createBulkRequest(bulkR);
			
			final String bulkAsXml = jaxbh.marshal(jaxbBulk);
			
			response = ersClient.executeBulk("identity", "endpoint", "endpointbulkrequest",  bulkAsXml);
			
			RestUtils.printResponseHeaders(response);
			
			if(RestUtils.getResponseStatus(response) == HttpServletResponse.SC_ACCEPTED){
				
				bulkID = RestUtils.getNewBornId(response);
				
			}else{
				
				System.out.println("\nRESPONSE CONTENT: \n" + "	" +  RestUtils.getResponseContentAsString(response));
				
			}
	
		} catch (Exception e) {
			System.out.println("\nException :" + e.getMessage());
		}finally{
			if(response != null){
				try {
					response.getEntity().consumeContent();
				} catch (IOException e) {
					System.out.println("\nException :" + e.getMessage());
				}
			}
		}
		
		
		System.out.println("\n Bulk started ! Bulk ID: " + bulkID);
		
		return bulkID;		
	}
		
		
		
		
	/**
	 * 	
	 * @param numOfEps
	 * @return
	 */
	private static EndpointBulkRequest.ResourcesList generateStubEPs(int numOfEps) {
		
		long mac = 0xAABBCCDD0000L;
		
		EndpointBulkRequest.ResourcesList resList = new EndpointBulkRequest.ResourcesList();
		
	    for (long i = 0; i < numOfEps ; i++){
	    	
	    	Endpoint ep = new Endpoint();
	    	ep.setDescription(EP_DSC_ATTR);
	    	ep.setPortalUser("user_" + i);
	    	ep.setMac(generateMac(mac++));
	    
	    	resList.getEndpoint().add(ep);
	    }
	        
		return resList;
	}



	private  static String generateMac(final long mac) {

	    if (mac > 0xFFFFFFFFFFFFL || mac < 0){
	    	throw new IllegalArgumentException("mac out of range");
	    }
	        
	    final StringBuffer macAddr = new StringBuffer(Long.toString(mac, 16));
	    while (macAddr.length() < 12){
	    	macAddr.insert(0, "0");
	    }

	    for (int j = macAddr.length() - 2; j >= 2; j-=2){
	    	macAddr.insert(j, ":");
	    }
	        
	    return macAddr.toString().toUpperCase();
	}

	



	/**
	 * connectivityCheck
	 * send getVersion request to server in order to try connect
	 * and make sure the credentials are OK
	 * @param ersClient
	 */
	private static void connectivityCheck(ERSClient ersClient) {
		HttpResponse response = null;
		try {
			
			System.out.println("\n###############################################################" + 
	           		   		   "\n#  Connectivity Check:                                        #" +
	           		   		   "\n#  About to send GET Endpoint version request to server       #" +
	           		   		   "\n#  in order to make sure that connection can be established.  #" +
	           		   		   "\n###############################################################");
			
			//getting endpoint server version, just for checking that the  
			response = ersClient.getVersion("identity","endpoint");
			String responseAsString = RestUtils.getResponseContentAsString(response);
			int responseStatus = RestUtils.getResponseStatus(response);
			if (responseStatus == HttpServletResponse.SC_OK) {
				System.out.println("\nConnection to ISE server success");
			}else if(responseStatus == HttpServletResponse.SC_UNAUTHORIZED){
				System.out.println("\n###############################################################" + 
						           "\n#  401 - UNAUTHORIZED:                                        #" +
						           "\n#  Connection to ISE server failed !!!                        #" +
						           "\n#  Make sure you are using  valid Admin Credentials.          #" +
						           "\n#  Please note that the admin must have 'ERS-Admin' privilages #" +
						           "\n###############################################################");				
				System.exit(-1);
			}else{
				System.out.println("\nRESPONSE CONTENT: \n" + "	" +  responseAsString);
			}
		} catch (Exception e) {
			e.printStackTrace();
		} finally{
			//release the connection
			if(response != null){
				try {
				    if (response.getEntity() != null) {
                        response.getEntity().consumeContent();
                    }
				} catch (IOException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
			}
		}
	}

	
	/**
	 * getAllEndPoints
	 * send to server GET request and retrieve all endpoins currently in server (paged).
	 * @param ersClient
	 */
	private static List<BaseResource> searchEndPoints(ERSClient ersClient, boolean verboseOn, String queryStringParams) {
		
		HttpResponse response = null;
		
		List<BaseResource> baseResources = null;  
		
		try {
		
			response = ersClient.getAll("identity","endpoint", queryStringParams);
			
			
			if(verboseOn){
				RestUtils.printResponseHeaders(response);
			}
			
			String searchResultAsString = RestUtils.getResponseContentAsString(response);
			
			if (RestUtils.getResponseStatus(response) == HttpServletResponse.SC_OK) {
				
				
				
				if(verboseOn){
					System.out.println("\nRESPONSE CONTENT: \n" + "	" +  searchResultAsString);
				}
				
					
				//unmarshal the result from xml to java object
				SearchResult sr = (SearchResult)RestUtils.fromXML(searchResultAsString, SearchResult.class);
				
				System.out.println("\n Total endpoints found: " + sr.getTotal());
				
				baseResources =  sr.getResources().getResource();
				
				Iterator<BaseResource> it = baseResources.iterator();
				while (it.hasNext()) {
					BaseResource br = (BaseResource) it.next();
					if(verboseOn){
						System.out.println("\n	endpoint (base resource) details: \n id=" + br.getId() + " \n name=" + br.getName() +
								" \n description=" + br.getDescription() + " \n link=" + br.getLink());
					}
					
				}
				
				
			}else{
				System.out.println("\nRESPONSE CONTENT: \n" + "	" +  searchResultAsString);
			}

		} catch (Exception e) {
			e.printStackTrace();
		} finally{
			//release the connection
			if(response != null){
				try {
				    if (response.getEntity() != null) {
                        response.getEntity().consumeContent();
                    }
				} catch (IOException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
			}
		}
		
		return baseResources;
	}

	
	
	
	/**
	 * getAllDemoEPsFromDB
	 * send to server GET request and retrieve all endpoins that was once added by bulk demo.
	 * @param ersClient
	 */
	private static IdList  getAllDemoEPsFromDB(ERSClient ersClient) {
		
		HttpResponse response = null;
		
		List<BaseResource> baseResources = new ArrayList<BaseResource>();  
		
		IdList ids = new IdList();
		
		try {
		
			int page = 1;
			int totalCount = 0;
			
			
			while(true){
				
				response = ersClient.getAll("identity","endpoint", "?page=" + page + "&size=100");
				String searchResultAsString = RestUtils.getResponseContentAsString(response);
				
				if (RestUtils.getResponseStatus(response) == HttpServletResponse.SC_OK) {
					SearchResult sr = (SearchResult)RestUtils.fromXML(searchResultAsString, SearchResult.class);
					totalCount = sr.getTotal();
					baseResources.addAll(sr.getResources().getResource());
					
				}else{
					System.out.println("\nRESPONSE CONTENT: \n" + "	" +  searchResultAsString);
					break;
				}
				page++;
				if(page*100 >= totalCount){
					break;
				}
				
			}
			
			for(BaseResource br : baseResources){
				//TODO: after michal fix for return description as well, shouls uncomment
				//if(br.getDescription() != null && br.getDescription().contains(EP_DSC_ATTR)){
					ids.getId().add(br.getId());
				//}
			}
			
		} catch (Exception e) {
			e.printStackTrace();
		} finally{
			if(response != null){
				try {
				    if (response.getEntity() != null) {
                        response.getEntity().consumeContent();
                    }
				} catch (IOException e) {
					e.printStackTrace();
				}
			}
		}		
		return ids;
	}
}
