//**************************************************
// Copyright (c) 2013-2015 Cisco Systems, Inc.
// All rights reserved.
//**************************************************
package com.cisco.ise.ers.demo;


import java.io.IOException;
import java.net.Socket;
import java.net.UnknownHostException;
import java.security.cert.X509Certificate;
import javax.net.ssl.SSLContext;
import javax.net.ssl.SSLException;
import javax.net.ssl.SSLSession;
import javax.net.ssl.SSLSocket;
import javax.net.ssl.TrustManager;
import org.apache.http.HttpException;
import org.apache.http.HttpHost;
import org.apache.http.HttpRequest;
import org.apache.http.HttpRequestInterceptor;
import org.apache.http.HttpResponse;
import org.apache.http.auth.AuthScheme;
import org.apache.http.auth.AuthScope;
import org.apache.http.auth.AuthState;
import org.apache.http.auth.Credentials;
import org.apache.http.auth.UsernamePasswordCredentials;
import org.apache.http.client.CredentialsProvider;
import org.apache.http.client.methods.HttpDelete;
import org.apache.http.client.methods.HttpEntityEnclosingRequestBase;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.client.methods.HttpPut;
import org.apache.http.client.protocol.ClientContext;
import org.apache.http.conn.scheme.Scheme;
import org.apache.http.conn.ssl.SSLSocketFactory;
import org.apache.http.conn.ssl.X509HostnameVerifier;
import org.apache.http.entity.StringEntity;
import org.apache.http.impl.auth.BasicScheme;
import org.apache.http.impl.client.DefaultHttpClient;
import org.apache.http.protocol.BasicHttpContext;
import org.apache.http.protocol.ExecutionContext;
import org.apache.http.protocol.HttpContext;


/**
 * ERSClient provides the HTTP connectivity for ISE-ERS consumer application
 *
 */
public class ERSClient {

	private static final String HEADER_ACCEPT = "application/xml";

	private static final String HEADER_CONTENT_TYPE = "application/xml";
	
	private DefaultHttpClient httpclient = null;
	private HttpHost targetHost = null;
	private BasicHttpContext localcontext = null;

	public ERSClient(String admin, String password, String serverHost) {

		// Set HTTP client with the credentials provided on constructor
		httpclient = new DefaultHttpClient();
		httpclient.getCredentialsProvider().setCredentials(
				new AuthScope(serverHost, 9060),
				new UsernamePasswordCredentials(admin, password));
		
		localcontext = new BasicHttpContext();

		// Set Authentication scheme to preemptive,
		// which means that credentials are send on first message to server, no
		// prompt for credentials
		BasicScheme basicAuth = new BasicScheme();
		localcontext.setAttribute("preemptive-auth", basicAuth);

		httpclient.addRequestInterceptor(new PreemptiveAuth(), 0);

		targetHost = new HttpHost(serverHost, 9060, "https");

		// System.out.println("to target: " + targetHost);

		// Certificate trust manager, set to allow any certificate
		TrustManager[] trustAllCerts = new TrustManager[1];
		TrustManager tm = new FakeTrustManager();
		trustAllCerts[0] = tm;

		try {
			SSLContext sc = SSLContext.getInstance(SSLSocketFactory.TLS);
			
			
			sc.init(null, trustAllCerts, null);

			TLSSocketFactory socketFactory = new TLSSocketFactory(sc);
			
			X509HostnameVerifier hostnameVerifier = new FakeX509HostnameVerifier(); 
				
			socketFactory.setHostnameVerifier(hostnameVerifier);

			Scheme https = new Scheme("https", socketFactory, 443);

			httpclient.getConnectionManager().getSchemeRegistry()
					.register(https);

		} catch (Exception e) {
			e.printStackTrace();
		}
	}
	
	public static class TLSSocketFactory extends SSLSocketFactory {
		  private static final String[] ENABLED_PROTOCOLS = new String[] {"TLSv1.1","TLSv1.2"};

		  public TLSSocketFactory(SSLContext sslContext) {
		      super(sslContext);
		  }

		  @Override
		  public Socket createSocket() throws IOException {
			  Socket socket = super.createSocket();
			  if(socket instanceof SSLSocket){
			      ((SSLSocket) socket).setEnabledProtocols(ENABLED_PROTOCOLS);
			  }
			  return socket;
		  }

		  @Override
		  public Socket createSocket(Socket socket, String host, int port,boolean autoClose) throws IOException, UnknownHostException {
			  Socket sock = super.createSocket(socket, host, port, autoClose);
			  if(socket instanceof SSLSocket){
			      ((SSLSocket) socket).setEnabledProtocols(ENABLED_PROTOCOLS);
			  }
			  return sock;
		  }
		  
	  }


	
	// Set request intercepter to apply preemptive authentication
	static class PreemptiveAuth implements HttpRequestInterceptor {

		public void process(final HttpRequest request, final HttpContext context)
				throws HttpException, IOException {

			AuthState authState = (AuthState) context
					.getAttribute(ClientContext.TARGET_AUTH_STATE);

			if (authState.getAuthScheme() == null) {
				AuthScheme authScheme = (AuthScheme) context
						.getAttribute("preemptive-auth");
				CredentialsProvider credsProvider = (CredentialsProvider) context
						.getAttribute(ClientContext.CREDS_PROVIDER);
				HttpHost targetHost = (HttpHost) context
						.getAttribute(ExecutionContext.HTTP_TARGET_HOST);
				if (authScheme != null) {
					Credentials creds = credsProvider
							.getCredentials(new AuthScope(targetHost
									.getHostName(), targetHost.getPort()));
					if (creds == null) {
						throw new HttpException(
								"No credentials for preemptive authentication");
					}
					authState.setAuthScheme(authScheme);
					authState.setCredentials(creds);
				}
			}

		}

	}

	/**
	 * HostnameVerifier that Accept all hostname, without verification.
	 * 
	 * THIS X509HostnameVerifier IS FOR DEMO PURPOSES ONLY
	 * SHOULD NOT BE USED FOR PRODUCTION !!!
     * FOR PRODUCTION IT IS REQUIRED TO USE A REAL HOSTVERIFIER
	 */
	public static class FakeX509HostnameVerifier implements X509HostnameVerifier{
		 
		
		public void verify(String arg0, SSLSocket arg1)
				throws IOException {
			
		}

		public void verify(String arg0, X509Certificate arg1)
				throws SSLException {

		}

		public void verify(String arg0, String[] arg1, String[] arg2)
				throws SSLException {

		}

		public boolean verify(String hostname, SSLSession session) {
			return false;
		}
	}
	
	
	/**
	 * TrustManager that approves any certificate.
	 * 
	 * THIS TrustManager IS FOR DEMO PURPOSES ONLY
	 * SHOULD NOT BE USED FOR PRODUCTION
	 * FOR PRODUCTION IT IS REQUIRED TO USE A REAL TRUSTMANAGER
	 *
	 */
	public static class FakeTrustManager implements javax.net.ssl.TrustManager,
			javax.net.ssl.X509TrustManager {
		public java.security.cert.X509Certificate[] getAcceptedIssuers() {
			return null;
		}

		public boolean isServerTrusted(
				java.security.cert.X509Certificate[] certs) {
			return true;
		}

		public boolean isClientTrusted(
				java.security.cert.X509Certificate[] certs) {
			return true;
		}

		public void checkServerTrusted(
				java.security.cert.X509Certificate[] certs, String authType)
				throws java.security.cert.CertificateException {
			return;
		}

		public void checkClientTrusted(
				java.security.cert.X509Certificate[] certs, String authType)
				throws java.security.cert.CertificateException {
			return;
		}
	}

	
	
	
	
	

	public void cleanUp() {
		httpclient.getConnectionManager().shutdown();
	}
	
	
	
	/**
	 * send GET request - get all resources of the specified type.
	 * @param nameSpace
	 * @param resourceType
	 * @return HttpResponse
	 * 
	 */
	public HttpResponse getAll(String nameSpace, String resourceType, String queryStringParams) {
		
		HttpResponse response = null;

		try {
			
			if(queryStringParams == null){
				queryStringParams = "";
			}
			
			HttpGet httpGet = new HttpGet(getURI(resourceType + queryStringParams) );
			
			httpGet.setHeader("accept", HEADER_ACCEPT);
			
			RestUtils.printRequestHeaders(httpGet.getAllHeaders());
			
			response = httpclient.execute(targetHost, httpGet,
					localcontext);
		} catch (Exception e) {
			e.printStackTrace();
		}
		return response;
		
	}


	
	/**
	 * send GET request to fetch the specified resource type of the specified resourceId
	 * @param nameSpace
	 * @param resourceType
	 * @param resourceId
	 * @return HttpResponse
	 * 
	 */
	public HttpResponse getById(String nameSpace, String resourceType, String resourceId) {
		
		HttpResponse response = null;

		try {
			
			HttpGet httpGet = new HttpGet(getURI(resourceType) + "/" + resourceId);
			
			httpGet.setHeader("accept", HEADER_ACCEPT);
			
			RestUtils.printRequestHeaders(httpGet.getAllHeaders());
			
			response = httpclient.execute(targetHost, httpGet,
					localcontext);
			
			
		} catch (Exception e) {
			e.printStackTrace();
		}
		return response;
		
	}
	
	
	/**
	 * send GET request to fetch the specified resource version
	 * @param nameSpace
	 * @param resourceType
	 * @return HttpResponse
	 * 
	 */
	public HttpResponse getVersion(String nameSpace, String resourceType) {
		
		HttpResponse response = null;

		try {
			
			HttpGet httpGet = new HttpGet(getURI(resourceType) + "/versioninfo");
			
			httpGet.setHeader("accept", HEADER_ACCEPT);
			
			RestUtils.printRequestHeaders(httpGet.getAllHeaders());
			
			response = httpclient.execute(targetHost, httpGet,
					localcontext);
			
			
		} catch (Exception e) {
			e.printStackTrace();
		}
		return response;
		
	}
	
	
	
	
	/**
	 * send DELETE request to delete the specified resource type of the specified resourceId
	 * @param nameSpace
	 * @param resourceType
	 * @param resourceId
	 * @return HttpResponse
	 * 
	 */
	public HttpResponse deleteById(String nameSpace, String resourceType, String resourceId) {
		
		HttpResponse response = null;

		try {
			
			HttpDelete httpDelete = new HttpDelete(getURI(resourceType) + "/" + resourceId);
			
			httpDelete.setHeader("accept", HEADER_ACCEPT);
			
			RestUtils.printRequestHeaders(httpDelete.getAllHeaders());
			
			response = httpclient.execute(targetHost, httpDelete,
					localcontext);
			
			
		} catch (Exception e) {
			e.printStackTrace();
		}
		return response;
		
	}
	
	
	
	
	/**
	 * sends POST request to create the specified resource.
	 * @param nameSpace
	 * @param resourceType
	 * @param resourceXml
	 * @return HttpResponse
	 */
	public HttpResponse create(String nameSpace, String resourceType, String resourceXml) {
			HttpResponse response = null;

			try {
				HttpPost httpPost = new HttpPost(getURI(resourceType));
				StringEntity stringEntity = new StringEntity(resourceXml, "UTF-8");
				
				
				httpPost.setEntity(stringEntity);
				httpPost.setHeader("Content-Type", HEADER_CONTENT_TYPE);
				RestUtils.printRequestHeaders(httpPost.getAllHeaders());
				
				response = httpclient.execute(targetHost, httpPost, localcontext);
				

			} catch (Exception e) {
				e.printStackTrace();
			}
			return response;
		}
	
	
	/**
	 * sends PUT request to update the specified resource.
	 * @param nameSpace
	 * @param resourceType
	 * @param resourceid
	 * @param resourceXml
	 * @return HttpResponse
	 */
	public HttpResponse update(String nameSpace, String resourceType, String resourceid ,String resourceXml) {
			HttpResponse response = null;

			try {
				HttpPut httpPut = new HttpPut(getURI(resourceType) + "/" + resourceid);
				StringEntity stringEntity = new StringEntity(resourceXml, "UTF-8");
				
				
				httpPut.setEntity(stringEntity);
				httpPut.setHeader("content-type", HEADER_CONTENT_TYPE);
				RestUtils.printRequestHeaders(httpPut.getAllHeaders());
				
				response = httpclient.execute(targetHost, httpPut, localcontext);
				

			} catch (Exception e) {
				e.printStackTrace();
			}
			return response;
		}

    /**
     * sends PUT request to register the specified resource.
     * @param nameSpace
     * @param resourceType
     * @param resourceXml
     * @return HttpResponse
     */
	public HttpResponse register(String nameSpace, String resourceType, String resourceXml) {
	    HttpResponse response = null;

	    try {
	        HttpPut httpPut = new HttpPut(getURI(resourceType) + "/register");
            StringEntity stringEntity = new StringEntity(resourceXml, "UTF-8");
            
            
            httpPut.setEntity(stringEntity);
            httpPut.setHeader("content-type",HEADER_CONTENT_TYPE);
            RestUtils.printRequestHeaders(httpPut.getAllHeaders());
            
            response = httpclient.execute(targetHost, httpPut, localcontext);
	    } catch (Exception e) {
	        e.printStackTrace();
	    }
	    return response;
	}

	/**
	 * sends PUT request to deregister the specified resource.
	 * @param nameSpace
	 * @param resourceType
	 * @param resourceXml
	 * @return HttpResponse
	 */
	public HttpResponse deregister(String nameSpace, String resourceType, String resourceId) {
	    HttpResponse response = null;

	    try {
	        HttpPut httpPut = new HttpPut(getURI(resourceType) + "/" + resourceId + "/dergister");
            
            httpPut.setHeader("content-type", HEADER_CONTENT_TYPE);
            RestUtils.printRequestHeaders(httpPut.getAllHeaders());
            
            response = httpclient.execute(targetHost, httpPut, localcontext);

	    } catch (Exception e) {
	        e.printStackTrace();
	    }
	    return response;
	}

	/**
	 * sends PUT request to perform a custom operation.
	 * @param nameSpace
	 * @param resourceType
	 * @param resourceXml
	 * @return HttpResponse
	 */
	public HttpResponse executeBulk(final String nameSpace, final String resourceType,
			final String bulkType, final String reqContent) {
	    HttpResponse response = null;

	    try {
	        HttpPut httpPut = new HttpPut(getURI(resourceType) + "/bulk/submit");
            StringEntity stringEntity = new StringEntity(reqContent, "UTF-8");
            
            
            httpPut.setEntity(stringEntity);
            httpPut.setHeader("content-type", HEADER_CONTENT_TYPE);
            RestUtils.printRequestHeaders(httpPut.getAllHeaders());
            
            response = httpclient.execute(targetHost, httpPut, localcontext);
	    } catch (Exception e) {
	        e.printStackTrace();
	    }
	    return response;
	}
	
	
	
	/**
	 * Sends GET request to retrieve the execution progress of the specified bulk.
	 * @param resourceType - the resource on which the Bulk operates
	 * @param bulkId
	 * @return
	 */
	public HttpResponse getBulkStatus(String resourceType, String bulkId) {
		
		HttpResponse response = null;

		try {
			
			HttpGet httpGet = new HttpGet(getURI(resourceType) + "/bulk/" + bulkId);
			
			httpGet.setHeader("accept", "application/vnd.com.cisco.ise.ers.bulkstatus.1.0+xml");
			
			RestUtils.printRequestHeaders(httpGet.getAllHeaders());
			
			response = httpclient.execute(targetHost, httpGet,
					localcontext);
			
			
		} catch (Exception e) {
			e.printStackTrace();
		}
		return response;
		
	}
	
	/**
	 * Allows for custom operations that have an enclosing Resource XML.
	 * 
	 * @param nameSpace - Namespace of resource (eg. identity or ca)
	 * @param resourceType - The type of resource to act on (eg. endpoint or endpointcert)
	 * @param resourceXml - The XML payload in the request
	 * @param operation - The name of the custom operation to add onto the request URI
	 * @param method - The HTTP Method to use. Should be either PUT or POST
	 * @return
	 */
	public HttpResponse customOperation(String nameSpace, String resourceType, String resourceXml, String operation,
	        String method) {
	    HttpResponse response = null;

        try {
            HttpEntityEnclosingRequestBase httpRequest = getEntityEnclosingMethod(resourceType, operation, method);
            if (httpRequest == null) {
                System.out.println("Only PUT and POST methods can have an enclosing resource XML."); // NOPMD
                return null;
            }
            StringEntity stringEntity = new StringEntity(resourceXml, "UTF-8");
            
            
            httpRequest.setEntity(stringEntity);
            httpRequest.setHeader("Content-Type", HEADER_CONTENT_TYPE);
            RestUtils.printRequestHeaders(httpRequest.getAllHeaders());
            
            response = httpclient.execute(targetHost, httpRequest, localcontext);
        } catch (Exception e) {
            e.printStackTrace(); // NOPMD
        }
        return response;
	}
	
	/*
	 * Only PUT and POST allow for enclosing data
	 */
	private HttpEntityEnclosingRequestBase getEntityEnclosingMethod(String resourceType, String operation, String method) {
	    if (method.equals(HttpPut.METHOD_NAME)) {
	        return new HttpPut(getURI(resourceType) + "/" + operation);
	    }
	    if (method.equals(HttpPost.METHOD_NAME)) {
	        return new HttpPost(getURI(resourceType) + "/" + operation);
	    }
	    return null;
	}
	
	
	
	
	
	
	
	
	/**
	 * 
	 * @param resourceType
	 * @return
	 */
	private String getURI(String resourceType) {
		System.out.println("\nRequest URI: " + "/ers/config/" + resourceType);
		return "/ers/config/" + resourceType;
	}

	
	
	
	
}
