//**************************************************
// Copyright (c) 2015 Cisco Systems, Inc.
// All rights reserved.
//**************************************************

package com.cisco.ise.ers.demo;


import java.io.IOException;
import java.util.Iterator;
import java.util.List;
import javax.servlet.http.HttpServletResponse;
import javax.xml.bind.JAXBElement;
import org.apache.http.HttpResponse;
import com.cisco.ise.ers.BaseResource;
import com.cisco.ise.ers.v2.SearchResult;
import com.cisco.ise.ers.UpdatedField;
import com.cisco.ise.ers.UpdatedFieldsList;
import com.cisco.ise.ers.identity.Endpoint;
import com.cisco.ise.ers.identity.ObjectFactory;


/**
 * EndPointDemo
 *
 * This is as example for client code that uses ISE Endpoint Rest APIs to
 * search endpoints, delete, register and deregister.
 * The only ISE jars that it depends on is the ers-schema jar.
 * 
 * 
 * This example runs the following Rest Interface calls: 
 * 1. Get all endpoints
 * 2. Create 3 endpoints
 * 3. Register endpoint
 * 4. Get endpoint by ID (the registered endpoint)
 * 5. Deregister endpoint
 * 6. Get endpoint by ID (the registered endpoint)
 * 7. Deregister endpoint
 * 8. Get endpoint by ID (the de-registered endpoint)
 * 9. Delete by ID the 3 endpoints
 *
 * @since 1.2
 *
 */
@SuppressWarnings({"PMD.SystemPrintln"})
public class EndPointDemo {
	
	static ERSClient ersClient;
	
	static ObjectFactory identityFactory = new ObjectFactory();
	
	
	static JAXBHelper jaxbh = new JAXBHelper(identityFactory.getClass().getPackage().getName());
	
	/**
	 * expect 3 arguments: serverHostIp, Admin endpointname of type ERS Admin, and its password
	 * Then it runs 
	 * @param args
	 * @throws Exception
	 */
	public static void main(String[] args) throws Exception {
	
		String serverHost = null;
		String endpointName = null;
		String password = null;

		if (args != null && args.length == 3) {
			
			serverHost = args[0]; // Server Host
			endpointName = args[1]; // Administrator name
			password = args[2]; // Administrator password
			
			System.out.println("\n ISE ip:" + serverHost + " endpoint:" + endpointName + " psw:" + password);
			
			// Get an instance of Rest Client
			ERSClient ersClient = new ERSClient(endpointName, password, serverHost);
			
			System.out.println("\n\n\n\n\n\n" + "--------------------------------- START --------------------------------------------------");
			
			// Connectivity Check
			connectivityCheck(ersClient);
			
			// demo user approve
			DemoAppHelpers.demoApprove();
			
			// Get all endpoints
			System.out.println("\n\n\n\n\n\n" + "----------- Get all EndPoints ----------------------------------------------------------------");
			System.out.println("\n###############################################################" + 
			           		   "\n#  Get All Endpoints:                                         #" +
			           		   "\n#  This demo sends GET request to server to get all(paged)    #" +
			           		   "\n#  endpoints. The expected response would be status 200       #" +
			           		   "\n#  with Search Result as the reponse content, containing the  #" +
			           		   "\n#  first page of the endpoints. The demo will also            #" +
			           		   "\n#  marshal the xml response content to java object of type    #" +
			           		   "\n#  search result, and will print its attributes.              #" +
			           		   "\n###############################################################");
	
			getAllEndPoints(ersClient, true, "?page=1&size=20");

			// create endpoint
			System.out.println("\n\n\n\n\n\n" + "----------- create endpoints -----------------------------------------------------------------");
			System.out.println("\n###############################################################" + 
			           		   "\n#  Create endpoints:                                           #" +
			           		   "\n#  This demo sends POST request to create new endpoints        #" +
			           		   "\n#  The expected response would be status 201 whereas the       #" +
			           		   "\n#  Location header will hold the newborn endpoint id.          #" +
			           		   "\n###############################################################");
		
			String endpointId = createEndPoint(ersClient, "aa:bb:11:22:33:44", "user1");
			String endpointId1 = createEndPoint(ersClient, "aa:bb:11:22:33:55", "user2");
			String endpointId2 = createEndPoint(ersClient, "aa:bb:11:22:33:66", "user3");
			
			// Register endpoint
			System.out.println("\n\n\n\n\n\n" + "----------- register endpoint -----------------------------------------------------------------");
            System.out.println("\n###############################################################" + 
                               "\n#  Register endpoint:                                         #" +
                               "\n#  This demo sends PUT request to register an existing        #" +
                               "\n#  endpoint mac:     aa:bb:11:22:33:44                        #" +
                               "\n#  The expected response would be status 204 no content.      #" +
                               "\n###############################################################");

			registerEndPoint(ersClient, "aa:bb:11:22:33:44", "user7");

			//get specify endpoint by id
			System.out.println("\n\n\n\n\n\n" + "----------- get Endpoint by id -------------------------------------------------------");
			System.out.println("\n#################################################################" + 
			           		   "\n#  Get endpoint by id:                                          #" +
			           		   "\n#  This demo sends GET request to fetch a specified endpoint by #" +
			           		   "\n#  its id. The expected response would be status 200 whereas    #" +
			           		   "\n#  the response content will hold the requested endpoint.       #" +
			           		   "\n#  The demo will also unmarshal the xml response content to     #" +
			           		   "\n#  java object of type endpoint, and will print                 #" +
			           		   "\n#  its attributes.                                              #" +
			           		   "\n#################################################################");
			           		   
			getEndPointByID(ersClient, endpointId);

			// Deregister endpoint
            System.out.println("\n\n\n\n\n\n" + "----------- deregister endpoint -----------------------------------------------------------------");
            System.out.println("\n###############################################################" + 
                               "\n#  De register endpoint:                                      #" +
                               "\n#  This demo sends PUT request to deregister an existing      #" +
                               "\n#  endpoint mac:     aa:bb:11:22:33:44                        #" +
                               "\n#  The expected response would be status 201.                 #" +
                               "\n###############################################################");

            deregisterEndPoint(ersClient, endpointId);

            System.out.println("\n\n\n\n\n\n" + "----------- get Endpoint by id -------------------------------------------------------");
            System.out.println("\n#################################################################" + 
                               "\n#  Get endpoint by id after deregister.                         #" +
                               "\n#################################################################");
                               
            getEndPointByID(ersClient, endpointId);

            
			//update endpoint
			System.out.println("\n\n\n\n\n\n" + "----------- update endpoint ------------------------------------------------------------------");
			System.out.println("\n####################################################################" + 
			           		   "\n#  Update endpoint:                                                    #" +
			           		   "\n#  This demo sends PUT request to server for updating an endpoint  #" +
			           		   "\n#  The java endpoint object is marshaled to xml to become the      #" +
			           		   "\n#  request content.                                                #" +
			           		   "\n#  The expected response would be status 200                       #" +
			           		   "\n#  with UpdatedFields as the reponse content which will be         #" +
			           		   "\n#  marshal to java object of type UpdatedFields, and will          #" +
			           		   "\n#  print it to console .                                           #" +
			           		   "\n####################################################################");
			updateEndPoint(ersClient, endpointId);

			
			//delete endpoint by id
			System.out.println("\n\n\n\n\n\n" + "----------- delete endpoint by id (3) ---------------------------------------------------------");
			System.out.println("\n###############################################################" + 
			           		   "\n#  Delete endpoint by id:                                     #" +
			           		   "\n#  This demo sends DELETE request to delete the endpoint by   #" +
			           		   "\n#  id. The expected response would be status 204 (no content) #" +
			           		   "\n#  Called 3 times to delete all created endpoints             #" +
			           		   "\n###############################################################");
			deleteEndPointById(ersClient, endpointId);
			deleteEndPointById(ersClient, endpointId1);
			deleteEndPointById(ersClient, endpointId2);
			
			System.out.println("\n\n\n\n\n\n" + "--------------------------------- END ----------------------------------------------------");

		}else{
			System.out.println("\n illegal argument. \n syntax: \n EndPointDemo.sh [host-ip] [endpoint-name] [password]\n");
		}
	}
	
	
	
	

	/**
	 * connectivityCheck
	 * send getVersion request to server in order to try connect
	 * and make sure the credentials are OK
	 * @param ersClient
	 */
	private static void connectivityCheck(ERSClient ersClient) {
		HttpResponse response = null;
		try {
			
			System.out.println("\n###############################################################" + 
	           		   		   "\n#  Connectivity Check:                                        #" +
	           		   		   "\n#  About to send GET Endpoint version request to server       #" +
	           		   		   "\n#  in order to make sure that connection can be established.  #" +
	           		   		   "\n###############################################################");
			
			//getting endpoint server version, just for checking that the  
			response = ersClient.getVersion("identity","endpoint");
			String responseAsString = RestUtils.getResponseContentAsString(response);
			int responseStatus = RestUtils.getResponseStatus(response);
			if (responseStatus == HttpServletResponse.SC_OK) {
				System.out.println("\nConnection to ISE server success");
			}else if(responseStatus == HttpServletResponse.SC_UNAUTHORIZED){
				System.out.println("\n###############################################################" + 
						           "\n#  401 - UNAUTHORIZED:                                        #" +
						           "\n#  Connection to ISE server failed !!!                        #" +
						           "\n#  Make sure you are using  valid Admin Credentials.          #" +
						           "\n#  Please note that the admin must have 'ERS-Admin' privilages #" +
						           "\n###############################################################");				
				System.exit(-1);
			}else{
				System.out.println("\nRESPONSE CONTENT: \n" + "	" +  responseAsString);
			}
		} catch (Exception e) {
			e.printStackTrace();
		} finally{
			//release the connection
			if(response != null){
				try {
				    if (response.getEntity() != null) {
                        response.getEntity().consumeContent();
                    }
				} catch (IOException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
			}
		}
	}

	private static void deleteEndPointById(ERSClient ersClient, String endpointId) {
		HttpResponse response = null;
		
		try {
			
			response = ersClient.deleteById("identity","endpoint", endpointId);
			RestUtils.printResponseHeaders(response);
			String responseAsString = RestUtils.getResponseContentAsString(response);
			
			System.out.println("\nRESPONSE CONTENT: \n" + "	" +  responseAsString);
			
			if (RestUtils.getResponseStatus(response) == HttpServletResponse.SC_NO_CONTENT) {
					
				System.out.println("endpoint id " + endpointId + " deleted" );
				
			}else{
				System.out.println("failed to delete endpoint with id " + endpointId);
			}

		} catch (Exception e) {
			e.printStackTrace();
		} finally{
			//release the connection
			if(response.getEntity() != null){
				try {
				    if (response.getEntity() != null) {
                        response.getEntity().consumeContent();
                    }
				} catch (IOException e) {
					// TODO Auto-generated catch block
					//e.printStackTrace();
				}
			}
		}
	
	}


	private static void updateEndPoint(ERSClient ersClient, String endpointId) {
		
		HttpResponse response = null;
		Endpoint endpoint = null;
		
		try {
		    endpoint = getEndPointByID(ersClient, endpointId);
			if(endpoint != null){
				endpoint.setStaticGroupAssignment(true);
				endpoint.setStaticGroupAssignment(true);
				
				JAXBElement<Endpoint> jaxbEndPoint = identityFactory.createEndpoint(endpoint);
				String endpointAsXml = jaxbh.marshal(jaxbEndPoint);

				response = ersClient.update("identity", "endpoint", endpointId, endpointAsXml);
				
				RestUtils.printResponseHeaders(response);
				String responseAsString = RestUtils.getResponseContentAsString(response);
				System.out.println("\nRESPONSE CONTENT: \n" + "	" + responseAsString);
				
				if (RestUtils.getResponseStatus(response) == HttpServletResponse.SC_OK) {
					
				
					UpdatedFieldsList updatedFields = (UpdatedFieldsList)RestUtils.fromXML(responseAsString, UpdatedFieldsList.class);
					List<UpdatedField> uflds = updatedFields.getUpdatedField();
					
					for(UpdatedField uf : uflds){
						
						System.out.println(uf.getField() + " field was updated: oldVal=" + uf.getOldValue() + " newVal=" + uf.getNewValue() + "\n");
					}
				}
			}
		} catch (Exception e) {
			e.printStackTrace();
		} finally{
			//release the connection
			if(response != null){
				try {
				    if (response.getEntity() != null) {
				        response.getEntity().consumeContent();
				    }
				} catch (IOException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
			}
		}
	
	}

	/**
	 * 
	 * searchEndPointByName
	 * send search request with filter.
	 * filter operator can be: 
	 * EQ,NEQ,GT,LT,STARTSW,NSTARTSW,ENDSW,NENDSW,CONTAINS,NCONTAINS;
	 * @param ersClient
	 * @param name
	 * @return
	 */
	private static Endpoint searchEndPointByName(ERSClient ersClient, String name) {
		
		Endpoint endpoint = null;
		List<BaseResource> baseResources  = getAllEndPoints(ersClient, false, "?filter=name.EQ." + name);
		
		if(baseResources != null && baseResources.size() == 1){
			endpoint = getEndPointByID(ersClient, baseResources.get(0).getId()); 
		}
		return endpoint;
	}

	private static Endpoint getEndPointByID(ERSClient ersClient, String endpointId) {
		
		HttpResponse response = null;
		Endpoint endpoint = null;
		
		try {
			
			response = ersClient.getById("identity","endpoint",endpointId);
			RestUtils.printResponseHeaders(response);
			String responseAsString = RestUtils.getResponseContentAsString(response);
			System.out.println("\nRESPONSE CONTENT: \n" +  "	" + responseAsString);
		
			
			
			if (RestUtils.getResponseStatus(response) == HttpServletResponse.SC_OK) {
					
				//unmarshal the result from xml to java object
				endpoint = (Endpoint)RestUtils.fromXML(responseAsString, Endpoint.class);
				
				if (endpoint!=null){
					System.out.println("\n endpoint " + endpoint.getMac() + " fetched!");
					System.out.println("\n endpoint " + endpoint.getMac());
				}
			}

		} catch (Exception e) {
			e.printStackTrace();
		} finally{
			//release the connection
			if(response != null){
				try {
					if (response.getEntity() != null) {
					    response.getEntity().consumeContent();
					}
				} catch (IOException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
			}
		}
		
		return endpoint;
		
	}

	/**
	 * getAllEndPoints
	 * send to server GET request and retrieve all endpoins currently in server (paged).
	 * @param ersClient
	 */
	private static List<BaseResource> getAllEndPoints(ERSClient ersClient, boolean verboseOn, String queryStringParams) {
		
		HttpResponse response = null;
		
		List<BaseResource> baseResources = null;  
		
		try {
		
			response = ersClient.getAll("identity","endpoint", queryStringParams);
			
			
			if(verboseOn){
				RestUtils.printResponseHeaders(response);
			}
			
			String searchResultAsString = RestUtils.getResponseContentAsString(response);
			
			if (RestUtils.getResponseStatus(response) == HttpServletResponse.SC_OK) {
				
				
				
				if(verboseOn){
					System.out.println("\nRESPONSE CONTENT: \n" + "	" +  searchResultAsString);
				}
				
					
				//unmarshal the result from xml to java object
				//SearchResult sr = (SearchResult)RestUtils.fromXML(searchResultAsString, SearchResult.class);
				com.cisco.ise.ers.v2.SearchResult sr = (com.cisco.ise.ers.v2.SearchResult)RestUtils.fromXML(searchResultAsString, com.cisco.ise.ers.v2.SearchResult.class);
				System.out.println("\n Total endpoints found: " + sr.getTotal());
				
				baseResources =  sr.getResources().getResource();
				
				Iterator<BaseResource> it = baseResources.iterator();
				while (it.hasNext()) {
					BaseResource br = (BaseResource) it.next();
					if(verboseOn){
						System.out.println("\n	endpoint (base resource) details: \n id=" + br.getId() + " \n name=" + br.getName() +
								" \n description=" + br.getDescription() + " \n link=" + br.getLink());
					}
					
				}
				
				
			}else{
				System.out.println("\nRESPONSE CONTENT: \n" + "	" +  searchResultAsString);
			}

		} catch (Exception e) {
			e.printStackTrace();
		} finally{
			//release the connection
			if(response != null){
				try {
				    if (response.getEntity() != null) {
                        response.getEntity().consumeContent();
                    }
				} catch (IOException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
			}
		}
		
		return baseResources;
	}

	/**
	 * create an endpoint and return its id
	 * @param ersClient
	 * @param mac address
	 * @param user portal user
	 * @return
	 */
	private static String createEndPoint(ERSClient ersClient, String mac, String user) {
		
		System.out.println("\n\n *** about to create new endpoint mac: " + mac);
		
		String newBornId = null;
		HttpResponse response = null;

		try {
		
			Endpoint endpoint = new Endpoint();
			endpoint.setMac(mac);
			endpoint.setPortalUser(user);
			endpoint.setDescription("sdk demo endpoint");
			
			
			JAXBElement<Endpoint> jaxbEndPoint = identityFactory.createEndpoint(endpoint);
			
			String endpointAsXml = jaxbh.marshal(jaxbEndPoint);
			
			
			response = ersClient.create("identity","endpoint", endpointAsXml);
			
			RestUtils.printResponseHeaders(response);
			
			if(RestUtils.getResponseStatus(response) == HttpServletResponse.SC_CREATED){
				
				newBornId = RestUtils.getNewBornId(response);
				
			}else{
				//error -> print response content
				
				System.out.println("\nRESPONSE CONTENT: \n" + "	" +  RestUtils.getResponseContentAsString(response));
				
			}
	
		} catch (Exception e) {
			e.printStackTrace();
		}finally{
			//release the connection
			if(response != null){
				try {
				    if (response.getEntity() != null) {
                        response.getEntity().consumeContent();
                    }
				} catch (IOException e) {
					e.printStackTrace();
				}
			}
		}

		return newBornId;		
	}

	/**
     * register an endpoint, can be new endpoint or an existing endpoint.
     * @param ersClient
     * @param mac address
     * @param user portal user
     * @return
     */
    private static void registerEndPoint(ERSClient ersClient, String mac, String user) {
        
        System.out.println("\n\n *** about to create new endpoint mac: " + mac);
        
        HttpResponse response = null;

        try {
        
            Endpoint endpoint = new Endpoint();
            endpoint.setMac(mac);
            endpoint.setPortalUser(user);
            endpoint.setDescription("sdk demo endpoint");

            JAXBElement<Endpoint> jaxbEndPoint = identityFactory.createEndpoint(endpoint);

            String endpointAsXml = jaxbh.marshal(jaxbEndPoint);
            
            
            response = ersClient.register("identity","endpoint", endpointAsXml);
            
            RestUtils.printResponseHeaders(response);
            
            if(RestUtils.getResponseStatus(response) == HttpServletResponse.SC_NO_CONTENT) {
                System.out.println("Successfull registration");
            }else{
                //error -> print response content
                
                System.out.println("\nRESPONSE CONTENT: \n" + " " +  RestUtils.getResponseContentAsString(response));
                
            }
    
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    /**
     * Deregister an endpoint
     * @param ersClient
     * @param id endpoint id
     * @param user portal user
     */
    private static void deregisterEndPoint(ERSClient ersClient, String endpointId) {

        System.out.println("\n\n *** about to deregister endpoint id: " + endpointId);
        
        HttpResponse response = null;

        try {
        
            
            response = ersClient.deregister("identity", "endpoint", endpointId);
            
            RestUtils.printResponseHeaders(response);
            
            if (RestUtils.getResponseStatus(response) == HttpServletResponse.SC_NO_CONTENT) {

                System.out.println("Successfull registration");
                
            }else{
                //error -> print response content
                
                System.out.println("\nRESPONSE CONTENT: \n" + " " +  RestUtils.getResponseContentAsString(response));
                
            }
    
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
}
