//**************************************************
// Copyright (c) 2013-2016 Cisco Systems, Inc.
// All rights reserved.
//**************************************************

package com.cisco.ise.ers.demo;


import java.io.IOException;
import java.util.Iterator;
import java.util.List;
import javax.servlet.http.HttpServletResponse;
import javax.xml.bind.JAXBElement;
import org.apache.http.HttpResponse;
import org.apache.http.util.EntityUtils;
import com.cisco.ise.ers.BaseResource;
import com.cisco.ise.ers.v2.SearchResult;
import com.cisco.ise.ers.UpdatedField;
import com.cisco.ise.ers.UpdatedFieldsList;
import com.cisco.ise.ers.identity.InternalUser;
import com.cisco.ise.ers.identity.InternalUser.CustomAttributes;
import com.cisco.ise.ers.identity.ObjectFactory;


/**
 * InternalUserDemo
 *
 * This is as example of client code that uses ISE InternalUser Rest APIs to
 * configure users. The only ISE jars that it depends upon is the ers-schema jar.
 * 
 * 
 * This example runs the following Rest Interface calls: 
 * 1. Get all users
 * 2. Create new users 
 * 3. Get user by name (filter by name) 
 * 4. Get user by Id 
 * 5. Update user 
 * 6. Delete user by object Id
 * 7. Delete User by name 
 * 8. Delete by ID the 2 user
 * 
 */
public class InternalUserDemo {

	
	
	static ERSClient ersClient;
	static ObjectFactory identityFactory = new ObjectFactory();
	static JAXBHelper jaxbh = new JAXBHelper(identityFactory.getClass().getPackage().getName());
	
	/**
	 * expect 3 arguments: serverHostIp, Admin username of type ERS Admin, and its password
	 * Then it runs 
	 * @param args
	 * @throws Exception
	 */
	public static void main(String[] args) throws Exception {
	
		String serverHost = null;
		String userName = null;
		String password = null;

		if (args != null && args.length == 3) {
			
			serverHost = args[0]; // Server Host
			userName = args[1]; // Administrator name
			password = args[2]; // Administrator password
			
			System.out.println("\n ISE ip:" + serverHost + " user:" + userName + " psw:" + password);
			
			// Get an instance of Rest Client
			ERSClient ersClient = new ERSClient(userName, password, serverHost);
			
			System.out.println("\n\n\n\n\n\n" + "--------------------------------- START --------------------------------------------------");
			
			// Connectivity Check
			connectivityCheck(ersClient);

			
			// demo approve
			DemoAppHelpers.demoApprove();
			
			// Get all users
			System.out.println("\n\n\n\n\n\n" + "----------- Get all Users ----------------------------------------------------------------");
			System.out.println("\n###############################################################" + 
			           		   "\n#  Get All Internal Users:                                    #" +
			           		   "\n#  This demo sends GET request to server to get all(paged)    #" +
			           		   "\n#  Internal users. The expected response would be status 200  #" +
			           		   "\n#  with Search Result as the reponse content containing the   #" +
			           		   "\n#  first page of the internal users. The demo will also       #" +
			           		   "\n#  marshal the xml response content to java object of type    #" +
			           		   "\n#  search result and will print its attributes.               #" +
			           		   "\n###############################################################");
	
			getAllUsers(ersClient, true, "?page=1&size=50");
			
			// create users
			System.out.println("\n\n\n\n\n\n" + "----------- create users -----------------------------------------------------------------");
			System.out.println("\n###############################################################" + 
			           		   "\n#  Create user:                                               #" +
			           		   "\n#  This demo sends POST request to create new user            #" +
			           		   "\n#  The expected response would be status 201 whereas the      #" +
			           		   "\n#  LOCATION header will hold the newborn user id.             #" +
			           		   "\n###############################################################");
			String user1id = createUser(ersClient, "userERSDemo1", "Password123");
			String user1id1 = createUser(ersClient, "userERSDemo2", "Password123");
			String user1id2 =createUser(ersClient, "userERSDemo3", "Password123");
			String user1id3 =createUser(ersClient, "userERSDemo4", "Password123");
			
			// Get all users
			System.out.println("\n\n\n\n\n\n" + "----------- Get all Users ----------------------------------------------------------------");
			System.out.println("\n###############################################################" + 
			           		   "\n#  Get All Internal Users:                                    #" +
			           		   "\n#  Send GET All once more to retrieve and show the new users  #" +						           		   
			           		   "\n###############################################################");
			getAllUsers(ersClient, true, "?page=1&size=50");
			
			//System.exit(1);
			
			
			//get specify user by id
			
			System.out.println("\n\n\n\n\n\n" + "----------- get specify user by id -------------------------------------------------------");
			System.out.println("\n###############################################################" + 
			           		   "\n#  Get user by id:                                            #" +
			           		   "\n#  This demo sends GET request to fetch a specified user by   #" +
			           		   "\n#  its id. The expected response would be status 200 whereas  #" +
			           		   "\n#  the response content will hold the requested internal user.#" +
			           		   "\n#  The demo will also unmarshal the xml response content to   #" +
			           		   "\n#  java object of type internal user and will print           #" +
			           		   "\n#  its attributes.                                            #" +
			           		   "\n###############################################################");
			           		   
			getUserByID(ersClient,user1id);
			
			//search user by name
			System.out.println("\n\n\n\n\n\n" + "----------- search user by name ----------------------------------------------------------");
			System.out.println("\n###############################################################" + 
			           		   "\n#  Get user by name:                                          #" +
			           		   "\n#  This demo sends GET request to server to get all(paged)    #" +
			           		   "\n#  Internal users filtered by name.                           #" +
			           		   "\n#  The expected response would be status 200                  #" +
			           		   "\n#  with Search Result as the reponse content, containing the  #" +
			           		   "\n#  filtered users with name=userERSDemo2.                     #" +
			           		   "\n#  Then it marshals the xml response content to java object of#" + 
			           		   "\n#  type SearchResult, and will print its attributes.          #" +
			           		   "\n###############################################################");
			searchUserByName(ersClient,"userERSDemo2");
			
			//update user
			System.out.println("\n\n\n\n\n\n" + "----------- update user ------------------------------------------------------------------");
			System.out.println("\n###############################################################" + 
			           		   "\n#  Update user:                                               #" +
			           		   "\n#  This demo sends PUT request to server for updating a user  #" +
			           		   "\n#  The java user object is marshaled to xml to become the     #" +
			           		   "\n#  request content.                                           #" +
			           		   "\n#  The expected response would be status 200                  #" +
			           		   "\n#  with UpdatedFields as the reponse content which will be    #" +
			           		   "\n#  Then it marshals to java object of type UpdatedFields, and #" +
			           		   "\n#  print it to console .                                      #" +
			           		   "\n###############################################################");
			updateUser(ersClient, user1id);
				
			//delete user by id
			System.out.println("\n\n\n\n\n\n" + "----------- delete user by id ------------------------------------------------------------");
			System.out.println("\n###############################################################" + 
			           		   "\n#  Delete user by id:                                         #" +
			           		   "\n#  This demo sends DELETE request to delete the user by id    #" +
			           		   "\n#  The expected response would be status 204 (no content)     #" +
			           		   "\n###############################################################");
			deleteUserById(ersClient, user1id);
			
			//delete by name
			System.out.println("\n\n\n\n\n\n" + "----------- delete user by name ----------------------------------------------------------");
			System.out.println("\n###############################################################" + 
			           		   "\n#  Delete user by name:                                       #" +
			           		   "\n#  This demo demonstrate how to delete a user by its name.    #" +
			           		   "\n#  This is done by first searching it, then getting its id    #" +
			           		   "\n#  and then delete by the id.                                 #" +
			           		   "\n#  The expected response would be status 204 (no content)     #" +
			           		   "\n###############################################################");
			
			

			deleteUserByName(ersClient, "userERSDemo2");
			
			//delete user by id
			System.out.println("\n\n\n\n\n\n" + "----------- delete user by id ------------------------------------------------------------");
			System.out.println("\n###############################################################" + 
			           		   "\n#  Delete user by id:                                         #" +
			           		   "\n#  This demo sends DELETE request to delete the user by id    #" +
			           		   "\n#  The expected response would be status 204 (no content)     #" +
			           		   "\n###############################################################");
			deleteUserById(ersClient, user1id2);
			deleteUserById(ersClient, user1id3);
			
			System.out.println("\n\n\n\n\n\n" + "--------------------------------- END ----------------------------------------------------");
			
		}else{
			System.out.println("\n illegal argument. \n syntax: \n InternalUserDemo.sh [host-ip] [user-name] [password]\n");
		}
	}
	
	
	
	

	/**
	 * connectivityCheck
	 * send getVersion request to server in order to try connect
	 * and make sure the credentials are OK
	 * @param ersClient
	 */
	private static void connectivityCheck(ERSClient ersClient) {
		HttpResponse response = null;
		try {
			
			System.out.println("\n###############################################################" + 
	           		   		   "\n#  Connectivity Check:                                        #" +
	           		   		   "\n#  About to send GET Internaluser version request to server   #" +
	           		   		   "\n#  in order to make sure that connection can be established.  #" +
	           		   		   "\n###############################################################");
			
			//getting internal user server version, just for checking that the  
			response = ersClient.getVersion("identity","internaluser");
			String responseAsString = RestUtils.getResponseContentAsString(response);
			int responseStatus = RestUtils.getResponseStatus(response);
			if (responseStatus == HttpServletResponse.SC_OK) {
				System.out.println("\nConnection to ISE server success");
			}else if(responseStatus == HttpServletResponse.SC_UNAUTHORIZED){
				System.out.println("\n###############################################################" + 
						           "\n#  401 - UNAUTHORIZED:                                        #" +
						           "\n#  Connection to ISE server failed !!!                        #" +
						           "\n#  Make sure ERS is enabled (CLI: application configure ise). #" +
						           "\n#  Make sure you are using  valid ERS Admin Credentials.      #" +
						           "\n#  Please note that the admin must have 'ERS-Admin' privileges#" +
						           "\n###############################################################");				
				System.exit(-1);
			}else{
				System.out.println("\nRESPONSE CONTENT: \n" + "	" +  responseAsString);
			}
		} catch (Exception e) {
			e.printStackTrace();
		} finally{
			//release the connection
			if(response != null){
				try {
					EntityUtils.consume(response.getEntity());
				} catch (IOException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
			}
		}
	}


	private static void deleteUserByName(ERSClient ersClient, String name) {
		
		InternalUser user = searchUserByName(ersClient, name);
		
		if(user != null){
			deleteUserById(ersClient, user.getId());
		}else{
			System.out.println("could not retrieve user: " + name + "delete failed");
		}
		
	}

	private static void deleteUserById(ERSClient ersClient, String userid) {
		HttpResponse response = null;
		
		try {
			
			response = ersClient.deleteById("identity","internaluser",userid);
			RestUtils.printResponseHeaders(response);
			String responseAsString = RestUtils.getResponseContentAsString(response);
		
			
			
			System.out.println("\nRESPONSE CONTENT: \n" + "	" +  responseAsString);
			
			
			
			if (RestUtils.getResponseStatus(response) == HttpServletResponse.SC_NO_CONTENT) {
					
				System.out.println("user id " + userid + " deleted" );
				
			}else{
				System.out.println("failed to delete user id" + userid);
			}

		} catch (Exception e) {
			e.printStackTrace();
		} finally{
			//release the connection
			if(response.getEntity() != null){
				try {
					EntityUtils.consume(response.getEntity());
				} catch (IOException e) {
					// TODO Auto-generated catch block
					//e.printStackTrace();
				}
			}
		}
	
	}






	private static void updateUser(ERSClient ersClient, String userid) {
		
		HttpResponse response = null;
		InternalUser user = null;
		
		try {
			user = getUserByID(ersClient, userid);
			if(user != null){
				
				user.setEmail("updated@ers.sdk.com");
				user.setFirstName("updated");
				user.setLastName("updated");
				user.setDescription("updated");
				user.setEnablePassword("Passord123");
				
				JAXBElement<InternalUser> jaxbInternalUser = identityFactory.createInternaluser(user);
				
				String userAsXml = jaxbh.marshal(jaxbInternalUser);
				
				response = ersClient.update("identity","internaluser", userid, userAsXml);
				
				RestUtils.printResponseHeaders(response);
				String responseAsString = RestUtils.getResponseContentAsString(response);
				System.out.println("\nRESPONSE CONTENT: \n" + "	" + responseAsString);
				
				if (RestUtils.getResponseStatus(response) == HttpServletResponse.SC_OK) {
					
				
					UpdatedFieldsList updatedFields = (UpdatedFieldsList)RestUtils.fromXML(responseAsString, UpdatedFieldsList.class);
					List<UpdatedField> uflds = updatedFields.getUpdatedField();
					
					for(UpdatedField uf : uflds){
						
						System.out.println(uf.getField() + " field was updated: oldVal=" + uf.getOldValue() + " newVal=" + uf.getNewValue() + "\n");
					}
					
					
				}
			}
			
		} catch (Exception e) {
			e.printStackTrace();
		} finally{
			//release the connection
			if(response != null){
				try {
					EntityUtils.consume(response.getEntity());
				} catch (IOException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
			}
		}
	
	}



	/**
	 * 
	 * searchUserByName
	 * send search request with filter.
	 * filter operator can be: 
	 * EQ,NEQ,GT,LT,STARTSW,NSTARTSW,ENDSW,NENDSW,CONTAINS,NCONTAINS;
	 * @param ersClient
	 * @param name
	 * @return
	 */
	private static InternalUser searchUserByName(ERSClient ersClient, String name) {
		
		InternalUser user = null;
		List<BaseResource> baseResources  = getAllUsers(ersClient, false, "?filter=name.EQ." + name);
		
		if(baseResources != null && baseResources.size() == 1){
			user = getUserByID(ersClient, baseResources.get(0).getId()); 
		}
		return user;
	}






	private static InternalUser getUserByID(ERSClient ersClient, String userId) {
		
		HttpResponse response = null;
		InternalUser user = null;
		
		try {
			
			response = ersClient.getById("identity","internaluser",userId);
			RestUtils.printResponseHeaders(response);
			String responseAsString = RestUtils.getResponseContentAsString(response);
			System.out.println("\nRESPONSE CONTENT: \n" +  "	" + responseAsString);
		
			
			
			if (RestUtils.getResponseStatus(response) == HttpServletResponse.SC_OK) {
					
				//unmarshal the result from xml to java object
				user = (InternalUser)RestUtils.fromXML(responseAsString, InternalUser.class);
				
				if (user!=null){
					System.out.println("\n user " + user.getName() + " fetched!");
					System.out.println("\n user " + user.getName() + " email is: " + user.getEmail());
				}
				
			}

		} catch (Exception e) {
			e.printStackTrace();
		} finally{
			//release the connection
			if(response != null){
				try {
					EntityUtils.consume(response.getEntity());
				} catch (IOException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
			}
		}
		
		return user;
		
	}






	/**
	 * getAllUsers
	 * send to server GET request and retrieve all user currently in server (paged).
	 * @param ersClient
	 */
	private static List<BaseResource> getAllUsers(ERSClient ersClient, boolean verboseOn, String queryStringParams) {
		
		HttpResponse response = null;
		
		List<BaseResource> baseResources = null;  
		
		try {
		
			response = ersClient.getAll("identity","internaluser", queryStringParams);
			
			
			if(verboseOn){
				RestUtils.printResponseHeaders(response);
			}
			
			
			String searchResultAsString = RestUtils.getResponseContentAsString(response);
			
			if (RestUtils.getResponseStatus(response) == HttpServletResponse.SC_OK) {
				
				
				
				if(verboseOn){
					System.out.println("\nRESPONSE CONTENT: \n" + "	" +  searchResultAsString);
				}
				
					
				//unmarshal the result from xml to java object
				SearchResult sr = (SearchResult)RestUtils.fromXML(searchResultAsString, SearchResult.class);
				
				
				System.out.println("\n Total users found: " + sr.getTotal());
				
				baseResources =  sr.getResources().getResource();
				
				Iterator<BaseResource> it = baseResources.iterator();
				while (it.hasNext()) {
					BaseResource br = (BaseResource) it.next();
					if(verboseOn){
						System.out.println("\n	user (base resource) details: \n id=" + br.getId() + " \n name=" + br.getName() +
								" \n description=" + br.getDescription() + " \n link=" + br.getLink());
					}
				}
				
				
			}else{
				System.out.println("\nRESPONSE CONTENT: \n" + "	" +  searchResultAsString);
			}

		} catch (Exception e) {
			System.out.println("get-all failed: " + e.getMessage());;
		} finally{
			//release the connection
			if(response != null){
				try {
					EntityUtils.consume(response.getEntity());
				} catch (IOException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
			}
		}
		
		return baseResources;
	}

	
	

	
	/**
	 * create a user and return its id
	 * @param ersClient
	 * @param userName
	 * @param psw
	 * @return
	 */
	private static String createUser(ERSClient ersClient, String userName, String psw) {
		
		
		System.out.println("\n\n *** about to create new user: user name = " + userName);
		
		
		String newBornId = null;
		HttpResponse response = null;
		
		try {
		
			InternalUser user = new InternalUser();
			user.setName(userName);
			user.setDescription("sdk demo user");
			user.setEmail(userName + "@ers.sdkdemo.com");
			user.setEnabled(true);
			user.setPassword(psw);
			user.setFirstName("firstn");
			user.setLastName("lastn");
			user.setEnablePassword("Paswword123");
			user.setCustomAttributes(new CustomAttributes());
			user.setPasswordIDStore("00000000-0000-0000-0000-000000000000");
                        user.setExpiryDateEnabled(false);
                        user.setExpiryDate("2016-12-11");
	
			JAXBElement<InternalUser> jaxbInternalUser = identityFactory.createInternaluser(user);
			
			String userAsXml = jaxbh.marshal(jaxbInternalUser);
			
			
			response = ersClient.create("identity","internaluser", userAsXml);
			
			RestUtils.printResponseHeaders(response);
			
			if(RestUtils.getResponseStatus(response) == HttpServletResponse.SC_CREATED){
				
				newBornId = RestUtils.getNewBornId(response);
				
			}else{
				//error -> print response content
				
				System.out.println("\nRESPONSE CONTENT: \n" + "	" +  RestUtils.getResponseContentAsString(response));
				
			}
	
		} catch (Exception e) {
			e.printStackTrace();
		}finally{
			//release the connection
			if(response != null){
				try {
					EntityUtils.consume(response.getEntity());
				} catch (IOException e) {
					e.printStackTrace();
				}
			}
		}
		
		return newBornId;
	}

	
}
