
//**************************************************
//Copyright (c) 2016 Cisco Systems, Inc.
//All rights reserved.
//**************************************************

package com.cisco.ise.ers.demo;


import java.io.IOException;
import java.util.Iterator;
import java.util.List;

import javax.servlet.http.HttpServletResponse;
import javax.xml.bind.JAXBElement;

import org.apache.http.HttpResponse;

import com.cisco.ise.ers.BaseResource;
import com.cisco.ise.ers.v2.SearchResult;
import com.cisco.ise.ers.UpdatedField;
import com.cisco.ise.ers.UpdatedFieldsList;
import com.cisco.ise.ers.demo.ERSClient;
import com.cisco.ise.ers.demo.JAXBHelper;
import com.cisco.ise.ers.demo.RestUtils;
import com.cisco.ise.ers.network.AuthenticationSettings;
import com.cisco.ise.ers.network.NetworkDevice.NetworkDeviceGroupList;
import com.cisco.ise.ers.network.ObjectFactory;
import com.cisco.ise.ers.network.NetworkDevice;
import com.cisco.ise.ers.network.NetworkDeviceIP;

/**
* NetworkDeviceDemo
*
* This is as example for client code that uses ISE NetworkDevice Rest APIs to
* search NetworkDevice, create update and delete.
* The only ISE jars that it depends on is the ers-schema jar.
* 
* 
* This example runs the following Rest Interface calls: 
* 1. Get all NetworkDevice
* 2. Create 2 NetworkDevice
* 3. Get NetworkDevice by ID 
* 4. Get NetworkDevice by name (filter by name)
* 5. Update Network-Device 
* 6. Delete Network-Device by object ID
* 7. Delete Network-Device by name
*
* @since 1.3
*
*/

public class NetworkDeviceDemo {
	
	
	
  static ERSClient ersClient;
	
	static ObjectFactory deviceFactory = new ObjectFactory();
	
	
	static JAXBHelper jaxbh = new JAXBHelper(deviceFactory.getClass().getPackage().getName());
	
	/**
	 * expect 3 arguments: serverHostIp, Admin username of type ERS Admin, and its password
	String serverHost = null;
	 * Then it runs 
	 * @param args
	 * @throws Exception
	 */
	public static void main(String[] args) throws Exception {
		
		String serverHost = null;
		String userName = null;
		String password = null;
		
		if (args != null && args.length == 3) {

			serverHost = args[0]; // Server Host
			userName = args[1]; // Administrator name
			password = args[2]; // Administrator password
			
			System.out.println("\n ISE ip:" + serverHost + " user:" + userName + " psw:" + password);
			
			// Get an instance of Rest Client
			ERSClient ersClient = new ERSClient(userName, password, serverHost);
			
			
			
			System.out.println("\n\n\n\n\n\n" + "--------------------------------- START --------------------------------------------------");
			
			// Connectivity Check
			connectivityCheck(ersClient);
			
			// Demo approve
			DemoAppHelpers.demoApprove();
			
			// Get all Network Devices
			System.out.println("\n\n\n\n\n\n" + "----------- Get all Network Devices --------------------------------------------------------------");
			System.out.println("\n###############################################################" + 
			           		   "\n#  Get All Network Devices:                                   #" +
					 		   "\n#  This demo sends GET request to server to get all(paged)    #" +
			         		   "\n#  Network Devices. The expected response would be status 200 #" +				           		   
						       "\n#  first page of the network devices. The demo will also      #" +
			         		   "\n#  marshal the xml response content to java object of type    #" +
		            		   "\n#  search result and will print its attributes.               #" +
			           		   "\n###############################################################");
				
		    getAllNetworkDevice(ersClient, true, "?page=1&size=20");
		    
		    // create Network Devices
		    System.out.println("\n\n\n\n\n\n" + "----------- create Network Devices ---------------------------------------------------------------");
		    System.out.println("\n###############################################################" + 	
			               	   "\n#  Create Network Devices:                                    #" +
			             	   "\n#  This demo sends POST request to create new Network Device  #" +
			             	   "\n#  The expected response would be status 201 whereas the      #" +
			             	   "\n#  LOCATION header will hold the newborn Network Device ID.   #" +
				 			   "\n###############################################################");
		    String networkDevice1 = createNetworkDevice(ersClient, "sdk-demo-networkDevice", "10.10.10.0");
		    createNetworkDevice(ersClient, "sdk-demo-networkDevice2", "20.20.10.2");
	            createNetworkDevice(ersClient, "sdk-demo-networkDevice3", "110-200.202-300.301-321.2");		
                    createNetworkDevice(ersClient, "sdk-demo-networkDevice5", "*.*.*.*");
 		   
                     createNetworkDevice(ersClient, "sdk-demo-networkDevicerange", "30.30.10.2-10");
		 
		    // Get all Network Devices
		    System.out.println("\n\n\n\n\n\n" + "----------- Get all Network Devices --------------------------------------------------------------");
		    System.out.println("\n###############################################################" + 
		             		   "\n#  Get All Network Devices:                                   #" +
		             		   "\n#  Send GET All once more to retrieve and show the new        #" +
		            		   "\n#  Network Device                                             #" +
		            		   "\n###############################################################");
		    getAllNetworkDevice(ersClient, true, "?page=1&size=50");
		
		    //get specify Network Device by ID
		    System.out.println("\n\n\n\n\n\n" + "----------- get Network Device by ID -------------------------------------------------------------");
		    System.out.println("\n###############################################################" + 
		                       "\n#  Get Network Device by ID:                                  #" +
		            		   "\n#  This demo sends GET request to fetch Network Device ID.    #" +
		            		   "\n#  The expected response would be status 200 whereas the      #" +
		           		       "\n#  response content will hold the requested Network Device.   #" +
		           		       "\n#  The demo will also unmarshal the xml response content to   #" +
		           		       "\n#  java object of type Network Device and will print          #" +
		           		       "\n#  its attributes.                                            #" +
		           		       "\n###############################################################");
		    getNetworkDeviceByID(ersClient, networkDevice1);
		   
		    //search Network Device by name
		    System.out.println("\n\n\n\n\n\n" + "----------- search Network Device by name --------------------------------------------------------");
		    System.out.println("\n###############################################################" + 
		            		   "\n#  Get Network Device by name:                                #" +
		           		       "\n#  This demo sends GET request to server to get all(paged)    #" +
		           		       "\n#  Network Device filtered by name.                           #" +
		           		       "\n#  The expected response would be status 200                  #" +
		           		       "\n#  with Search Result as the reponse content, containing the  #" +
		           		       "\n#  filtered Network Device with name=sdk-demo-networkDevice.  #" +
		           		       "\n#  Then it marshals the xml response content to java object   #" + 
		           		       "\n#  of type SearchResult, and will print its attributes.       #" +
		           		       "\n###############################################################");
		    searchNetworkDeviceByName(ersClient, "sdk-demo-networkDevice");
		    
		    //update Network Device
		    System.out.println("\n\n\n\n\n\n" + "----------- update Network Device----------------------------------------------------------------");
		    System.out.println("\n###############################################################" + 
		             		   "\n#  Update Network Device:                                     #" +
		            		   "\n#  This demo sends PUT request to server,                     #" +
		            		   "\n#  for updating a Network Device.                             #" +
		            		   "\n#  The java Network Device object is marshaled to xml to      #" +
		            		   "\n#  become the request content.                                #" +
		            		   "\n#  The expected response would be status 200                  #" +
		            		   "\n#  with UpdatedFields as the reponse content which will be    #" +
		           	    	   "\n#  Then it marshals to java object of type UpdatedFields, and #" +
		           		       "\n#  print it to console .                                      #" +
		           		       "\n###############################################################");
		    updateNetworkDevice(ersClient, networkDevice1);
		
		   //delete Network Device by ID
		   System.out.println("\n\n\n\n\n\n" + "----------- delete Network Device by ID -----------------------------------------------------------");
		   System.out.println("\n###############################################################" + 
		           	   	      "\n#  Delete Network Device by ID:                               #" +
		           		      "\n#  This demo sends DELETE request to delete the Device by ID  #" +
		           		      "\n#  The expected response would be status 204 (no content)     #" +
		           		      "\n###############################################################");
		   deleteNetworkDeviceById(ersClient, networkDevice1);
		   
	       //delete Network Device by name
	       System.out.println("\n\n\n\n\n\n" + "----------- delete Network Device by name ---------------------------------------------------------");
		   System.out.println("\n###############################################################" + 
		                  	  "\n#  Delete Network Device by name:                             #" +
		           		      "\n#  This demo demonstrate how to delete a Device by its name.  #" +
		           		      "\n#  This is done by first searching it, then getting its ID    #" +
		           		      "\n#  and then delete by the ID.                                 #" +
		           		      "\n#  The expected response would be status 204 (no content)     #" +
		           		      "\n###############################################################");
		   deleteNetworkDeviceByName(ersClient, "sdk-demo-networkDevice2");
		
		   System.out.println("\n\n\n\n\n\n" + "--------------------------------- END ----------------------------------------------------");
		
		}else{
			System.out.println("\n illegal argument. \n syntax: \n InternalUserDemo.sh [host-ip] [user-name] [password]\n");
		}
	}

	/**
	 * delete Network Device by name
	 * @param ersClient
	 * @param name
	 */
	private static void deleteNetworkDeviceByName(ERSClient ersClient, String name) {

		NetworkDevice networkDevice = searchNetworkDeviceByName(ersClient, name);

		if(networkDevice != null){
			deleteNetworkDeviceById(ersClient, networkDevice.getId());
		}else{
			System.out.println("could not retrieve NetworkDevice: " + name + " delete failed");
		}

	}


	/**
	 * delete all Network Device in ISE DB
	 * @param ersClient
	 */
	private static void deleteAll(ERSClient ersClient) {

		List<BaseResource> baseResources = getAllNetworkDevice(ersClient, false, null);
		System.out.println("about to delete + " + baseResources.size() + " NetworkDevice" );
		for(BaseResource br : baseResources){
			deleteNetworkDeviceById(ersClient, br.getId());
		}

	}

	/**
	 * delete Network Device by ID
	 * @param ersClient
	 * @param NetworkDeviceId
	 */
	private static void deleteNetworkDeviceById(ERSClient ersClient, String NetworkDeviceId) {
		HttpResponse response = null;

		try {

			response = ersClient.deleteById("network","networkdevice",NetworkDeviceId);
			RestUtils.printResponseHeaders(response);
			String responseAsString = RestUtils.getResponseContentAsString(response);



			System.out.println("\nRESPONSE CONTENT: \n" + "	" +  responseAsString);



			if (RestUtils.getResponseStatus(response) == HttpServletResponse.SC_NO_CONTENT) {

				System.out.println("networkdevice ID " + NetworkDeviceId + " deleted" );

			}else{
				System.out.println("failed to delete networkdevice ID :" + NetworkDeviceId);
			}

		} catch (Exception e) {
			e.printStackTrace();
		} finally{
			//release the connection
			if(response.getEntity() != null){
				try {
					response.getEntity().consumeContent();
				} catch (IOException e) {
					// TODO Auto-generated catch block
					//e.printStackTrace();
				}
			}
		}
	}






	/**
	 * update Network Device
	 * @param ersClient
	 * @param deviceID
	 */
	private static void updateNetworkDevice(ERSClient ersClient, String deviceID) {

		HttpResponse response = null;
		NetworkDevice networkDevice = null;

		try {
			networkDevice = getNetworkDeviceByID(ersClient, deviceID);
			if(networkDevice != null){

				networkDevice.setName("updated-NetworkDevice");
				networkDevice.setDescription("updated");
				NetworkDeviceIP e = new NetworkDeviceIP();
				e.setIpaddress("10.10.10.10");
		                e.setIpaddressExclude("10.106.252.186");	
                        	e.setMask(10);
				networkDevice.getNetworkDeviceIPList().getNetworkDeviceIP().add(e);

				JAXBElement<NetworkDevice> jaxbInternalUser = deviceFactory.createNetworkdevice(networkDevice);

				String userAsXml = jaxbh.marshal(jaxbInternalUser);

				response = ersClient.update("network","networkdevice", deviceID, userAsXml);

				RestUtils.printResponseHeaders(response);
				String responseAsString = RestUtils.getResponseContentAsString(response);
				System.out.println("\nRESPONSE CONTENT: \n" + "	" + responseAsString);

				if (RestUtils.getResponseStatus(response) == HttpServletResponse.SC_OK) {


					UpdatedFieldsList updatedFields = (UpdatedFieldsList)RestUtils.fromXML(responseAsString, UpdatedFieldsList.class);
					List<UpdatedField> uflds = updatedFields.getUpdatedField();

					for(UpdatedField uf : uflds){

						System.out.println(uf.getField() + " field was updated: oldVal=" + uf.getOldValue() + " newVal=" + uf.getNewValue() + "\n");
					}
				}
			}

		} catch (Exception e) {
			e.printStackTrace();
		} finally{
			//release the connection
			if(response != null){
				try {
					response.getEntity().consumeContent();
				} catch (IOException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
			}
		}
	}



	
	
	
	/**
	 * 
	 * search Network Device by name
	 * send search request with filter.
	 * filter operator can be: 
	 * EQ,NEQ,GT,LT,STARTSW,NSTARTSW,ENDSW,NENDSW,CONTAINS,NCONTAINS;
	 * @param ersClient
	 * @param name
	 * @return
	 */
	private static NetworkDevice searchNetworkDeviceByName(ERSClient ersClient, String name) {

		NetworkDevice networkDevice = null;
		List<BaseResource> baseResources  = getAllNetworkDevice(ersClient, false, "?filter=name.EQ." + name);

		if(baseResources != null && baseResources.size() == 1){
			networkDevice = getNetworkDeviceByID(ersClient, baseResources.get(0).getId()); 
		}
		return networkDevice;
	}



	
	
	
	/**
	 * connectivity check
	 * send getVersion request to server in order to try connect
	 * and make sure the credentials are OK
	 * @param ersClient
	 */
	private static void connectivityCheck(ERSClient ersClient) {

		HttpResponse response = null;
		try {
			
          
			System.out.println("\n###############################################################" + 
			                   "\n#  Connectivity Check:                                        #" +
					           "\n#  About to send GET Network Device version request to server #" +
					           "\n#  in order to make sure that connection can be established.  #" +
					           "\n###############################################################");
          
			//getting NetworkDevice server version, just for checking that the  
			response = ersClient.getVersion("network","networkdevice");
			String responseAsString = RestUtils.getResponseContentAsString(response);
			int responseStatus = RestUtils.getResponseStatus(response);
			if (responseStatus == HttpServletResponse.SC_OK) {
				System.out.println("\nConnection to ISE server success");
			}else if(responseStatus == HttpServletResponse.SC_UNAUTHORIZED){
				System.out.println("\n###############################################################" + 
						           "\n#  401 - UNAUTHORIZED:                                        #" +
				            	   "\n#  Connection to ISE server failed !!!                        #" +
					               "\n#  Make sure you are using  valid Admin Credentials.          #" +
						           "\n#  Please note that the admin must have 'ERS-Admin' privilages#" +
				            	   "\n###############################################################");				
				System.exit(-1);
			}else{
				System.out.println("\nRESPONSE CONTENT: \n" + "	" +  responseAsString);
			}
		} catch (Exception e) {
			e.printStackTrace();
		} finally{
			//release the connection
			if(response != null){
				try {
					if (response.getEntity() != null) {
						response.getEntity().consumeContent();
					}
				} catch (IOException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
			}
		}	
	}




	
  /**
   * get all Network Device
   * @param ersClient
   * @param verboseOn
   * @param queryStringParams
   * @return
   */
	private static List<BaseResource> getAllNetworkDevice(ERSClient ersClient, boolean verboseOn, String queryStringParams) {

		HttpResponse response = null;

		List<BaseResource> baseResources = null;  

		try {

			response = ersClient.getAll("network","networkdevice", queryStringParams);


			if(verboseOn){
				RestUtils.printResponseHeaders(response);
			}

			String searchResultAsString = RestUtils.getResponseContentAsString(response);

			if (RestUtils.getResponseStatus(response) == HttpServletResponse.SC_OK) {
				
				
				
				if(verboseOn){
					System.out.println("\nRESPONSE CONTENT: \n" + "	" +  searchResultAsString);
				}


				//unmarshal the result from xml to java object
				SearchResult sr = (SearchResult)RestUtils.fromXML(searchResultAsString, SearchResult.class);

				System.out.println("\n Total networkdevice found: " + sr.getTotal());

				baseResources =  sr.getResources().getResource();

				Iterator<BaseResource> it = baseResources.iterator();
				while (it.hasNext()) {
					BaseResource br = (BaseResource) it.next();
					if(verboseOn){
						System.out.println("\n	networkdevice (base resource) details: \n ID=" + br.getId() + " \n name=" + br.getName() +
								" \n description=" + br.getDescription() + " \n link=" + br.getLink());
					}

				}


			}else{
				System.out.println("\nRESPONSE CONTENT: \n" + "	" +  searchResultAsString);
			}

		} catch (Exception e) {
			e.printStackTrace();
		} finally{
			//release the connection
			if(response != null){
				try {
					if (response.getEntity() != null) {
						response.getEntity().consumeContent();
					}
				} catch (IOException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
			}
		}

		return baseResources;
	}
	

	
	
	
	
	/**
	 * create an Network Device and return its id
	 * @param ersClient
	 * @param device name
	 * @param device IP
	 * @return
	 */
	private static String createNetworkDevice(ERSClient ersClient, String name, String IP) {

		System.out.println("\n\n *** about to create new NetworkDevice name: " + name);

		String newBornId = null;
		HttpResponse response = null;

		try {

			NetworkDevice networkDevice = new NetworkDevice();

			networkDevice.setName(name);
			networkDevice.setDescription("sdk demo networkDevice");

			NetworkDeviceGroupList NDGList = new NetworkDeviceGroupList();
			NDGList.getNetworkDeviceGroup().add("Location#All Locations");
			NDGList.getNetworkDeviceGroup().add("Device Type#All Device Types");
			networkDevice.setNetworkDeviceGroupList(NDGList);

			NetworkDeviceIP  newIp = new NetworkDeviceIP();
		

                	newIp.setIpaddress(IP);
                        newIp.setIpaddressExclude("10.106.252.185"); 
     	        	newIp.setMask(32);
			NetworkDevice.NetworkDeviceIPList ipList = new NetworkDevice.NetworkDeviceIPList();
			ipList.getNetworkDeviceIP().add(newIp);
			networkDevice.setNetworkDeviceIPList(ipList);

			AuthenticationSettings auth = new AuthenticationSettings();
			auth.setEnableKeyWrap(false);
			networkDevice.setAuthenticationSettings(auth);



			JAXBElement<NetworkDevice> jaxbNetworkDevice = deviceFactory.createNetworkdevice(networkDevice);

			String networkDeviceintAsXml = jaxbh.marshal(jaxbNetworkDevice);


			response = ersClient.create("network","networkdevice", networkDeviceintAsXml);

			RestUtils.printResponseHeaders(response);

			if(RestUtils.getResponseStatus(response) == HttpServletResponse.SC_CREATED){

				newBornId = RestUtils.getNewBornId(response);

			}else{
				//error -> print response content

				System.out.println("\nRESPONSE CONTENT: \n" + "	" +  RestUtils.getResponseContentAsString(response));

			}

		} catch (Exception e) {
			e.printStackTrace();
		}finally{
			//release the connection
			if(response != null){
				try {
					if (response.getEntity() != null) {
						response.getEntity().consumeContent();
					}
				} catch (IOException e) {
					e.printStackTrace();
				}
			}

		}

		return newBornId;		
	}

	/**
	 * get Network Device by ID
	 * @param ersClient
	 * @param NetworkDeviceId
	 * @return
	 */
	private static NetworkDevice getNetworkDeviceByID(ERSClient ersClient, String NetworkDeviceId) {

		HttpResponse response = null;
		NetworkDevice networkDevice = null;

		try {

			response = ersClient.getById("network","networkdevice",NetworkDeviceId);
			RestUtils.printResponseHeaders(response);
			String responseAsString = RestUtils.getResponseContentAsString(response);
			System.out.println("\nRESPONSE CONTENT: \n" +  "	" + responseAsString);



			if (RestUtils.getResponseStatus(response) == HttpServletResponse.SC_OK) {

				//unmarshal the result from xml to java object
				networkDevice = (NetworkDevice)RestUtils.fromXML(responseAsString, NetworkDevice.class);

				if (networkDevice!=null){
					System.out.println("\n networkDevice " + networkDevice.getName() + " fetched!");
					System.out.println("\n networkDevice " + networkDevice.getName() + " ID is: " + networkDevice.getId());
				}

			}

		} catch (Exception e) {
			e.printStackTrace();
		} finally{
			//release the connection
			if(response != null){
				try {
					response.getEntity().consumeContent();
				} catch (IOException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
			}
		}

		return networkDevice;

	}

}
