//**************************************************
// Copyright (c) 2013-2015 Cisco Systems, Inc.
// All rights reserved.
//**************************************************
package com.cisco.ise.ers.demo;


import java.io.ByteArrayInputStream;
import java.io.StringWriter;
import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBElement;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Marshaller;
import javax.xml.bind.Unmarshaller;
import javax.xml.transform.stream.StreamSource;
import org.apache.http.Header;
import org.apache.http.HttpResponse;
import org.apache.http.util.EntityUtils;


public class RestUtils {

	
	
	/**
	 * toXML
	 * marshaling: convert the given Object of the given object class
	 * to xml string and return it. 
	 * @param obj
	 * @param objClass
	 * @return String: the object in xml
	 * @throws JAXBException
	 */
	public static String toXML(Object obj, Class<?> objClass) throws JAXBException{
		
		JAXBContext jc = JAXBContext.newInstance(objClass);
		java.io.StringWriter sw = new StringWriter();
        Marshaller marshaller = jc.createMarshaller();
        marshaller.setProperty(Marshaller.JAXB_ENCODING, "UTF-8");
        marshaller.setProperty(Marshaller.JAXB_FORMATTED_OUTPUT, Boolean.TRUE);
        marshaller.marshal(obj, sw);
        
        return sw.toString();
		
	}
	
	
	
	/**
	 * fromXML
	 * unmarshaling: convert the given xml string to
	 * an object of the given object class and return it.  
	 * @param xmlObj
	 * @param objClass
	 * @return
	 * @throws JAXBException 
	 */
	public static Object fromXML(String xmlObj, Class<?> objClass) throws JAXBException{
		
		JAXBContext jc = JAXBContext.newInstance(objClass);
		
		Unmarshaller u = jc.createUnmarshaller();
		u.setEventHandler(new javax.xml.bind.helpers.DefaultValidationEventHandler());
		StreamSource sts = new StreamSource(new ByteArrayInputStream(xmlObj.getBytes()));
		
		JAXBElement<?> root = u.unmarshal(sts, objClass);
		
		Object objFromXML = root.getValue();
		
		return objFromXML;
		
	}
	
	
	
	public static StreamSource xmlStringToStream(String xmlString){
		return new StreamSource(new ByteArrayInputStream(xmlString.getBytes()));
	}
	
	
	
	public static int  getResponseStatus(HttpResponse response){
		
		System.out.println("\nRESPONSE STATUS: \n" + "	" + response.getStatusLine().toString() + "\n");
		
		return response.getStatusLine().getStatusCode();
		
	}
	
	
	
	

	public static String getResponseContentAsString(HttpResponse response) throws Exception {
		String responseContentAsString = "n/a";
		
		if(response.getEntity() != null){
			//this line is required in order to consume the response and free the connection.
			responseContentAsString = EntityUtils.toString(response.getEntity());
		}
		return responseContentAsString;
	}


	
	
	
	
	public static void printResponseHeaders(HttpResponse response){
		
		System.out.println("\nRESPONSE HEADERS:");
		
		Header headers[] = response.getAllHeaders();
		for(Header h:headers){
			System.out.println("	" + h.getName() + ": " + h.getValue());
		}
		
		System.out.println("\n");
	}
	
	
	public static void printRequestHeaders(Header headers[]){
		
		System.out.println("\nREQUEST HEADERS:");
		
		for(Header h:headers){
			System.out.println("	" + h.getName() + ": " + h.getValue());
		}
		
		System.out.println("\n");
	}
	
	
	
	
	public static String getNewBornId(HttpResponse response){
		
		
		String location = "";
		String newBornId;
		
		Header headers[] = response.getAllHeaders();
		for(Header h:headers){
			if(h.getName().equalsIgnoreCase("location")){
				location = h.getValue();
			}
			
		}
		String tokens[] = location.split("/");
		newBornId = tokens[tokens.length-1];
		System.out.println("\n new born id = " + newBornId );
		return newBornId;
	}
	
	
	
	

}
