//**************************************************
//Copyright (c) 2015 Cisco Systems, Inc.
//All rights reserved.
//**************************************************


package com.cisco.ise.ers.demo;


import java.io.IOException;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import javax.servlet.http.HttpServletResponse;
import javax.xml.bind.JAXBElement;

import org.apache.http.HttpResponse;


import com.cisco.ise.ers.BaseResource;
import com.cisco.ise.ers.v2.SearchResult;
import com.cisco.ise.ers.UpdatedField;
import com.cisco.ise.ers.UpdatedFieldsList;
import com.cisco.ise.ers.demo.ERSClient;
import com.cisco.ise.ers.demo.JAXBHelper;
import com.cisco.ise.ers.demo.RestUtils;
import com.cisco.ise.ers.trustsec.Sgacl;
import com.cisco.ise.ers.trustsec.ObjectFactory;

/**
 * SgaclDemo
 *
 * This is as example for client code that uses ISE NetworkDevice Rest APIs to
 * search Sgt, create update and delete.
 * The only ISE jars that it depends on is the ers-schema jar.
 * 
 * 
 * This example runs the following Rest Interface calls: 
 * 1. Get all Sgacl
 * 2. Create 3 Sgacl
 * 3. Get Sgacl by ID 
 * 4. Get Sgacl by name (filter by name)
 * 5. Update Sgacl 
 * 6. Delete Sgacl by object ID
 * 7. Delete Sgacl by name
 * 8. Delete all Sgacl 
 *
 * @since 1.5
 *
 */

public class SgaclDemo {

	static List<String> sgaclIdList = new ArrayList<String>();
	static ERSClient ersClient;

	static ObjectFactory deviceFactory = new ObjectFactory();


	static JAXBHelper jaxbh = new JAXBHelper(deviceFactory.getClass().getPackage().getName());
	
	/**
	 * expect 3 arguments: serverHostIp, Admin username of type ERS Admin, and its password
	String serverHost = null;
	 * Then it runs 
	 * @param args
	 * @throws Exception
	 */
	public static void main(String[] args) throws Exception {

		String serverHost = null;
		String userName = null;
		String password = null;


		if (args != null && args.length == 3) {

			serverHost = args[0]; // Server Host
			userName = args[1]; // Administrator name
			password = args[2]; // Administrator password

			System.out.println("\n ISE ip:" + serverHost + " user:" + userName + " psw:" + password);

			// Get an instance of Rest Client
			ERSClient ersClient = new ERSClient(userName, password, serverHost);



			System.out.println("\n\n\n\n\n\n" + "--------------------------------- START ----------------------------------------------");

			// Connectivity Check
			connectivityCheck(ersClient);

			DemoAppHelpers.demoApprove();

			// Get all Sgt
			System.out.println("\n\n\n\n\n\n" + "----------- Get all Sgacl --------------------------------------------------------------");
			System.out.println("\n###############################################################" + 
                               "\n#  Get All Sgacl:                                              #" +
                               "\n#  This demo sends GET request to server to get all(paged)     #" +
                               "\n#  Sgacl. The expected response would be status 200            #" +				           		   
                               "\n#  first page of the Sgacl. The demo will also marshal the xml #" +
                               "\n#  response content to java object of type search result and   #" +
                               "\n#  will print its attributes.                                  #" +
                               "\n###############################################################");

			getAllSgacl(ersClient, true, "?page=1&size=20");

			// create Sgacl
			System.out.println("\n\n\n\n\n\n" + "----------- create Sgacl ---------------------------------------------------------------");
			System.out.println("\n###############################################################" + 	
			                   "\n#  Create Sgacl:                                              #" +
			                   "\n#  This demo sends POST request to create new Sgacl           #" +
			                   "\n#  The expected response would be status 201 whereas the      #" +
			                   "\n#  LOCATION header will hold the newborn Sgt ID.              #" +
			                   "\n###############################################################");
			
			String sgacl1 = createSgacl(ersClient, "sdkDemoSgacl", "demo");
			sgaclIdList.add(sgacl1);
			sgaclIdList.add(createSgacl(ersClient, "sdkDemoSgcl1", "demo"));
			createSgacl(ersClient, "sdkDemoSgacl2", "demo");

			// Get all Sgacl
			System.out.println("\n\n\n\n\n\n" + "----------- Get all Sgacl --------------------------------------------------------------");
			System.out.println("\n###############################################################" + 
	                           "\n#  Get all Sgacl:                                             #" +
	                           "\n#  Send GET All once more to retrieve and show the new        #" +
	                           "\n#  Sgacl                                                      #" +
	                           "\n###############################################################");

			getAllSgacl(ersClient, true, "?page=1&size=50");
			//get specify Sgacl by ID
			System.out.println("\n\n\n\n\n\n" + "----------- get Sgacl by ID ------------------------------------------------------------");
			System.out.println("\n###############################################################" + 
                               "\n#  Get Sgacl by ID:                                           #" +
                               "\n#  This demo sends GET request to fetch Sgacl ID.             #" +
                               "\n#  The expected response would be status 200 whereas the      #" +
                               "\n#  response content will hold the requested Sgacl.            #" +
                               "\n#  The demo will also unmarshal the xml response content to   #" +
                               "\n#  java object of type Sgacl and will print its attributes.   #" +
                               "\n###############################################################");

			getSgaclByID(ersClient, sgacl1);

			//search Sgacl by name
			System.out.println("\n\n\n\n\n\n" + "----------- search Sgacl by name -------------------------------------------------------");
			System.out.println("\n###############################################################" + 
                               "\n#  Get Sgacl by name:                                         #" +
                               "\n#  This demo sends GET request to server to get all(paged)    #" +
                               "\n#  Sgacl filtered by name.                                    #" +
                               "\n#  The expected response would be status 200                  #" +
                               "\n#  with Search Result as the reponse content, containing the  #" +
                               "\n#  filtered Sgt with name=sdk-demo-Sgacl.                     #" +
                               "\n#  Then it marshals the xml response content to java object   #" + 
                               "\n#  of type SearchResult, and will print its attributes.       #" +
                               "\n###############################################################");

			searchSgaclByName(ersClient, "sdkDemoSgacl");

			//update Sgacl
			System.out.println("\n\n\n\n\n\n" + "----------- update Sgacl ----------------------------------------------------------------");
			System.out.println("\n###############################################################" + 
                               "\n#  Update Sgacl:                                              #" +
                               "\n#  This demo sends PUT request to server,                     #" +
                               "\n#  for updating a Sgacl.                                      #" +
                               "\n#  The java Sgacl object is marshaled to xml to become the    #" +
                               "\n#  request content.                                           #" +
                               "\n#  The expected response would be status 200                  #" +
                               "\n#  with UpdatedFields as the reponse content which will be    #" +
                               "\n#  Then it marshals to java object of type UpdatedFields, and #" +
                               "\n#  print it to console .                                      #" +
                               "\n###############################################################");

			updateSgacl(ersClient, sgacl1);

			//delete Sgacl by ID
			System.out.println("\n\n\n\n\n\n" + "----------- delete Sgacl by ID ------------------------------------------------------------");
			System.out.println("\n###############################################################" + 
                               "\n#  Delete Sgacl by ID:                                        #" +
                               "\n#  This demo sends DELETE request to delete the Sgacl by ID   #" +
                               "\n#  The expected response would be status 204 (no content)     #" +
                               "\n###############################################################");

			deleteSgaclById(ersClient, sgacl1);
			sgaclIdList.remove(sgacl1);

			//delete Sgacl by name
			System.out.println("\n\n\n\n\n\n" + "----------- delete Sgacl by name ------------------------------------------------------------");
			System.out.println("\n###############################################################" + 
                               "\n#  Delete Sgacl by name:                                      #" +
                               "\n#  This demo demonstrate how to delete a Sgacl by its name.   #" +
                               "\n#  This is done by first searching it, then getting its ID    #" +
                               "\n#  and then delete by the ID.                                 #" +
                               "\n#  The expected response would be status 204 (no content)     #" +
                               "\n###############################################################");
			deleteSgaclByName(ersClient, "sdkDemoSgacl2");
			

			//clean up Sgacls
			System.out.println("\n\n\n\n\n\n" + "----------- clean up Sgacls--------------------------------------------------------------------");
			System.out.println("\n###############################################################" + 
                               "\n#  clean up Sgacls  :                                         #" +
                               "\n#  This demo demonstrate how to delete all Sgacls.            #" +
                               "\n#  This is done by first getting all Sgt, then iterating      #" +
                               "\n#  over the SearchResult and  deleting one by one (by ID)     #" +
                               "\n###############################################################");
			deleteAll(ersClient);

			System.out.println("\n\n\n\n\n\n" + "--------------------------------- END ----------------------------------------------------");


		} else{
			System.out.println("\n illegal argument. \n required: \n run-ers-Sgacl-demo.sh [host-ip] [user-name] [password]\n");
		}
	}

	/**
	 * delete all Sgacl in ISE DB
	 * @param ersClient
	 */
	private static void deleteAll(ERSClient ersClient) {
		System.out.println("about to delete + " + sgaclIdList.size() + " Sgacl" );
		for(String sgtId : sgaclIdList){
			deleteSgaclById(ersClient, sgtId);
		}

	}

	/**
	 * delete Sgacl by name
	 * @param ersClient
	 * @param name
	 */
	private static void deleteSgaclByName(ERSClient ersClient, String name) {

		Sgacl sgacl = searchSgaclByName(ersClient, name);

		if(sgacl != null){
			deleteSgaclById(ersClient, sgacl.getId());
		}else{
			System.out.println("could not retrieve Sgt: " + name + " delete failed");
		}

	}

	/**
	 * connectivity check
	 * send getVersion request to server in order to try connect
	 * and make sure the credentials are OK
	 * @param ersClient
	 */
	private static void connectivityCheck(ERSClient ersClient) {

		HttpResponse response = null;
		try {


			System.out.println("\n###############################################################" + 
                               "\n#  Connectivity Check:                                        #" +
                               "\n#  About to send GET Sgt version request to server in order   #" +
                               "\n#  to make sure that connection can be established.           #" +
                               "\n###############################################################");

			//getting Sgt server version, just for checking that the  
			response = ersClient.getVersion("trustsec","sgt");
			String responseAsString = RestUtils.getResponseContentAsString(response);
			int responseStatus = RestUtils.getResponseStatus(response);
			if (responseStatus == HttpServletResponse.SC_OK) {
				System.out.println("\nConnection to ISE server success");
			}else if(responseStatus == HttpServletResponse.SC_UNAUTHORIZED){
				System.out.println("\n###############################################################" + 
                                   "\n#  401 - UNAUTHORIZED:                                        #" +
                                   "\n#  Connection to ISE server failed !!!                        #" +
                                   "\n#  Make sure you are using  valid Admin Credentials.          #" +
                                   "\n#  Please note that the admin must have 'ERS-Admin' privilages#" +
                                   "\n###############################################################");				
				System.exit(-1);
			}else{
				System.out.println("\nRESPONSE CONTENT: \n" + "	" +  responseAsString);
			}
		} catch (Exception e) {
			e.printStackTrace();
		} finally{
			//release the connection
			if(response != null){
				try {
					if (response.getEntity() != null) {
						response.getEntity().consumeContent();
					}
				} catch (IOException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
			}
		}	
	}

	/**
	 * get all Sgacl
	 * @param ersClient
	 * @param verboseOn
	 * @param queryStringParams
	 * @return
	 */
	private static List<BaseResource> getAllSgacl(ERSClient ersClient, boolean verboseOn, String queryStringParams) {

		HttpResponse response = null;

		List<BaseResource> baseResources = null;  

		try {

			response = ersClient.getAll("trustsec","sgacl", queryStringParams);


			if(verboseOn){
				RestUtils.printResponseHeaders(response);
			}

			String searchResultAsString = RestUtils.getResponseContentAsString(response);

			if (RestUtils.getResponseStatus(response) == HttpServletResponse.SC_OK) {

				if(verboseOn){
					System.out.println("\nRESPONSE CONTENT: \n" + "	" +  searchResultAsString);
				}


				//unmarshal the result from xml to java object
				SearchResult sr = (SearchResult)RestUtils.fromXML(searchResultAsString, SearchResult.class);

				System.out.println("\n Total Sgacl found: " + sr.getTotal());

				baseResources =  sr.getResources().getResource();

				Iterator<BaseResource> it = baseResources.iterator();
				while (it.hasNext()) {
					BaseResource br = (BaseResource) it.next();
					if(verboseOn){
						System.out.println("\n	Sgacl (base resource) details: \n ID=" + br.getId() + " \n name=" + br.getName() +
								" \n description=" + br.getDescription() + " \n link=" + br.getLink());
					}
				}
			}else{
				System.out.println("\nRESPONSE CONTENT: \n" + "	" +  searchResultAsString);
			}

		} catch (Exception e) {
			e.printStackTrace();
		} finally{
			//release the connection
			if(response != null){
				try {
					if (response.getEntity() != null) {
						response.getEntity().consumeContent();
					}
				} catch (IOException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
			}
		}

		return baseResources;
	}

	/**
	 * create an Sgacl and return its id
	 * @param ersClient
	 * @param Sgacl name
	 * @param Sgacl ipVersion
	 * @param Sgacl ACLcontent
	 * @return
	 */
	private static String createSgacl(ERSClient ersClient, String name, String aCLcontent) {

		System.out.println("\n\n *** about to create new Sgacl name: " + name);

		String newBornId = null;
		HttpResponse response = null;

		try {

			Sgacl sgacl = new Sgacl();

			sgacl.setName(name);
			sgacl.setDescription("sgacl demo Sgt");
			sgacl.setACLContent(aCLcontent);
			sgacl.setGenerationId("1");


			JAXBElement<Sgacl> jaxbNetworkDevice = deviceFactory.createSgacl(sgacl);

			String SgaclAsXml = jaxbh.marshal(jaxbNetworkDevice);


			response = ersClient.create("trustsec","sgacl", SgaclAsXml);

			RestUtils.printResponseHeaders(response);

			if(RestUtils.getResponseStatus(response) == HttpServletResponse.SC_CREATED){

				newBornId = RestUtils.getNewBornId(response);

			}else{
				//error -> print response content

				System.out.println("\nRESPONSE CONTENT: \n" + "	" +  RestUtils.getResponseContentAsString(response));

			}

		} catch (Exception e) {
			e.printStackTrace();
		}finally{
			//release the connection
			if(response != null){
				try {
					if (response.getEntity() != null) {
						response.getEntity().consumeContent();
					}
				} catch (IOException e) {
					e.printStackTrace();
				}
			}

		}

		return newBornId;		
	}

	/**
	 * get Sgacl by ID
	 * @param ersClient
	 * @param sgaclId
	 * @return
	 */
	private static Sgacl getSgaclByID(ERSClient ersClient, String sgaclId) {

		HttpResponse response = null;
		Sgacl sgacl = null;

		try {

			response = ersClient.getById("trustsec","sgacl",sgaclId);
			RestUtils.printResponseHeaders(response);
			String responseAsString = RestUtils.getResponseContentAsString(response);
			System.out.println("\nRESPONSE CONTENT: \n" +  "	" + responseAsString);



			if (RestUtils.getResponseStatus(response) == HttpServletResponse.SC_OK) {

				//unmarshal the result from xml to java object
				sgacl = (Sgacl)RestUtils.fromXML(responseAsString, Sgacl.class);

				if (sgacl!=null){
					System.out.println("\n Sgacl " + sgacl.getName() + " fetched!");
					System.out.println("\n Sgacl " + sgacl.getName() + " ID is: " + sgacl.getId());
				}

			}

		} catch (Exception e) {
			e.printStackTrace();
		} finally{
			//release the connection
			if(response != null){
				try {
					response.getEntity().consumeContent();
				} catch (IOException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
			}
		}

		return sgacl;

	}

	/**
	 * 
	 * search Sgacl by name
	 * send search request with filter.
	 * filter operator can be: 
	 * EQ,NEQ,GT,LT,STARTSW,NSTARTSW,ENDSW,NENDSW,CONTAINS,NCONTAINS;
	 * @param ersClient
	 * @param name
	 * @return
	 */
	private static Sgacl searchSgaclByName(ERSClient ersClient, String name) {

		Sgacl sgacl = null;
		List<BaseResource> baseResources  = getAllSgacl(ersClient, false, "?filter=name.EQ." + name);

		if(baseResources != null && baseResources.size() == 1){
			sgacl = getSgaclByID(ersClient, baseResources.get(0).getId()); 
		}
		return sgacl;
	}

	/**
	 * update Sgacl
	 * @param ersClient
	 * @param sgaclID
	 */
	private static void updateSgacl(ERSClient ersClient, String sgaclID) {

		HttpResponse response = null;
		Sgacl sgacl = null;

		try {

			sgacl = getSgaclByID(ersClient, sgaclID);

			if(sgacl != null){

				sgacl.setName("updatedSgacl");
				sgacl.setDescription("updated");
				sgacl.setACLContent("ACLContent updated");
				sgacl.setGenerationId("0");

				JAXBElement<Sgacl> jaxbInternalUser = deviceFactory.createSgacl(sgacl);

				String userAsXml = jaxbh.marshal(jaxbInternalUser);

				response = ersClient.update("trustsec","sgacl", sgaclID, userAsXml);

				RestUtils.printResponseHeaders(response);
				String responseAsString = RestUtils.getResponseContentAsString(response);
				System.out.println("\nRESPONSE CONTENT: \n" + "	" + responseAsString);

				if (RestUtils.getResponseStatus(response) == HttpServletResponse.SC_OK) {


					UpdatedFieldsList updatedFields = (UpdatedFieldsList)RestUtils.fromXML(responseAsString, UpdatedFieldsList.class);
					List<UpdatedField> uflds = updatedFields.getUpdatedField();

					for(UpdatedField uf : uflds){

						System.out.println(uf.getField() + " field was updated: oldVal=" + uf.getOldValue() + " newVal=" + uf.getNewValue() + "\n");
					}
				}
			}

		} catch (Exception e) {
			e.printStackTrace();
		} finally{
			//release the connection
			if(response != null){
				try {
					response.getEntity().consumeContent();
				} catch (IOException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
			}
		}
	}

	/**
	 * delete Sgacl by ID
	 * @param ersClient
	 * @param SgaclId
	 */
	private static void deleteSgaclById(ERSClient ersClient, String SgaclId) {
		HttpResponse response = null;

		try {

			response = ersClient.deleteById("trustsec","sgacl",SgaclId);
			RestUtils.printResponseHeaders(response);
			String responseAsString = RestUtils.getResponseContentAsString(response);

			System.out.println("\nRESPONSE CONTENT: \n" + "	" +  responseAsString);

			if (RestUtils.getResponseStatus(response) == HttpServletResponse.SC_NO_CONTENT) {

				System.out.println("Sgacl ID " + SgaclId + " deleted" );

			}else{
				System.out.println("failed to delete Sgacl ID :" + SgaclId);
			}

		} catch (Exception e) {
			e.printStackTrace();
		} finally{
			//release the connection
			if(response.getEntity() != null){
				try {
					response.getEntity().consumeContent();
				} catch (IOException e) {
					// TODO Auto-generated catch block
					//e.printStackTrace();
				}
			}
		}
	}

}
