
//**************************************************
//Copyright (c) 2015 Cisco Systems, Inc.
//All rights reserved.
//**************************************************

package com.cisco.ise.ers.demo;


import java.io.IOException;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import javax.servlet.http.HttpServletResponse;
import javax.xml.bind.JAXBElement;

import org.apache.http.HttpResponse;

import com.cisco.ise.ers.BaseResource;
import com.cisco.ise.ers.v2.SearchResult;
import com.cisco.ise.ers.UpdatedField;
import com.cisco.ise.ers.UpdatedFieldsList;
import com.cisco.ise.ers.demo.ERSClient;
import com.cisco.ise.ers.demo.JAXBHelper;
import com.cisco.ise.ers.demo.RestUtils;
import com.cisco.ise.ers.trustsec.Sgt;
import com.cisco.ise.ers.trustsec.ObjectFactory;

/**
* SgtDemo
*
* This is as example for client code that uses ISE NetworkDevice Rest APIs to
* search Sgt, create update and delete.
* The only ISE jars that it depends on is the ers-schema jar.
* 
* 
* This example runs the following Rest Interface calls: 
* 1. Get all Sgt
* 2. Create 3 Sgt
* 3. Get Sgt by ID 
* 4. Get Sgt by name (filter by name)
* 5. Update Sgt 
* 6. Delete Sgt by object ID
* 7. Delete Sgt by name
* 8. Delete all Sgt 
*
* @since 1.5
*
*/

public class SgtDemo {
	
	
	static List<String> sgtIdList = new ArrayList<String>();
	static ERSClient ersClient;
	
	static ObjectFactory deviceFactory = new ObjectFactory();
	
	static JAXBHelper jaxbh = new JAXBHelper(deviceFactory.getClass().getPackage().getName());
	
	/**
	 * expect 3 arguments: serverHostIp, Admin username of type ERS Admin, and its password
	String serverHost = null;
	 * Then it runs 
	 * @param args
	 * @throws Exception
	 */
	public static void main(String[] args) throws Exception {
		
		String serverHost = null;
		String userName = null;
		String password = null;
		
		
		if (args != null && args.length == 3) {

			serverHost = args[0]; // Server Host
			userName = args[1]; // Administrator name
			password = args[2]; // Administrator password
			
			System.out.println("\n ISE ip:" + serverHost + " user:" + userName + " psw:" + password);
			
			// Get an instance of Rest Client
			ERSClient ersClient = new ERSClient(userName, password, serverHost);
			
			
			
			System.out.println("\n\n\n\n\n\n" + "--------------------------------- START ----------------------------------------------");
			
			// Connectivity Check
			connectivityCheck(ersClient);
			
			DemoAppHelpers.demoApprove();
			
			// Get all Sgt
			System.out.println("\n\n\n\n\n\n" + "----------- Get all Sgt --------------------------------------------------------------");
			System.out.println("\n###############################################################" + 
			           		   "\n#  Get All Sgt:                                               #" +
					 		   "\n#  This demo sends GET request to server to get all(paged)    #" +
			         		   "\n#  Sgt. The expected response would be status 200             #" +				           		   
						       "\n#  first page of the Sgt. The demo will also marshal the xml  #" +
			         		   "\n#  response content to java object of type search result and  #" +
		            		   "\n#  will print its attributes.                                 #" +
			           		   "\n###############################################################");
				
		    getAllSgt(ersClient, true, "?page=1&size=20");
		    
		    // create Sgt
		    System.out.println("\n\n\n\n\n\n" + "----------- create Sgt ---------------------------------------------------------------");
		    System.out.println("\n###############################################################" + 	
			               	   "\n#  Create Sgt:                                                #" +
			             	   "\n#  This demo sends POST request to create new Sgt             #" +
			             	   "\n#  The expected response would be status 201 whereas the      #" +
			             	   "\n#  LOCATION header will hold the newborn Sgt ID.              #" +
				 			   "\n###############################################################");
		    String sgt1 = createSgt(ersClient, "sdkDemoSgt", 102);
		    sgtIdList.add(sgt1);
		    sgtIdList.add(createSgt(ersClient, "sdkDemoSgt1", 103));
		    sgtIdList.add(createSgt(ersClient, "sdkDemoSgt2", 104));
		 
		 
		    // Get all Sgt
		    System.out.println("\n\n\n\n\n\n" + "----------- Get all Sgt --------------------------------------------------------------");
		    System.out.println("\n###############################################################" + 
		             		   "\n#  Get all Sgt:                                               #" +
		             		   "\n#  Send GET All once more to retrieve and show the new        #" +
		            		   "\n#  Sgt                                                        #" +
		            		   "\n###############################################################");
		    getAllSgt(ersClient, true, "?page=1&size=50");
		
		    //get specify Sgt by ID
		    System.out.println("\n\n\n\n\n\n" + "----------- get sgt by ID ------------------------------------------------------------");
		    System.out.println("\n###############################################################" + 
		                       "\n#  Get Sgt by ID:                                             #" +
		            		   "\n#  This demo sends GET request to fetch Sgt ID.               #" +
		            		   "\n#  The expected response would be status 200 whereas the      #" +
		           		       "\n#  response content will hold the requested Sgt.              #" +
		           		       "\n#  The demo will also unmarshal the xml response content to   #" +
		           		       "\n#  java object of type Sgt and will print its attributes.     #" +
		           		       "\n###############################################################");
		    getSgtByID(ersClient, sgt1);
		   
		    //search Sgt by name
		    System.out.println("\n\n\n\n\n\n" + "----------- search Sgt by name -------------------------------------------------------");
		    System.out.println("\n###############################################################" + 
		            		   "\n#  Get Sgt by name:                                           #" +
		           		       "\n#  This demo sends GET request to server to get all(paged)    #" +
		           		       "\n#  Sgt filtered by name.                                      #" +
		           		       "\n#  The expected response would be status 200                  #" +
		           		       "\n#  with Search Result as the reponse content, containing the  #" +
		           		       "\n#  filtered Sgt with name=sdk-demo-Sgt.                       #" +
		           		       "\n#  Then it marshals the xml response content to java object   #" + 
		           		       "\n#  of type SearchResult, and will print its attributes.       #" +
		           		       "\n###############################################################");
		    searchSgtByName(ersClient, "sdkDemoSgt");
		    
		    //update Sgt
		    System.out.println("\n\n\n\n\n\n" + "----------- update Sgt ----------------------------------------------------------------");
		    System.out.println("\n###############################################################" + 
		             		   "\n#  Update Sgt:                                                #" +
		            		   "\n#  This demo sends PUT request to server,                     #" +
		            		   "\n#  for updating a Sgt.                                        #" +
		            		   "\n#  The java Sgt object is marshaled to xml to become the      #" +
		            		   "\n#  request content.  						                    #" +
		            		   "\n#  The expected response would be status 200                  #" +
		            		   "\n#  with UpdatedFields as the reponse content which will be    #" +
		           	    	   "\n#  Then it marshals to java object of type UpdatedFields, and #" +
		           		       "\n#  print it to console .                                      #" +
		           		       "\n###############################################################");
		    updateSgt(ersClient, sgt1);
		
		   //delete Sgt by ID
		   System.out.println("\n\n\n\n\n\n" + "----------- delete Sgt by ID -------------------------------------------------------------");
		   System.out.println("\n###############################################################" + 
		           	   	      "\n#  Delete Sgt by ID:                                          #" +
		           		      "\n#  This demo sends DELETE request to delete the Sgt by ID     #" +
		           		      "\n#  The expected response would be status 204 (no content)     #" +
		           		      "\n###############################################################");
		   deleteSgtById(ersClient, sgt1);
		   
	       //delete Sgt by name
	       System.out.println("\n\n\n\n\n\n" + "----------- delete Sgt by name --------------------------------------------------------------------");
		   System.out.println("\n###############################################################" + 
		                  	  "\n#  Delete Sgt by name:              			               #" +
		           		      "\n#  This demo demonstrate how to delete a Sgt by its name.     #" +
		           		      "\n#  This is done by first searching it, then getting its ID    #" +
		           		      "\n#  and then delete by the ID.                                 #" +
		           		      "\n#  The expected response would be status 204 (no content)     #" +
		           		      "\n###############################################################");
		   deleteSgtByName(ersClient, "sdkDemoSgt2");

		
		   System.out.println("\n\n\n\n\n\n" + "--------------------------------- END ----------------------------------------------------");
		
		}else{
			System.out.println("\n illegal argument. \n syntax: \n run-ers-Sgt-demo.sh [host-ip] [user-name] [password]\n");
		}
	}

	/**
	 * delete Sgt by name
	 * @param ersClient
	 * @param name
	 */
	private static void deleteSgtByName(ERSClient ersClient, String name) {

		Sgt sgt = searchSgtByName(ersClient, name);

		if(sgt != null){
			deleteSgtById(ersClient, sgt.getId());
		}else{
			System.out.println("could not retrieve Sgt: " + name + " delete failed");
		}

	}
	

	/**
	 * delete Sgt by ID
	 * @param ersClient
	 * @param SgtId
	 */
	private static void deleteSgtById(ERSClient ersClient, String SgtId) {
		HttpResponse response = null;

		try {

			response = ersClient.deleteById("trustsec","sgt",SgtId);
			RestUtils.printResponseHeaders(response);
			String responseAsString = RestUtils.getResponseContentAsString(response);



			System.out.println("\nRESPONSE CONTENT: \n" + "	" +  responseAsString);



			if (RestUtils.getResponseStatus(response) == HttpServletResponse.SC_NO_CONTENT) {

				System.out.println("Sgt ID " + SgtId + " deleted" );

			}else{
				System.out.println("failed to delete Sgt ID :" + SgtId);
			}

		} catch (Exception e) {
			e.printStackTrace();
		} finally{
			//release the connection
			if(response.getEntity() != null){
				try {
					response.getEntity().consumeContent();
				} catch (IOException e) {
					// TODO Auto-generated catch block
					//e.printStackTrace();
				}
			}
		}
	}

	/**
	 * update Sgt
	 * @param ersClient
	 * @param sgtId
	 */
	private static void updateSgt(ERSClient ersClient, String sgtId) {

		HttpResponse response = null;
		Sgt sgt = null;

		try {
			sgt = getSgtByID(ersClient, sgtId);
			if(sgt != null){

				sgt.setName("updatedSgt");
				sgt.setDescription("updated");
				sgt.setValue(105);

				JAXBElement<Sgt> jaxbInternalUser = deviceFactory.createSgt(sgt);

				String userAsXml = jaxbh.marshal(jaxbInternalUser);

				response = ersClient.update("trustsec","sgt", sgtId, userAsXml);

				RestUtils.printResponseHeaders(response);
				String responseAsString = RestUtils.getResponseContentAsString(response);
				System.out.println("\nRESPONSE CONTENT: \n" + "	" + responseAsString);

				if (RestUtils.getResponseStatus(response) == HttpServletResponse.SC_OK) {


					UpdatedFieldsList updatedFields = (UpdatedFieldsList)RestUtils.fromXML(responseAsString, UpdatedFieldsList.class);
					List<UpdatedField> uflds = updatedFields.getUpdatedField();

					for(UpdatedField uf : uflds){

						System.out.println(uf.getField() + " field was updated: oldVal=" + uf.getOldValue() + " newVal=" + uf.getNewValue() + "\n");
					}
				}
			}

		} catch (Exception e) {
			e.printStackTrace();
		} finally{
			//release the connection
			if(response != null){
				try {
					response.getEntity().consumeContent();
				} catch (IOException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
			}
		}
	}



	
	
	
	/**
	 * 
	 * search Sgt by name
	 * send search request with filter.
	 * filter operator can be: 
	 * EQ,NEQ,GT,LT,STARTSW,NSTARTSW,ENDSW,NENDSW,CONTAINS,NCONTAINS;
	 * @param ersClient
	 * @param name
	 * @return
	 */
	private static Sgt searchSgtByName(ERSClient ersClient, String name) {

		Sgt sgt = null;
		List<BaseResource> baseResources  = getAllSgt(ersClient, false, "?filter=name.EQ." + name);

		if(baseResources != null && baseResources.size() == 1){
			sgt = getSgtByID(ersClient, baseResources.get(0).getId()); 
		}
		return sgt;
	}



	
	
	
	/**
	 * connectivity check
	 * send getVersion request to server in order to try connect
	 * and make sure the credentials are OK
	 * @param ersClient
	 */
	private static void connectivityCheck(ERSClient ersClient) {

		HttpResponse response = null;
		try {
			
          
			System.out.println("\n###############################################################" + 
			                   "\n#  Connectivity Check:                                        #" +
					           "\n#  About to send GET Sgt version request to server in order   #" +
					           "\n#  to make sure that connection can be established.           #" +
					           "\n###############################################################");
          
			//getting Sgt server version, just for checking that the  
			response = ersClient.getVersion("trustsec","sgt");
			String responseAsString = RestUtils.getResponseContentAsString(response);
			int responseStatus = RestUtils.getResponseStatus(response);
			if (responseStatus == HttpServletResponse.SC_OK) {
				System.out.println("\nConnection to ISE server success");
			}else if(responseStatus == HttpServletResponse.SC_UNAUTHORIZED){
				System.out.println("\n###############################################################" + 
						           "\n#  401 - UNAUTHORIZED:                                        #" +
				            	   "\n#  Connection to ISE server failed !!!                        #" +
					               "\n#  Make sure you are using  valid Admin Credentials.          #" +
						           "\n#  Please note that the admin must have 'ERS-Admin' privilages#" +
				            	   "\n###############################################################");				
				System.exit(-1);
			}else{
				System.out.println("\nRESPONSE CONTENT: \n" + "	" +  responseAsString);
			}
		} catch (Exception e) {
			e.printStackTrace();
		} finally{
			//release the connection
			if(response != null){
				try {
					if (response.getEntity() != null) {
						response.getEntity().consumeContent();
					}
				} catch (IOException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
			}
		}	
	}




	
  /**
   * get all Sgt
   * @param ersClient
   * @param verboseOn
   * @param queryStringParams
   * @return
   */
	private static List<BaseResource> getAllSgt(ERSClient ersClient, boolean verboseOn, String queryStringParams) {

		HttpResponse response = null;

		List<BaseResource> baseResources = null;  

		try {

			response = ersClient.getAll("trustsec","sgt", queryStringParams);


			if(verboseOn){
				RestUtils.printResponseHeaders(response);
			}

			String searchResultAsString = RestUtils.getResponseContentAsString(response);

			if (RestUtils.getResponseStatus(response) == HttpServletResponse.SC_OK) {
				
				
				
				if(verboseOn){
					System.out.println("\nRESPONSE CONTENT: \n" + "	" +  searchResultAsString);
				}


				//unmarshal the result from xml to java object
				SearchResult sr = (SearchResult)RestUtils.fromXML(searchResultAsString, SearchResult.class);

				System.out.println("\n Total Sgt found: " + sr.getTotal());

				baseResources =  sr.getResources().getResource();

				Iterator<BaseResource> it = baseResources.iterator();
				while (it.hasNext()) {
					BaseResource br = (BaseResource) it.next();
					if(verboseOn){
						System.out.println("\n	Sgt (base resource) details: \n ID=" + br.getId() + " \n name=" + br.getName() +
								" \n description=" + br.getDescription() + " \n link=" + br.getLink());
					}

				}


			}else{
				System.out.println("\nRESPONSE CONTENT: \n" + "	" +  searchResultAsString);
			}

		} catch (Exception e) {
			e.printStackTrace();
		} finally{
			//release the connection
			if(response != null){
				try {
					if (response.getEntity() != null) {
						response.getEntity().consumeContent();
					}
				} catch (IOException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
			}
		}

		return baseResources;
	}
	

	
	
	
	
	/**
	 * create an Sgt and return its id
	 * @param ersClient
	 * @param Sgt name
	 * @param Sgt value
	 * @return
	 */
	private static String createSgt(ERSClient ersClient, String name, int value) {

		System.out.println("\n\n *** about to create new Sgt name: " + name);

		String newBornId = null;
		HttpResponse response = null;

		try {

			Sgt sgt = new Sgt();

			sgt.setName(name);
			sgt.setDescription("sdk demo Sgt");
			sgt.setValue(value);

			JAXBElement<Sgt> jaxbNetworkDevice = deviceFactory.createSgt(sgt);

			String SgtAsXml = jaxbh.marshal(jaxbNetworkDevice);


			response = ersClient.create("trustsec","sgt", SgtAsXml);

			RestUtils.printResponseHeaders(response);

			if(RestUtils.getResponseStatus(response) == HttpServletResponse.SC_CREATED){

				newBornId = RestUtils.getNewBornId(response);

			}else{
				//error -> print response content

				System.out.println("\nRESPONSE CONTENT: \n" + "	" +  RestUtils.getResponseContentAsString(response));

			}

		} catch (Exception e) {
			e.printStackTrace();
		}finally{
			//release the connection
			if(response != null){
				try {
					if (response.getEntity() != null) {
						response.getEntity().consumeContent();
					}
				} catch (IOException e) {
					e.printStackTrace();
				}
			}

		}

		return newBornId;		
	}

	/**
	 * get Sgt by ID
	 * @param ersClient
	 * @param sgtId
	 * @return
	 */
	private static Sgt getSgtByID(ERSClient ersClient, String sgtId) {

		HttpResponse response = null;
		Sgt sgt = null;

		try {

			response = ersClient.getById("trustsec","sgt",sgtId);
			RestUtils.printResponseHeaders(response);
			String responseAsString = RestUtils.getResponseContentAsString(response);
			System.out.println("\nRESPONSE CONTENT: \n" +  "	" + responseAsString);



			if (RestUtils.getResponseStatus(response) == HttpServletResponse.SC_OK) {

				//unmarshal the result from xml to java object
				sgt = (Sgt)RestUtils.fromXML(responseAsString, Sgt.class);

				if (sgt!=null){
					System.out.println("\n Sgt " + sgt.getName() + " fetched!");
					System.out.println("\n Sgt " + sgt.getName() + " ID is: " + sgt.getId());
				}

			}

		} catch (Exception e) {
			e.printStackTrace();
		} finally{
			//release the connection
			if(response != null){
				try {
					response.getEntity().consumeContent();
				} catch (IOException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
			}
		}

		return sgt;

	}
}
