//**************************************************
//Copyright (c) 2015 Cisco Systems, Inc.
//All rights reserved.
//**************************************************

package com.cisco.ise.ers.demo;

import java.io.IOException;
import java.util.Iterator;
import java.util.List;

import javax.servlet.http.HttpServletResponse;
import javax.xml.bind.JAXBElement;

import org.apache.http.HttpResponse;

import com.cisco.ise.ers.BaseResource;
import com.cisco.ise.ers.v2.SearchResult;
import com.cisco.ise.ers.UpdatedField;
import com.cisco.ise.ers.UpdatedFieldsList;
import com.cisco.ise.ers.demo.ERSClient;
import com.cisco.ise.ers.demo.JAXBHelper;
import com.cisco.ise.ers.demo.RestUtils;
import com.cisco.ise.ers.sxp.ErsSxpConnection;
import com.cisco.ise.ers.sxp.ObjectFactory;

public class SxpConnectionsDemo {

	static ERSClient ersClient;

	static ObjectFactory connectionFactory = new ObjectFactory();

	static JAXBHelper jaxbh = new JAXBHelper(connectionFactory.getClass().getPackage().getName());

	/**
	 * expect 3 arguments: serverHostIp, Admin username of type ERS Admin, and
	 * its password String serverHost = null; Then it runs
	 * 
	 * @param args
	 * @throws Exception
	 */
	public static void main(String[] args) throws Exception {

		String serverHost = null;
		String userName = null;
		String password = null;
		String sxpN = null;
		String sxpVpn = null;

		if (args != null && args.length == 5) {

			serverHost = args[0]; // Server Host
			userName = args[1]; // Administrator name
			password = args[2]; // Administrator password
			
			sxpN = args[3]; // Sxp Node Name
			sxpVpn = args[4]; // Sxp Vpn Name
			
			System.out.println("\n ISE ip:" + serverHost + " user:" + userName	+ " psw:" + password +" Sxp Node: " + sxpN + " Sxp Vpn:" + sxpVpn);

			// Get an instance of Rest Client
			ERSClient ersClient = new ERSClient(userName, password, serverHost);

			System.out.println("\n\n\n\n\n\n--------------------------------- START --------------------------------------------------");

			// Connectivity Check
			connectivityCheck(ersClient);

			// create Network Devices
			
			ErsSxpConnection ersSxpConnection = new ErsSxpConnection();

			ersSxpConnection.setSxpPeer("ERS-SxpPeer-demo1");
			ersSxpConnection.setIpAddress("1.20.3.50");
	        ersSxpConnection.setSxpVpn(sxpVpn);
	        ersSxpConnection.setSxpNode(sxpN);   
			ersSxpConnection.setSxpMode("SPEAKER");
			ersSxpConnection.setSxpVersion("VERSION_4");
			ersSxpConnection.setEnabled(true);
			//ersSxpConnection.setSxpDescription("Connection Demo");
			ersSxpConnection.setDescription("Sxp Connection Demo");
	        
			System.out.println("\n\n\n\n\n\n----------- create Sxp Connections ---------------------------------------------------------------");
			System.out.println("\n###############################################################"
							+ "\n#  Create Sxp Connections:                                          #"
							+ "\n#  This demo sends POST request to create new Sxp Connection        #"
							+ "\n#  The expected response would be status 201 whereas the            #"
							+ "\n#  LOCATION header will hold the newborn Sxp Connection ID.         #"
							+ "\n###############################################################");
			String sxpConn1 = createSxpConnection(ersClient, ersSxpConnection);
			
			// Get all Sxp Connections
			System.out.println("\n\n\n\n\n\n" + "----------- Get all Sxp Connections --------------------------------------------------------------");
			System.out.println("\n###############################################################" + 
			           		   "\n#  Get All Sxp Connections:                                   #" +
					 		   "\n#  This demo sends GET request to server to get all(paged)    #" +
			         		   "\n#  Sxp Connections. The expected response would be status 200 #" +				           		   
						       "\n#  first page of the Sxp Connections. The demo will also      #" +
			         		   "\n#  marshal the xml response content to java object of type    #" +
		            		   "\n#  search result and will print its attributes.               #" +
			           		   "\n###############################################################");
				
			getAllSxpConnections(ersClient, true, "?page=1&size=20");
					    
		   //get specify Sxp Peer ID
		    System.out.println("\n\n\n\n\n\n" + "----------- get Sxp Connection by ID -------------------------------------------------------------");
		    System.out.println("\n###############################################################" + 
		                       "\n#  Get Sxp Peer by ID:                                        #" +
		            		   "\n#  This demo sends GET request to fetch Sxp Peer ID.          #" +
		            		   "\n#  The expected response would be status 200 whereas the      #" +
		           		       "\n#  response content will hold the requested Network Device.   #" +
		           		       "\n#  The demo will also unmarshal the xml response content to   #" +
		           		       "\n#  java object of type Network Device and will print          #" +
		           		       "\n#  its attributes.                                            #" +
		           		       "\n###############################################################");
		    getSxpConnectionByID(ersClient, sxpConn1);
		   
		  //update Network Device
		    System.out.println("\n\n\n\n\n\n" + "----------- update Sxp Connection ----------------------------------------------------------------");
		    System.out.println("\n###############################################################" + 
		             		   "\n#  Update Sxp Connection:                                     #" +
		            		   "\n#  This demo sends PUT request to server,                     #" +
		            		   "\n#  for updating a Sxp Connection.                             #" +
		            		   "\n#  The java Sxp Connection object is marshaled to xml to      #" +
		            		   "\n#  become the request content.                                #" +
		            		   "\n#  The expected response would be status 200                  #" +
		            		   "\n#  with UpdatedFields as the reponse content which will be    #" +
		           	    	   "\n#  Then it marshals to java object of type UpdatedFields, and #" +
		           		       "\n#  print it to console .                                      #" +
		           		       "\n###############################################################");
		    updateSxpConnection(ersClient, sxpConn1);
		
		   //delete SxpPeer by ID
		   System.out.println("\n\n\n\n\n\n" + "----------- delete Sxp Connection by ID -----------------------------------------------------------");
		   System.out.println("\n#######################################################################" + 
		           	   	      "\n#  Delete Sxp Connection by ID:                                       #" +
		           		      "\n#  This demo sends DELETE request to delete the Sxp Connection by ID  #" +
		           		      "\n#  The expected response would be status 204 (no content)             #" +
		           		      "\n#######################################################################");
		   deleteSxpConnectionById(ersClient, sxpConn1);
					   
		   System.out.println("\n\n\n\n\n\n--------------------------------- END ----------------------------------------------------");

		} else {
			System.out.println("\n illegal argument. \n syntax: \n [host-ip] [user-name] [password] [sxp-node] [sxp-vpn]\n");
		}

	}

	/**
	 * connectivity check send getVersion request to server in order to try
	 * connect and make sure the credentials are OK
	 * 
	 * @param ersClient
	 */
	private static void connectivityCheck(ERSClient ersClient) {

		HttpResponse response = null;
		try {

			System.out.println("\n###############################################################"
							+ "\n#  Connectivity Check:                                        #"
							+ "\n#  About to send GET Sxp Connection version request to server #"
							+ "\n#  in order to make sure that connection can be established.  #"
							+ "\n###############################################################");

			// getting SxpPeer server version, just for checking that the
			response = ersClient.getVersion("sxp", "sxpconnections");
			String responseAsString = RestUtils.getResponseContentAsString(response);
			int responseStatus = RestUtils.getResponseStatus(response);
			if (responseStatus == HttpServletResponse.SC_OK) {
				System.out.println("\nConnection to ISE server success");
			} else if (responseStatus == HttpServletResponse.SC_UNAUTHORIZED) {
				System.out.println("\n###############################################################"
								+ "\n#  401 - UNAUTHORIZED:                                        #"
								+ "\n#  Connection to ISE server failed !!!                        #"
								+ "\n#  Make sure you are using  valid Admin Credentials.          #"
								+ "\n#  Please note that the admin must have 'ERS-Admin' privilages#"
								+ "\n###############################################################");
				System.exit(-1);
			} else {
				System.out.println("\nRESPONSE CONTENT: \n" + "	"+ responseAsString);
			}
		} catch (Exception e) {
			e.printStackTrace();
		} finally {
			// release the connection
			if (response != null) {
				try {
					if (response.getEntity() != null) {
						response.getEntity().consumeContent();
					}
				} catch (IOException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
			}
		}
	}
	
	
	private static String createSxpConnection(ERSClient ersClient, ErsSxpConnection ersSxpConnection) {

		System.out.println("\n\n *** about to create new Sxp Connection...");

		String newBornId = null;
		HttpResponse response = null;

		try {

			JAXBElement<ErsSxpConnection> jaxbSxpConnection = connectionFactory.createErsSxpConnection(ersSxpConnection);

			String sxpConnectionintAsXml = jaxbh.marshal(jaxbSxpConnection);

			System.out.println("Request Xml...");
			System.out.println(sxpConnectionintAsXml);
			System.out.println("");
			response = ersClient.create("sxp", "sxpconnections", sxpConnectionintAsXml);

			RestUtils.printResponseHeaders(response);

			if (RestUtils.getResponseStatus(response) == HttpServletResponse.SC_CREATED) {

				newBornId = RestUtils.getNewBornId(response);

			} else {
				// error -> print response content

				System.out.println("\nRESPONSE CONTENT: \n" + " " + RestUtils.getResponseContentAsString(response));

			}

		} catch (Exception e) {
			e.printStackTrace();
		} finally {
			// release the connection
			if (response != null) {
				try {
					if (response.getEntity() != null) {
						response.getEntity().consumeContent();
					}
				} catch (IOException e) {
					e.printStackTrace();
				}
			}

		}

		return newBornId;
	}
	
	private static List<BaseResource> getAllSxpConnections(ERSClient ersClient, boolean verboseOn, String queryStringParams) {

		HttpResponse response = null;

		List<BaseResource> baseResources = null;  

		try {

			response = ersClient.getAll("sxp","sxpconnections", queryStringParams);


			if(verboseOn){
				RestUtils.printResponseHeaders(response);
			}

			String searchResultAsString = RestUtils.getResponseContentAsString(response);

			if (RestUtils.getResponseStatus(response) == HttpServletResponse.SC_OK) {
				
				
				
				if(verboseOn){
					System.out.println("\nRESPONSE CONTENT: \n" + "	" +  searchResultAsString);
				}


				//unmarshal the result from xml to java object
				SearchResult sr = (SearchResult)RestUtils.fromXML(searchResultAsString, SearchResult.class);

				System.out.println("\n Total Sxp Connections found: " + sr.getTotal());

				baseResources =  sr.getResources().getResource();

				Iterator<BaseResource> it = baseResources.iterator();
				while (it.hasNext()) {
					BaseResource br = (BaseResource) it.next();
					if(verboseOn){
						System.out.println("\n	Sxp Connections details: \n Id =" + br.getId() + " \n name=" + br.getName() +
								" \n description=" + br.getDescription() + " \n link=" + br.getLink());
					}

				}

			}else{
				System.out.println("\nRESPONSE CONTENT: \n" + "	" +  searchResultAsString);
			}

		} catch (Exception e) {
			e.printStackTrace();
		} finally{
			//release the connection
			if(response != null){
				try {
					if (response.getEntity() != null) {
						response.getEntity().consumeContent();
					}
				} catch (IOException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
			}
		}

		return baseResources;
	}
	
	private static ErsSxpConnection getSxpConnectionByID(ERSClient ersClient, String sxpConnectionId) {

		HttpResponse response = null;
		ErsSxpConnection ersSxpConnection = null;

		try {
			System.out.println("Get Sxp Connection By Id : " + sxpConnectionId);
			response = ersClient.getById("sxp","sxpconnections",sxpConnectionId);
			RestUtils.printResponseHeaders(response);
			String responseAsString = RestUtils.getResponseContentAsString(response);
			System.out.println("\nRESPONSE CONTENT: \n" +  "	" + responseAsString);



			if (RestUtils.getResponseStatus(response) == HttpServletResponse.SC_OK) {

				//unmarshal the result from xml to java object
				ersSxpConnection = (ErsSxpConnection)RestUtils.fromXML(responseAsString, ErsSxpConnection.class);

				if (ersSxpConnection!=null){
					System.out.println("\n Retrived Sxp Connection, Id " + ersSxpConnection.getId() + " ,Sxp Peer : " + ersSxpConnection.getSxpPeer()+ " , Sxp Node : " + ersSxpConnection.getSxpNode() + ", Sxp Vpn :" + ersSxpConnection.getSxpVpn());
				}
			}
		} catch (Exception e) {
			e.printStackTrace();
		} finally{
			//release the connection
			if(response != null){
				try {
					response.getEntity().consumeContent();
				} catch (IOException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
			}
		}

		return ersSxpConnection;
	}
	
	private static void deleteSxpConnectionById(ERSClient ersClient, String sxpConnectionId) {
		HttpResponse response = null;

		try {

			response = ersClient.deleteById("sxp","sxpconnections",sxpConnectionId);
			RestUtils.printResponseHeaders(response);
			String responseAsString = RestUtils.getResponseContentAsString(response);


			System.out.println("\nRESPONSE CONTENT: \n" + "	" +  responseAsString);


			if (RestUtils.getResponseStatus(response) == HttpServletResponse.SC_NO_CONTENT) {

				System.out.println("Sxp Connection ID " + sxpConnectionId + " deleted" );

			}else{
				System.out.println("failed to delete Sxp Connection ID :" + sxpConnectionId);
			}

		} catch (Exception e) {
			e.printStackTrace();
		} finally{
			//release the connection
			if(response.getEntity() != null){
				try {
					response.getEntity().consumeContent();
				} catch (IOException e) {
					// TODO Auto-generated catch block
					//e.printStackTrace();
				}
			}
		}
	}
	
	private static void updateSxpConnection(ERSClient ersClient, String sxpConnectionId) {

		HttpResponse response = null;
		ErsSxpConnection ersSxpConnection = new ErsSxpConnection();

		try {
			ersSxpConnection = getSxpConnectionByID(ersClient, sxpConnectionId);
			if(ersSxpConnection != null){

				ersSxpConnection.setSxpMode("BOTH");
				ersSxpConnection.setSxpVersion("VERSION_3");
				ersSxpConnection.setDescription("Sxp Connection Demo Updated");
				
				JAXBElement<ErsSxpConnection> jaxbInternalUser = connectionFactory.createErsSxpConnection(ersSxpConnection);

				String userAsXml = jaxbh.marshal(jaxbInternalUser);

				response = ersClient.update("sxp","sxpconnections", sxpConnectionId, userAsXml);

				RestUtils.printResponseHeaders(response);
				String responseAsString = RestUtils.getResponseContentAsString(response);
				System.out.println("\nRESPONSE CONTENT: \n" + "	" + responseAsString);

				if (RestUtils.getResponseStatus(response) == HttpServletResponse.SC_OK) {


					UpdatedFieldsList updatedFields = (UpdatedFieldsList)RestUtils.fromXML(responseAsString, UpdatedFieldsList.class);
					List<UpdatedField> uflds = updatedFields.getUpdatedField();

					for(UpdatedField uf : uflds){

						System.out.println(uf.getField() + " field was updated: oldVal=" + uf.getOldValue() + " newVal=" + uf.getNewValue() + "\n");
					}
				}
			}

		} catch (Exception e) {
			e.printStackTrace();
		} finally{
			//release the connection
			if(response != null){
				try {
					response.getEntity().consumeContent();
				} catch (IOException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
			}
		}
	}
}
