//**************************************************
//Copyright (c) 2015 Cisco Systems, Inc.
//All rights reserved.
//**************************************************

package com.cisco.ise.ers.demo;

import java.io.IOException;
import java.util.Iterator;
import java.util.List;

import javax.servlet.http.HttpServletResponse;
import javax.xml.bind.JAXBElement;

import org.apache.http.HttpResponse;

import com.cisco.ise.ers.BaseResource;
import com.cisco.ise.ers.v2.SearchResult;
import com.cisco.ise.ers.UpdatedField;
import com.cisco.ise.ers.UpdatedFieldsList;
import com.cisco.ise.ers.demo.ERSClient;
import com.cisco.ise.ers.demo.JAXBHelper;
import com.cisco.ise.ers.demo.RestUtils;
import com.cisco.ise.ers.sxp.ErsSxpLocalBindings;
import com.cisco.ise.ers.sxp.ObjectFactory;


public class SxpLocalBindingsDemo {

	static ERSClient ersClient;

	static ObjectFactory localBindingFactory = new ObjectFactory();

	static JAXBHelper jaxbh = new JAXBHelper(localBindingFactory.getClass().getPackage().getName());

	/**
	 * expect 3 arguments: serverHostIp, Admin username of type ERS Admin, and
	 * its password String serverHost = null; Then it runs
	 * 
	 * @param args
	 * @throws Exception
	 */
	public static void main(String[] args) throws Exception {

		String serverHost = null;
		String userName = null;
		String password = null;
		String sxpVpn = null;
		String sgtId = null;
		
		if (args != null && args.length == 5) {

			serverHost = args[0]; // Server Host
			userName = args[1]; // Administrator name
			password = args[2]; // Administrator password
			sxpVpn = args[3]; // Sxp Vpn
			sgtId = args[4]; // Sgt Id

			System.out.println("\n ISE ip:" + serverHost + " user:" + userName + " psw:" + password + " Sxp Vpn:" +sxpVpn + " SGT Id : " + sgtId );

			// Get an instance of Rest Client
			ERSClient ersClient = new ERSClient(userName, password, serverHost);

			System.out.println("\n\n\n\n\n\n--------------------------------- START --------------------------------------------------");

			// Connectivity Check
			connectivityCheck(ersClient);

			// create Network Devices
			System.out.println("\n\n\n\n\n\n----------- create Sxp Local Bindins ---------------------------------------------------------------");
			System.out.println("\n########################################################################" 
							+ "\n#  Create Sxp Local Bindings:                                           #"
							+ "\n#  This demo sends POST request to create new Sxp Local Bindings        #"
							+ "\n#  The expected response would be status 201 whereas the                #"
							+ "\n#  LOCATION header will hold the newborn Sxp Peer ID.                   #"
							+ "\n#########################################################################");
			String sxpLocalBinding1 = createSxpLocalBinding(ersClient, "127.0.0.1/32",sgtId,sxpVpn);
			
			// Get all SxpLocalBindings
			System.out.println("\n\n\n\n\n\n" + "----------- Get all Sxp Local Bindings--------------------------------------------------------------");
			System.out.println("\n###############################################################" + 
			           		   "\n#  Get All Sxp Local Bindings:                                #" +
					 		   "\n#  This demo sends GET request to server to get all(paged)    #" +
			         		   "\n#  Sxp Peers. The expected response would be status 200       #" +				           		   
						       "\n#  first page of the Sxp Peers. The demo will also            #" +
			         		   "\n#  marshal the xml response content to java object of type    #" +
		            		   "\n#  search result and will print its attributes.               #" +
			           		   "\n###############################################################");
				
		    getAllSxpLocalBindings(ersClient, true, "?page=1&size=20");
					    
		   //get specify Sxp Local Binding ID
		    System.out.println("\n\n\n\n\n\n" + "----------- get Sxp Local Binding by ID -------------------------------------------------------------");
		    System.out.println("\n###############################################################" + 
		                       "\n#  Get Sxp Local Binding by ID:                               #" +
		            		   "\n#  This demo sends GET request to fetch Sxp Local Binding ID. #" +
		            		   "\n#  The expected response would be status 200 whereas the      #" +
		           		       "\n#  response content will hold the requested Network Device.   #" +
		           		       "\n#  The demo will also unmarshal the xml response content to   #" +
		           		       "\n#  java object of type Network Device and will print          #" +
		           		       "\n#  its attributes.                                            #" +
		           		       "\n###############################################################");
		   getSxpLocalBindingByID(ersClient, sxpLocalBinding1);
		   
		  //update Network Device
		    System.out.println("\n\n\n\n\n\n" + "----------- update Sxp Local Binding ----------------------------------------------------------------");
		    System.out.println("\n###############################################################" + 
		             		   "\n#  Update Sxp Local Binding:                                  #" +
		            		   "\n#  This demo sends PUT request to server,                     #" +
		            		   "\n#  for updating a Sxp Local Binding.                          #" +
		            		   "\n#  The java Sxp Local Binding object is marshaled to xml to   #" +
		            		   "\n#  become the request content.                                #" +
		            		   "\n#  The expected response would be status 200                  #" +
		            		   "\n#  with UpdatedFields as the reponse content which will be    #" +
		           	    	   "\n#  Then it marshals to java object of type UpdatedFields, and #" +
		           		       "\n#  print it to console .                                      #" +
		           		       "\n###############################################################");
		   updateSxpLocalBinding(ersClient, sxpLocalBinding1);
		
		   //delete Sxp Local Binding by ID
		   System.out.println("\n\n\n\n\n\n" + "----------- delete Sxp Local Binding by ID -----------------------------------------------------------");
		   System.out.println("\n##########################################################################" + 
		           	   	      "\n#  Delete Sxp Local Binding by ID:                                       #" +
		           		      "\n#  This demo sends DELETE request to delete the Sxp Local Binding by ID  #" +
		           		      "\n#  The expected response would be status 204 (no content)                #" +
		           		      "\n##########################################################################");
		   deleteSxpLocalBindingById(ersClient, sxpLocalBinding1);
		   
			   
		   
			System.out.println("\n\n\n\n\n\n--------------------------------- END ----------------------------------------------------");

		} else {
			System.out.println("\n illegal argument. \n syntax: \n [host-ip] [user-name] [password] [sxp vpn] [SGT Id]\n");
		}
		
		
	}

	/**
	 * connectivity check send getVersion request to server in order to try
	 * connect and make sure the credentials are OK
	 * 
	 * @param ersClient
	 */
	private static void connectivityCheck(ERSClient ersClient) {

		HttpResponse response = null;
		try {

			System.out.println("\n###################################################################"
							+ "\n#  Connectivity Check:                                             #"
							+ "\n#  About to send GET Sxp local bindings version request to server  #"
							+ "\n#  in order to make sure that connection can be established.       #"
							+ "\n####################################################################");

			// getting Sxp Local Binding server version, just for checking that the
			response = ersClient.getVersion("sxp", "sxplocalbindings");
			String responseAsString = RestUtils.getResponseContentAsString(response);
			int responseStatus = RestUtils.getResponseStatus(response);
			if (responseStatus == HttpServletResponse.SC_OK) {
				System.out.println("\nConnection to ISE server success");
			} else if (responseStatus == HttpServletResponse.SC_UNAUTHORIZED) {
				System.out.println("\n###############################################################"
								+ "\n#  401 - UNAUTHORIZED:                                         #"
								+ "\n#  Connection to ISE server failed !!!                         #"
								+ "\n#  Make sure you are using  valid Admin Credentials.           #"
								+ "\n#  Please note that the admin must have 'ERS-Admin' privilages #"
								+ "\n################################################################");
				System.exit(-1);
			} else {
				System.out.println("\nRESPONSE CONTENT: \n" + "	"
						+ responseAsString);
			}
		} catch (Exception e) {
			e.printStackTrace();
		} finally {
			// release the connection
			if (response != null) {
				try {
					if (response.getEntity() != null) {
						response.getEntity().consumeContent();
					}
				} catch (IOException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
			}
		}
	}

	
	public static String createSxpLocalBinding(ERSClient ersClient, String ipAddressOrHost,String sgtId,String sxpVpn) {

		System.out.println("\n\n *** about to create new Sxp local bindings,ipAddressOrHost : "	+ ipAddressOrHost + ", sgtId : " + sgtId +", sxpVpn : " + sxpVpn);

		String newBornId = null;
		HttpResponse response = null;

		try {

			//ErsSxpPeer ersSxpPeer = new ErsSxpPeer();
			ErsSxpLocalBindings ersSxpLocalBindings =  new ErsSxpLocalBindings();

			//ersSxpPeer.setPeerName(name);
			//ersSxpPeer.setPeerIpAddress(IP);
			
			ersSxpLocalBindings.setIpAddressOrHost(ipAddressOrHost);
			ersSxpLocalBindings.setSgt(sgtId);
			ersSxpLocalBindings.setSxpVpn(sxpVpn);

			JAXBElement<ErsSxpLocalBindings> jaxbSxpLocalBindings = localBindingFactory.createSxplocalbindings(ersSxpLocalBindings);

			String sxpLocalBindingsintAsXml = jaxbh.marshal(jaxbSxpLocalBindings);

			System.out.println("Request Xml...");
			System.out.println(sxpLocalBindingsintAsXml);
			System.out.println("");
			response = ersClient.create("sxp", "sxplocalbindings", sxpLocalBindingsintAsXml);

			RestUtils.printResponseHeaders(response);

			if (RestUtils.getResponseStatus(response) == HttpServletResponse.SC_CREATED) {

				newBornId = RestUtils.getNewBornId(response);

			} else {
				// error -> print response content

				System.out.println("\nRESPONSE CONTENT: \n" + " "+ RestUtils.getResponseContentAsString(response));

			}

		} catch (Exception e) {
			e.printStackTrace();
		} finally {
			// release the connection
			if (response != null) {
				try {
					if (response.getEntity() != null) {
						response.getEntity().consumeContent();
					}
				} catch (IOException e) {
					e.printStackTrace();
				}
			}

		}

		return newBornId;
	}

	private static List<BaseResource> getAllSxpLocalBindings(ERSClient ersClient, boolean verboseOn, String queryStringParams) {

		HttpResponse response = null;

		List<BaseResource> baseResources = null;  

		try {

			response = ersClient.getAll("sxp","sxplocalbindings", queryStringParams);


			if(verboseOn){
				RestUtils.printResponseHeaders(response);
			}

			String searchResultAsString = RestUtils.getResponseContentAsString(response);

			if (RestUtils.getResponseStatus(response) == HttpServletResponse.SC_OK) {
				
				
				
				if(verboseOn){
					System.out.println("\nRESPONSE CONTENT: \n" + "	" +  searchResultAsString);
				}


				//unmarshal the result from xml to java object
				SearchResult sr = (SearchResult)RestUtils.fromXML(searchResultAsString, SearchResult.class);

				System.out.println("\n Total Sxp Local Bindings found: " + sr.getTotal());

				baseResources =  sr.getResources().getResource();

				Iterator<BaseResource> it = baseResources.iterator();
				while (it.hasNext()) {
					BaseResource br = (BaseResource) it.next();
					if(verboseOn){
						System.out.println("\n	Sxp Local Bindings details: \n Id =" + br.getId() + " \n name=" + br.getName() +
								" \n description=" + br.getDescription() + " \n link=" + br.getLink());
					}

				}


			}else{
				System.out.println("\nRESPONSE CONTENT: \n" + "	" +  searchResultAsString);
			}

		} catch (Exception e) {
			e.printStackTrace();
		} finally{
			//release the connection
			if(response != null){
				try {
					if (response.getEntity() != null) {
						response.getEntity().consumeContent();
					}
				} catch (IOException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
			}
		}

		return baseResources;
	}
	
	private static ErsSxpLocalBindings getSxpLocalBindingByID(ERSClient ersClient, String sxpLocalBindingId) {

		HttpResponse response = null;
		//ErsSxpPeer ersSxpPeer = null;
		ErsSxpLocalBindings ersSxpLocalBindings = null; 

		try {
			System.out.println("Get Sxp Local Binding By Id : " + sxpLocalBindingId);
			response = ersClient.getById("sxp","sxplocalbindings",sxpLocalBindingId);
			RestUtils.printResponseHeaders(response);
			String responseAsString = RestUtils.getResponseContentAsString(response);
			System.out.println("\nRESPONSE CONTENT: \n" +  "	" + responseAsString);



			if (RestUtils.getResponseStatus(response) == HttpServletResponse.SC_OK) {

				//unmarshal the result from xml to java object
				ersSxpLocalBindings = (ErsSxpLocalBindings)RestUtils.fromXML(responseAsString, ErsSxpLocalBindings.class);

				if (ersSxpLocalBindings!=null){
					System.out.println("\n ersSxpLocalBindings " + ersSxpLocalBindings.getIpAddressOrHost() + " fetched!");
					System.out.println("\n ersSxpLocalBindings ID: " + ersSxpLocalBindings.getId()+ " , sgt Id : " + ersSxpLocalBindings.getSgt()+ " , vpn : " + ersSxpLocalBindings.getSxpVpn());
				}

			}

		} catch (Exception e) {
			e.printStackTrace();
		} finally{
			//release the connection
			if(response != null){
				try {
					response.getEntity().consumeContent();
				} catch (IOException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
			}
		}

		return ersSxpLocalBindings;
	}
	
	private static void updateSxpLocalBinding(ERSClient ersClient, String sxpLocalBindingId) {

		HttpResponse response = null;
		//ErsSxpPeer ersSxpPeer = new ErsSxpPeer();
		ErsSxpLocalBindings ersSxpLocalBinding = new ErsSxpLocalBindings();
		
		try {
			ersSxpLocalBinding = getSxpLocalBindingByID(ersClient, sxpLocalBindingId);
			if(ersSxpLocalBinding != null){
				
				ersSxpLocalBinding.setIpAddressOrHost("172.16.28.15/20");
				//ersSxpLocalBinding.setSgtValue(20);
				
				JAXBElement<ErsSxpLocalBindings> jaxbInternalUser = localBindingFactory.createSxplocalbindings(ersSxpLocalBinding);

				String userAsXml = jaxbh.marshal(jaxbInternalUser);

				response = ersClient.update("sxp","sxplocalbindings", sxpLocalBindingId, userAsXml);

				RestUtils.printResponseHeaders(response);
				String responseAsString = RestUtils.getResponseContentAsString(response);
				System.out.println("\nRESPONSE CONTENT: \n" + "	" + responseAsString);

				if (RestUtils.getResponseStatus(response) == HttpServletResponse.SC_OK) {


					UpdatedFieldsList updatedFields = (UpdatedFieldsList)RestUtils.fromXML(responseAsString, UpdatedFieldsList.class);
					List<UpdatedField> uflds = updatedFields.getUpdatedField();

					for(UpdatedField uf : uflds){

						System.out.println(uf.getField() + " field was updated: oldVal=" + uf.getOldValue() + " newVal=" + uf.getNewValue() + "\n");
					}
				}
			}

		} catch (Exception e) {
			e.printStackTrace();
		} finally{
			//release the connection
			if(response != null){
				try {
					response.getEntity().consumeContent();
				} catch (IOException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
			}
		}
	}
	
	private static void deleteSxpLocalBindingById(ERSClient ersClient, String sxpLocalBindingId) {
		HttpResponse response = null;

		try {

			response = ersClient.deleteById("sxp","sxplocalbindings",sxpLocalBindingId);
			RestUtils.printResponseHeaders(response);
			String responseAsString = RestUtils.getResponseContentAsString(response);


			System.out.println("\nRESPONSE CONTENT: \n" + "	" +  responseAsString);


			if (RestUtils.getResponseStatus(response) == HttpServletResponse.SC_NO_CONTENT) {

				System.out.println("Sxp Local Binding ID " + sxpLocalBindingId + " deleted" );

			}else{
				System.out.println("failed to delete Sxp Local Binding ID :" + sxpLocalBindingId);
			}

		} catch (Exception e) {
			e.printStackTrace();
		} finally{
			//release the connection
			if(response.getEntity() != null){
				try {
					response.getEntity().consumeContent();
				} catch (IOException e) {
					// TODO Auto-generated catch block
					//e.printStackTrace();
				}
			}
		}
	}
}
