//**************************************************
//Copyright (c) 2014-2015 Cisco Systems, Inc.
//All rights reserved.
//**************************************************

package com.cisco.ise.ers.demo;

import java.io.IOException;
import java.util.Iterator;
import java.util.List;

import javax.servlet.http.HttpServletResponse;
import javax.xml.bind.JAXBElement;

import org.apache.http.HttpResponse;

import com.cisco.ise.ers.BaseResource;
import com.cisco.ise.ers.v2.SearchResult;
import com.cisco.ise.ers.demo.ERSClient;
import com.cisco.ise.ers.demo.JAXBHelper;
import com.cisco.ise.ers.demo.RestUtils;
import com.cisco.ise.ers.sxp.ErsSxpVpn;
import com.cisco.ise.ers.sxp.ObjectFactory;

/**
 * @author kumalaks
 *
 */

public class SxpVpnsDemo {

	static ERSClient ersClient;

	static ObjectFactory vpnFactory = new ObjectFactory();

	static JAXBHelper jaxbh = new JAXBHelper(vpnFactory.getClass().getPackage().getName());

	/**
	 * expect 3 arguments: serverHostIp, Admin username of type ERS Admin, and
	 * its password String serverHost = null; Then it runs
	 * 
	 * @param args
	 * @throws Exception
	 */
	public static void main(String[] args) throws Exception {

		String serverHost = null;
		String userName = null;
		String password = null;

		if (args != null && args.length == 3) {

			serverHost = args[0]; // Server Host
			userName = args[1]; // Administrator name
			password = args[2]; // Administrator password

			System.out.println("\n ISE ip:" + serverHost + " user:" + userName + " psw:" + password);

			// Get an instance of Rest Client
			ERSClient ersClient = new ERSClient(userName, password, serverHost);

			System.out.println("\n\n\n\n\n\n--------------------------------- START --------------------------------------------------");

			// Connectivity Check
			connectivityCheck(ersClient);

			// create Sxp Node
			System.out.println("\n\n\n\n\n\n----------- create Sxp Vpns ---------------------------------------------------------------");
			System.out.println("\n###############################################################"
							+ "\n#  Create Sxp Vpns:                                            #"
							+ "\n#  This demo sends POST request to create new Sxp Vpn          #"
							+ "\n#  The expected response would be status 201 whereas the       #"
							+ "\n#  LOCATION header will hold the newborn Sxp Vpn ID.           #"
							+ "\n################################################################");
			ErsSxpVpn ersSxpVpn = new ErsSxpVpn();
			ersSxpVpn.setSxpVpnName("Sxp-Vpn-Demo1");
			
			
			String sxpVpn1 = createSxpVpn(ersClient, ersSxpVpn);
			
			// Get all SxpNodes
			System.out.println("\n\n\n\n\n\n" + "----------- Get all Sxp Vpns --------------------------------------------------------------");
			System.out.println("\n###############################################################" + 
			           		   "\n#  Get All Sxp Vpns:                                         #" +
					 		   "\n#  This demo sends GET request to server to get all(paged)    #" +
			         		   "\n#  Sxp Vpns. The expected response would be status 200       #" +				           		   
						       "\n#  first page of the Sxp Vpns. The demo will also            #" +
			         		   "\n#  marshal the xml response content to java object of type    #" +
		            		   "\n#  search result and will print its attributes.               #" +
			           		   "\n###############################################################");
				
		    getAllSxpVpns(ersClient, true, "?page=1&size=20");
					    
		    //get specify Sxp Node by ID
		    System.out.println("\n\n\n\n\n\n" + "----------- get Sxp Vpn by ID -------------------------------------------------------------");
		    System.out.println("\n###############################################################" + 
		                       "\n#  Get Sxp Vpn by ID:                                         #" +
		            		   "\n#  This demo sends GET request to fetch Sxp Vpn ID.           #" +
		            		   "\n#  The expected response would be status 200 whereas the      #" +
		           		       "\n#  response content will hold the requested Sxp Vpn.          #" +
		           		       "\n#  The demo will also unmarshal the xml response content to   #" +
		           		       "\n#  java object of type Sxp Vpn and will print                 #" +
		           		       "\n#  its attributes.                                            #" +
		           		       "\n###############################################################");
		    getSxpVpnByID(ersClient, sxpVpn1);
		   
		   	
		   //delete SxpVpn by ID
		   System.out.println("\n\n\n\n\n\n" + "----------- delete Sxp Vpn by ID -----------------------------------------------------------");
		   System.out.println("\n#################################################################" + 
		           	   	      "\n#  Delete Sxp Vpn by ID:                                       #" +
		           		      "\n#  This demo sends DELETE request to delete the Sxp Vpn by ID  #" +
		           		      "\n#  The expected response would be status 204 (no content)       #" +
		           		      "\n#################################################################");
		   deleteSxpVpnById(ersClient, sxpVpn1);
		   
		   System.out.println("\n\n\n\n\n\n--------------------------------- END ----------------------------------------------------");

		} else {
			System.out.println("\n illegal argument. \n syntax: [host-ip] [user-name] [password]\n");
		}
		
		
	}

	/**
	 * connectivity check send getVersion request to server in order to try
	 * connect and make sure the credentials are OK
	 * 
	 * @param ersClient
	 */
	private static void connectivityCheck(ERSClient ersClient) {

		HttpResponse response = null;
		try {

			System.out.println("\n###############################################################"
							+ "\n#  Connectivity Check:                                        #"
							+ "\n#  About to send GET Sxp Peer version request to server       #"
							+ "\n#  in order to make sure that connection can be established.  #"
							+ "\n###############################################################");

			// getting SxpPeer server version, just for checking that the
			response = ersClient.getVersion("sxp", "sxpvpns");
			String responseAsString = RestUtils.getResponseContentAsString(response);
			int responseStatus = RestUtils.getResponseStatus(response);
			if (responseStatus == HttpServletResponse.SC_OK) {
				System.out.println("\nConnection to ISE server success");
			} else if (responseStatus == HttpServletResponse.SC_UNAUTHORIZED) {
				System.out.println("\n###############################################################"
								+ "\n#  401 - UNAUTHORIZED:                                        #"
								+ "\n#  Connection to ISE server failed !!!                        #"
								+ "\n#  Make sure you are using  valid Admin Credentials.          #"
								+ "\n#  Please note that the admin must have 'ERS-Admin' privilages#"
								+ "\n###############################################################");
				System.exit(-1);
			} else {
				System.out.println("\nRESPONSE CONTENT: \n" + "	"
						+ responseAsString);
			}
		} catch (Exception e) {
			e.printStackTrace();
		} finally {
			// release the connection
			if (response != null) {
				try {
					if (response.getEntity() != null) {
						response.getEntity().consumeContent();
					}
				} catch (IOException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
			}
		}
	}

	
	public static String createSxpVpn(ERSClient ersClient, ErsSxpVpn ersSxpVpn) {

		System.out.println("\n\n *** about to create new Sxp Vpn...");

		String newBornId = null;
		HttpResponse response = null;

		try {

			JAXBElement<ErsSxpVpn> jaxbSxpVpn = vpnFactory.createErsSxpVpn(ersSxpVpn);

			String sxpVpnintAsXml = jaxbh.marshal(jaxbSxpVpn);

			System.out.println("Request Xml...");
			System.out.println(sxpVpnintAsXml);
			System.out.println("");
			response = ersClient.create("sxp", "sxpvpns", sxpVpnintAsXml);

			RestUtils.printResponseHeaders(response);

			if (RestUtils.getResponseStatus(response) == HttpServletResponse.SC_CREATED) {

				newBornId = RestUtils.getNewBornId(response);

			} else {
				// error -> print response content

				System.out.println("\nRESPONSE CONTENT: \n" + " "+ RestUtils.getResponseContentAsString(response));

			}

		} catch (Exception e) {
			e.printStackTrace();
		} finally {
			// release the connection
			if (response != null) {
				try {
					if (response.getEntity() != null) {
						response.getEntity().consumeContent();
					}
				} catch (IOException e) {
					e.printStackTrace();
				}
			}

		}

		return newBornId;
	}

	 
	private static List<BaseResource> getAllSxpVpns(ERSClient ersClient, boolean verboseOn, String queryStringParams) {

			HttpResponse response = null;

			List<BaseResource> baseResources = null;  

			try {

				response = ersClient.getAll("sxp","sxpvpns", queryStringParams);


				if(verboseOn){
					RestUtils.printResponseHeaders(response);
				}

				String searchResultAsString = RestUtils.getResponseContentAsString(response);

				if (RestUtils.getResponseStatus(response) == HttpServletResponse.SC_OK) {
					
					
					
					if(verboseOn){
						System.out.println("\nRESPONSE CONTENT: \n" + "	" +  searchResultAsString);
					}


					//unmarshal the result from xml to java object
					SearchResult sr = (SearchResult)RestUtils.fromXML(searchResultAsString, SearchResult.class);

					System.out.println("\n Total Sxp Vpns found: " + sr.getTotal());

					baseResources =  sr.getResources().getResource();

					Iterator<BaseResource> it = baseResources.iterator();
					while (it.hasNext()) {
						BaseResource br = (BaseResource) it.next();
						if(verboseOn){
							System.out.println("\n	Sxp Vpns details: \n Id =" + br.getId() + " \n name=" + br.getName() +
									" \n description=" + br.getDescription() + " \n link=" + br.getLink());
						}

					}


				}else{
					System.out.println("\nRESPONSE CONTENT: \n" + "	" +  searchResultAsString);
				}

			} catch (Exception e) {
				e.printStackTrace();
			} finally{
				//release the connection
				if(response != null){
					try {
						if (response.getEntity() != null) {
							response.getEntity().consumeContent();
						}
					} catch (IOException e) {
						// TODO Auto-generated catch block
						e.printStackTrace();
					}
				}
			}

			return baseResources;
		}
		
		
		private static ErsSxpVpn getSxpVpnByID(ERSClient ersClient, String sxpVpnId) {

			HttpResponse response = null;
			ErsSxpVpn ersSxpVpn = null;

			try {
				System.out.println("Get Sxp Vpn By Id : " + sxpVpnId);
				response = ersClient.getById("sxp","sxpvpns",sxpVpnId);
				RestUtils.printResponseHeaders(response);
				String responseAsString = RestUtils.getResponseContentAsString(response);
				System.out.println("\nRESPONSE CONTENT: \n" +  "	" + responseAsString);



				if (RestUtils.getResponseStatus(response) == HttpServletResponse.SC_OK) {

					//unmarshal the result from xml to java object
					ersSxpVpn = (ErsSxpVpn)RestUtils.fromXML(responseAsString, ErsSxpVpn.class);

					if (ersSxpVpn!=null){
						System.out.println("\n Host Name " + ersSxpVpn.getSxpVpnName() + " fetched!");
					}

				}

			} catch (Exception e) {
				e.printStackTrace();
			} finally{
				//release the connection
				if(response != null){
					try {
						response.getEntity().consumeContent();
					} catch (IOException e) {
						// TODO Auto-generated catch block
						e.printStackTrace();
					}
				}
			}

			return ersSxpVpn;

		}

	private static void deleteSxpVpnById(ERSClient ersClient, String sxpVpnId) {
			HttpResponse response = null;

			try {

				response = ersClient.deleteById("sxp","sxpvpns",sxpVpnId);
				RestUtils.printResponseHeaders(response);
				String responseAsString = RestUtils.getResponseContentAsString(response);


				System.out.println("\nRESPONSE CONTENT: \n" + "	" +  responseAsString);


				if (RestUtils.getResponseStatus(response) == HttpServletResponse.SC_NO_CONTENT) {

					System.out.println("Sxp Vpn ID " + sxpVpnId + " deleted" );

				}else{
					System.out.println("failed to delete SxpVpn ID :" + sxpVpnId);
				}

			} catch (Exception e) {
				e.printStackTrace();
			} finally{
				//release the connection
				if(response.getEntity() != null){
					try {
						response.getEntity().consumeContent();
					} catch (IOException e) {
						// TODO Auto-generated catch block
						//e.printStackTrace();
					}
				}
			}
		}
		
}
